<?php
 
namespace App\Console\Commands;

use Illuminate\Console\Command;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\VenueAdditionalInfo;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\CategoryController;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
use App\EventPerformers;
use DB;
class FetchTixstockData extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
     
    private $categoriesArray = [];
    protected $signature = 'fetch:tixstockdata {page}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $page=$this->arguments()['page'];
        $this->processData($page);
    }
    
    public function oldprocessData($page){
        
        $sources=Sources::where('name','Tixstock')->first();
        $url= env('TIXSTOCK_URL')."/v1/events/get?mapped_status=true&per_page=50&page=".$page;
        
        $response=$this->fetchData($url,$sources->token);
        
        Log::info("page" .$page." fetched");
        
        $sourceCategoryMapping = new CategoryController();
        $this->categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
        $categoryObj = new Categories();
                
        if(!empty($response['data'])){
            
            $start_time = microtime(true);
            
            foreach($response['data'] as $events)
            {
                //check venue details 
                $venueArray = [];
                
                if(isset($events['venue']['name']) && !empty($events['venue']['name'])){
                        //$checkVenue = Venues::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $venue['name']);
                        $checkVenue = Venues::where("name","like","%".$events['venue']['name']."%");
                }
                
                if(isset($events['venue']['city']) && !empty($events['venue']['city'])){
                       $checkVenue = $checkVenue->where("city","like","%".$events['venue']['city']."%"); 
                }
                
                if(isset($events['venue']['state']) && !empty($events['venue']['state'])){
                       $checkVenue = $checkVenue->where("state","like","%".$events['venue']['state']."%"); 
                }
                
                if(isset($events['venue']['country_code']) && !empty($events['venue']['country_code'])){
                       $checkVenue = $checkVenue->where("country", $events['venue']['country_code']); 
                }
                
                $checkVenue = $checkVenue->first();
                
                $venueArray = [
                        'name'=>isset($events['venue']['name']) && !empty($events['venue']['name']) ? $events['venue']['name'] : NULL,
                        'city'=>isset($events['venue']['city']) && !empty($events['venue']['city']) ? $events['venue']['city'] : "",
                        'state'=>isset($events['venue']['state']) && !empty($events['venue']['state']) ? $events['venue']['state'] : NULL,
                        'country'=>isset($events['venue']['country_code']) && !empty($events['venue']['country_code']) ? $events['venue']['country_code'] : NULL,
                    ];
                $venueId = "";
                
                //check venue id exist or not
                if(!empty($events['venue']['id'])){
                    $checkVenueId = Venues::where('tixstock_id',$events['venue']['id'])->where('source_id',$sources->id)->first();
                    if($checkVenueId){
                        $venueId = $checkVenueId->id;
                    }
                }
                
                if(empty($venueId)){
                    if(!$checkVenue){
                        $venueArray['tixstock_id'] = $events['venue']['id'];
                        $venueArray['source_id'] = $sources->id;
                        $venue=Venues::create($venueArray);
                        $venueId = $venue->id;
                    }else{
                        $venueId = $checkVenue->id;
                    }
                }
                
                $venue_address_id = null;
                $venueAddress = [
                                    'venue_id'=>$venueId,
                                    'address_line_1'=>isset($events['venue']['address_line_1']) && !empty($events['venue']['address_line_1']) ? $events['venue']['address_line_1'] : NULL,
                                    'address_line_2'=>isset($events['venue']['address_line_2']) && !empty($events['venue']['address_line_2']) ? $events['venue']['address_line_2'] : NULL,
                                    'post_code'=>isset($events['venue']['postcode']) && !empty($events['venue']['state']) ? str_replace(' ','',$events['venue']['postcode']) : NULL,
                                    'source_id'=>$sources->id
                                ];
                                  
                $checkVenueAddress= new VenueAdditionalInfo();
                if(isset($events['venue']['address_line_1']) && !empty($events['venue']['address_line_1'])){
                     $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $events['venue']['address_line_1']);
                }
                
                if(isset($events['venue']['address_line_2']) && !empty($events['venue']['address_line_2']) && $events['venue']['address_line_2'] != NULL){
                     $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_2) AGAINST(? IN BOOLEAN MODE)", $events['venue']['address_line_2']);
                }
                
                $checkVenueAddress= $checkVenueAddress->first();
                
                if(!$checkVenueAddress){
                        $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                        if(!$checkwithParam){
                        $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                        $venue_address_id = $venueadditionalinfo->id;
                        }else{ 
                            $venue_address_id = $checkwithParam->id;
                        }
                    
                }else{
                    if($checkVenueAddress->venue_id == $venueId){
                        $checkVenueAddress->address_line_1 = isset($events['venue']['address_line_1']) && !empty($events['venue']['address_line_1']) ? $events['venue']['address_line_1'] : $checkVenueAddress->address_line_1;
                        $checkVenueAddress->address_line_2 = isset($events['venue']['address_line_2']) && !empty($events['venue']['address_line_2']) ? $events['venue']['address_line_2'] : $checkVenueAddress->address_line_2;
                        $checkVenueAddress->post_code = isset($events['venue']['postcode']) && !empty($events['venue']['postcode']) ? str_replace(' ','',$events['venue']['postcode']) : $checkVenueAddress->post_code;
                        $checkVenueAddress->save();
                        $venue_address_id = $checkVenueAddress->id;
                    }else{
                        // if(!empty($venueAddress['post_code']) && $venueAddress['post_code'] != NULL){
                        //     if($checkVenueAddress->post_code != $venueAddress['post_code'])
                        //     {
                                $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                if(!$checkwithParam){
                                    $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                    $venue_address_id = $venueadditionalinfo->id;
                                }
                            // }else{
                            //     $venue_address_id = $checkVenueAddress->id;
                            // }
                        // }
                    }
                }
                
                
                //check venue section and category
                $eventcategoryArray= [];
                $eventsectionArray=[];
                if(!empty($events['venue_details'])){
                    foreach($events['venue_details'] as $venueDetail){
                        
                        //check venue category
                            $checkCategory = VenueCategory::where('name','like','%'.trim($venueDetail['name']).'%')
                                                        ->where('venue_id',$venueId)
                                                        // ->where('source_id',$sources->id)
                                                        ->first();
                            if(!$checkCategory){
                                $categoryArray=[
                                                'name'=>$venueDetail['name'],
                                                'venue_id'=>$venueId,
                                                'venue_category'=>$venueDetail['name'],
                                                'external_id'=>$venueDetail['id'],
                                                'source_id'=>$sources->id,
                                            ];
                                
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                                $categoryArray =[];
                                $eventcategoryArray[] = $categoryId;
                            }else{
                                $categoryId = $checkCategory->id;
                                $eventcategoryArray[] = $categoryId;
                            }
                        
                        //check section
                            foreach($venueDetail['sections'] as $section){
                                    $checkSection=VenueSection::where('name','like','%'.$section['name'].'%')
                                                                ->where('category_id',$categoryId)
                                                                ->first();
                                    if(!$checkSection){
                                        $sectionArray = [
                                                            'name'=>$section['name'],
                                                            'category_id'=>$categoryId,
                                                            'source_id'=>$sources->id,
                                                            'external_id'=> isset($section['id']) && !empty($section['id']) ? $section['id'] : '',
                                                        ];
                                        $section = VenueSection::create($sectionArray);
                                        $eventsectionArray[] = $section->id;
                                    }else{
                                        $section_id= $checkSection->id;
                                        $eventsectionArray[] = $section_id;
                                    }
                            }
                    }
                }
                
                //check performers 
                $count=1;
                $performer_1_id=null;
                $performer_2_id=null;
                $performers = [];
                foreach($events['performers'] as $performer){
                    
                        $checkPerformer = Performers::where('name','like','%'.trim($performer['name'].'%'));
                        // if(!empty($performer['id'])){
                        //     $checkPerformer=$checkPerformer->where('external_id',$performer['id']);
                        // }
                                                   
                        $checkPerformer=$checkPerformer->first();
                        if(!$checkPerformer){
                            //check performer id 
                            
                            $performerArray=[
                                                'name'=>trim($performer['name']),
                                                'name_variation'=>trim($performer['name']),
                                                'external_id'=>$performer['id'],
                                                'source_id'=>$sources->id,
                                                
                            ];
                            
                            $performer = Performers::create($performerArray);
                            if($count==1){
                                $performer_1_id = $performer->id;
                                $performers[] = $performer->id;
                            }
                            else{
                                $performer_2_id= $performer->id;
                                $performers[] = $performer->id;
                            }
                            
                        }else{
                            if(isset($performer['id']) && !empty($performer['id'])){
                                if($checkPerformer->external_id != $performer['id']){
                                    $checkPerformer->external_id = $performer['id'];
                                    $checkPerformer->save();
                                }
                            }
                            if($count==1){
                                $performer_1_id=$checkPerformer->id;
                                $performers[] = $checkPerformer->id;
                            }else{
                                $performer_2_id = $checkPerformer->id;
                                $performers[] = $checkPerformer->id;
                            }
                        }
                    
                    $count++;
                }
                
                //check for mapping of category in tree
            
                $events['category']['name'] = isset($events['category']['name']) && !empty($events['category']['name']) ? $events['category']['name'] : "No Category";
                
                if (false !== $key = array_search($events['category']['name'], $this->categoriesArray)) {
                    //add category source mapping 
                    $mappingArray['our_category_id'] = $key;
                    $mappingArray['source_category_name']=$events['category']['name'];
                    $mappingArray['source_id']= 1;
                    $check = SourceCategoryMapping::where($mappingArray)->first();
                    if(!$check){
                        SourceCategoryMapping::create($mappingArray);
                    }
                    $primaryCategory = $events['category']['name'];
                }else{
                    //check in category table
                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$events['category']['name'])->where('source_id',1)->first();
                    if($check){
                        $primaryCategory = $check->category->name;
                    }else{
                        $primaryCategory = $events['category']['name'];
                    }
                }
                
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryTree = $categoryObj->getParentCatgoryTree($primaryCategory);
                
                //get maps of venue
                $maps = VenueMaps::with('maps')->where('venue_id',$venueId)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
                
                //check events
                
                $event_name = isset($events['name']) ? $events['name'] : '';
                $event_date = isset($events['datetime']) ? date ('Y-m-d H:i:s',strtotime($events['datetime'])) : '';
                $event_date_utc = isset($events['datetime']) ? $events['datetime'] : '';
                $eventArray = [];
                $eventArray['search_type'] = $primaryCategory; //isset($events['category']['name']) && !empty($events['category']['name']) ? $events['category']['name'] : 'No Category';
                $eventArray['name'] = str_replace("'", "",$event_name);
                
                // $eventArray['performer_1_id'] = $performer_1_id;
                // $eventArray['performer_2_id'] = $performer_2_id;
                
                $eventArray['event_date'] = $event_date;
                $eventArray['event_date_utc'] = $event_date_utc;
                $eventArray['venue_id'] = $venueId;
                $eventArray['source_id']=$sources->id;
                $eventArray['mapped_status']=true;
                $eventArray['status'] = $events['status'];
                $eventArray['use_tixstock_venue_details'] = isset($events['use_tixstock_venue_details']) && !empty($events['use_tixstock_venue_details']) && $events['use_tixstock_venue_details'] == "true" ? 1 : 0 ;
                $eventArray['currency_code']= isset($events['currency']) ? $events['currency'] : '';
                $eventArray['venue_address_id'] = $venue_address_id;
                $eventArray['external_id'] = $events['id'];
                $eventArray['map_id'] = $map_id;
                
                /*  
                if(!empty($events['id'])){
                    $eventArray['id'] = $events['id'];
                }
                */
                
                //check event external id
                
                $event_date = date('Y-m-d',strtotime($events['datetime']));
                $checkEventExternalId = Events::where('external_id',$events['id'])->first();
                
                if(!$checkEventExternalId){
                    $event = Events::create($eventArray);
                    $eventId = $event->id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$eventId,'performer_id'=>$performer]);
                        }
                    }
                }else{
                        $event = Events::where('id',$checkEventExternalId->id)->update($eventArray);
                        $eventId = $checkEventExternalId->id;
                        EventPerformers::where('event_id',$eventId)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$eventId,'performer_id'=>$performer]);
                            }
                        }
                }
                
                if($events['category'] != null){
                    $categoryArray = implode(',',$eventcategoryArray);
                    $sectionArray = implode(',',$eventsectionArray);
                    $checkEventCategory = EventCategory::where('event_id',$eventId)->where('source_id',$sources->id)->first();
                    $eventCatArray=[
                                            'primary_category'=>$primaryCategory,
                                            'event_id'=>$eventId,
                                            'venue_id'=>$venueId,
                                            'source_id'=>$sources->id,
                                            'event_categories'=>$categoryArray,
                                            'event_section'=>$sectionArray
                                        ];
                    if(!$checkEventCategory){
                        EventCategory::create($eventCatArray);
                    }else{
                        EventCategory::where('id',$checkEventCategory->id)->update($eventCatArray);
                    }
                }
            }
            
            $end_time = microtime(true);
            
            // Calculate script execution time
            $execution_time = ($end_time - $start_time);
            echo " Execution time of script = ".$execution_time." sec";
            echo $page."\r\n";
            
            $page++;
            $this->processData($page);
        }else{
            echo "all data received";
            exit;
        }
    }
    
    public function processData($page){
        
        $sources=Sources::where('name','Tixstock')->first();
        $url= env('TIXSTOCK_URL')."/v1/events/get?mapped_status=true&per_page=50&page=".$page;
        $start_time = microtime(true);
        $response=$this->fetchData($url,$sources->token);
        
        Log::info("page" .$page." fetched");
        
        $sourceCategoryMapping = new CategoryController();
        $this->categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
        $categoryObj = new Categories();
        if(!empty($response['data'])){
            
           // Fetch data from the response
            $eventsData = $response['data'];
            
            // Prepare arrays for batch queries
            $venueArray = array_column($eventsData, 'venue');
            
            $venueNames = array_column($venueArray, 'name');
            
            $venueCities = array_column($venueArray, 'city');
           
            $venueStates = array_column($venueArray, 'state');
            
            $venueCountryCodes = array_column($venueArray, 'country_code');
            
            $venueIds = array_column($venueArray, 'id');
        
            $performerNames = array_column($eventsData, 'performers');
            
            // Eager load related data
            $venues = Venues::whereIn('name', $venueNames)
                ->orWhereIn('city', $venueCities)
                ->orWhereIn('state', $venueStates)
                ->orWhereIn('country', $venueCountryCodes)
                ->get()
                ->keyBy('name');
            
            $venueAdditionalInfos = VenueAdditionalInfo::whereIn('address_line_1', array_column($eventsData, 'venue.address_line_1'))
                ->orWhereIn('address_line_2', array_column($eventsData, 'venue.address_line_2'))
                ->get()
                ->keyBy('address_line_1');
            
            $performers = Performers::whereIn('name', array_column($performerNames, 'name'))
                ->get()
                ->keyBy('name');
            
            // Start a transaction to ensure data consistency.
            DB::beginTransaction();
            
            foreach ($eventsData as $event) {
                // Check if 'venue' key is set in the current event
                $eventVenue = $event['venue'] ?? null;
            
                // Check venue details
                $venueArray = [
                    'name' => $eventVenue['name'] ?? null,
                    'city' => $eventVenue['city'] ?? '',
                    'state' => $eventVenue['state'] ?? null,
                    'country' => $eventVenue['country_code'] ?? null,
                ];

                $checkVenue = $venues->get($venueArray['name']);
            
                if (!$checkVenue) {
                    $checkVenue = Venues::where($venueArray)->first();
            
                    if (!$checkVenue) {
                        $venueArray['tixstock_id'] = $event['venue']['id'];
                        $venueArray['source_id'] = $sources->id;
                        $checkVenue = Venues::create($venueArray);
                    }
                }
            
                // Check venue id exists or not
                $venueId = '';
                if (!empty($event['venue']['id'])) {
                    $checkVenueId = Venues::where('tixstock_id', $event['venue']['id'])
                        ->where('source_id', $sources->id)
                        ->first();
            
                    if ($checkVenueId) {
                        $venueId = $checkVenueId->id;
                    }
                }
            
                if (empty($venueId)) {
                    $venueId = $checkVenue->id;
                }
            
                // Check venue address
                $venueAddress = [
                    'venue_id' => $venueId,
                    'address_line_1' => $event['venue']['address_line_1'] ?? null,
                    'address_line_2' => $event['venue']['address_line_2'] ?? null,
                    'post_code' => isset($event['venue']['postcode']) ? str_replace(' ', '', $event['venue']['postcode']) : null,
                    'source_id' => $sources->id,
                ];
            
                $checkVenueAddress = $venueAdditionalInfos->get($venueAddress['address_line_1']);
                
                if (!$checkVenueAddress) {
                   
                    $checkVenueAddress = VenueAdditionalInfo::where($venueAddress)->first();
                    if (!$checkVenueAddress) {
                            $venueAddressInfo = VenueAdditionalInfo::create($venueAddress);
                            $venue_address_id = $venueAddressInfo->id;
                    }else{
                        $venue_address_id = $checkVenueAddress->id;
                    }
                } else {
                    if ($checkVenueAddress->venue_id == $venueId) {
                        $checkVenueAddress->address_line_1 = $venueAddress['address_line_1'];
                        $checkVenueAddress->address_line_2 = $venueAddress['address_line_2'];
                        $checkVenueAddress->post_code = $venueAddress['post_code'];
                        $checkVenueAddress->save();
                        $venue_address_id = $checkVenueAddress->id;
                    } else {
                        $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                        if (!$checkwithParam) {
                            $venueAddressInfo = VenueAdditionalInfo::create($venueAddress);
                            $venue_address_id = $venueAddressInfo->id;
                        }
                    }
                }
            
                // Check venue section and category
                $eventcategoryArray = [];
                $eventsectionArray = [];
                if (!empty($event['venue_details'])) {
                    foreach ($event['venue_details'] as $venueDetail) {
                        // Check venue category
                        $checkCategory = VenueCategory::where('name', 'like', '%' . trim($venueDetail['name']) . '%')
                            ->where('venue_id', $venueId)
                            ->first();
            
                        if (!$checkCategory) {
                            $categoryArray = [
                                'name' => $venueDetail['name'],
                                'venue_id' => $venueId,
                                'venue_category' => $venueDetail['name'],
                                'external_id' => $venueDetail['id'],
                                'source_id' => $sources->id,
                            ];
            
                            $category = VenueCategory::create($categoryArray);
                            $categoryId = $category->id;
                            $eventcategoryArray[] = $categoryId;
                        } else {
                            $categoryId = $checkCategory->id;
                            $eventcategoryArray[] = $categoryId;
                        }
            
                        // Check section
                        foreach ($venueDetail['sections'] as $section) {
                            $checkSection = VenueSection::where('name', 'like', '%' . $section['name'] . '%')
                                ->where('category_id', $categoryId)
                                ->first();
            
                            if (!$checkSection) {
                                $sectionArray = [
                                    'name' => $section['name'],
                                    'category_id' => $categoryId,
                                    'source_id' => $sources->id,
                                    'external_id' => $section['id'] ?? '',
                                ];
                                $section = VenueSection::create($sectionArray);
                                $eventsectionArray[] = $section->id;
                            } else {
                                $section_id = $checkSection->id;
                                $eventsectionArray[] = $section_id;
                            }
                        }
                    }
                }
            
                // Check performers
                $count = 1;
                $performer_1_id = null;
                $performer_2_id = null;
                $performersArray = [];
            
                foreach ($event['performers'] as $performer) {
                    $checkPerformer = $performers->get($performer['name']);
            
                    if (!$checkPerformer) {
                        $checkPerformer = Performers::where('name',trim($performer['name']))->first();
            
                        if (!$checkPerformer) {
                            $performerArray = [
                                'name' => trim($performer['name']),
                                'name_variation' => trim($performer['name']),
                                'external_id' => $performer['id'],
                                'source_id' => $sources->id,
                            ];
            
                            $checkPerformer = Performers::create($performerArray);
                        }
                    }
            
                    $performersArray[] = $checkPerformer->id;
            
                    if ($count == 1) {
                        $performer_1_id = $checkPerformer->id;
                    } else {
                        $performer_2_id = $checkPerformer->id;
                    }
            
                    $count++;
                }
            
                // Check for mapping of category in tree
                $primaryCategory = $event['category']['name'] ?? "No Category";
                if (false !== $key = array_search($primaryCategory, $this->categoriesArray)) {
                    // Add category source mapping
                    $mappingArray = [
                        'our_category_id' => $key,
                        'source_category_name' => $primaryCategory,
                        'source_id' => 1,
                    ];
            
                    $check = SourceCategoryMapping::where($mappingArray)->first();
                    if (!$check) {
                        SourceCategoryMapping::create($mappingArray);
                    }
                } else {
                    // Check in category table
                    $check = SourceCategoryMapping::with('category')
                        ->where('source_category_name', $primaryCategory)
                        ->where('source_id', 1)
                        ->first();
            
                    if ($check) {
                        $primaryCategory = $check->category->name;
                    }
                }
            
                // Get maps of venue
                $maps = VenueMaps::with('maps')
                    ->where('venue_id', $venueId)
                    ->get();
            
                $map_id = null;
                if ($maps->count() > 0) {
                    $categoryTree = $categoryObj->getParentCatgoryTree($primaryCategory);
            
                    if (!empty($categoryTree)) {
                        foreach ($maps as $map) {
                            $mapCategory = explode(',', $map->maps->categories);
                            $commonCategory = array_intersect($categoryTree, $mapCategory);
                            if (!empty($commonCategory)) {
                                $map_id = $map->map_id;
                            }
                        }
                    } else {
                        $map_id = $maps[0]->map_id;
                    }
                }
            
                // Check events
                $event_name = $event['name'] ?? '';
                $event_date = isset($event['datetime']) ? date('Y-m-d H:i:s', strtotime($event['datetime'])) : '';
                $event_date_utc = $event['datetime'] ?? '';
                $eventArray = [
                    'search_type' => $primaryCategory,
                    'name' => str_replace("'", "", $event_name),
                    'event_date' => $event_date,
                    'event_date_utc' => $event_date_utc,
                    'venue_id' => $venueId,
                    'source_id' => $sources->id,
                    'mapped_status' => true,
                    'status' => $event['status'],
                    'use_tixstock_venue_details' => isset($event['use_tixstock_venue_details']) && !empty($event['use_tixstock_venue_details']) && $event['use_tixstock_venue_details'] == "true" ? 1 : 0,
                    'currency_code' => $event['currency'] ?? '',
                    'venue_address_id' => $venue_address_id,
                    'external_id' => $event['id'],
                    'map_id' => $map_id,
                ];
            
                // Check event external id
                $checkEventExternalId = Events::where('external_id', $event['id'])->first();
            
                if (!$checkEventExternalId) {
                    $events = Events::create($eventArray);
                    $eventId = $events->id;
            
                    if (!empty($performersArray)) {
                        foreach ($performersArray as $performerId) {
                            EventPerformers::create(['event_id' => $eventId, 'performer_id' => $performerId]);
                        }
                    }
                } else {
                    Events::where('id', $checkEventExternalId->id)->update($eventArray);
                    $eventId = $checkEventExternalId->id;
                    if (!empty($performersArray)) {
                        EventPerformers::where('event_id', $checkEventExternalId->id)->delete();
                        foreach ($performersArray as $performerId) {
                            EventPerformers::create(['event_id' => $checkEventExternalId->id, 'performer_id' => $performerId]);
                        }
                    }
                }
                
                
                if($event['category'] != null){
                    $categoryArray = implode(',',$eventcategoryArray);
                    $sectionArray = implode(',',$eventsectionArray);
                    $checkEventCategory = EventCategory::where('event_id',$eventId)->where('source_id',$sources->id)->first();
                    $eventCatArray=[
                                            'primary_category'=>$primaryCategory,
                                            'event_id'=>$eventId,
                                            'venue_id'=>$venueId,
                                            'source_id'=>$sources->id,
                                            'event_categories'=>$categoryArray,
                                            'event_section'=>$sectionArray
                                        ];
                    if(!$checkEventCategory){
                        EventCategory::create($eventCatArray);
                    }else{
                        EventCategory::where('id',$checkEventCategory->id)->update($eventCatArray);
                    }
                }
                
            }
            
            // Commit the transaction after the loop.
            DB::commit();
            $end_time = microtime(true);
            
            // Calculate script execution time
            $execution_time = ($end_time - $start_time);
            echo "page - ".$page."  Execution time of script = ".$execution_time." sec.";
            echo "\r\n";
            
            $page++;
            $this->processData($page);
        }else{
            echo "all data received";
            exit;
        }
    }
    
    public function fetchData($url,$token)
    {

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "authorization: Bearer ".$token,
            "cache-control: no-cache",
            "content-type: application/x-www-form-urlencoded",
            "postman-token: 2dd383dd-1b1d-af80-96c9-c8ee7195cff0"
        ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return $err;
        } else {
            return json_decode($response,true);
        }
    }

    
    
    
}
