<?php
namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use DB;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\VenueAdditionalInfo;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\CategoryController;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
// use App\EventPerformers;
class FetchTixstockVenues extends Command{
     /**
     * The name and signature of the console command.
     *
     * @var string
     */
     
    private $categoriesArray = [];
    protected $signature = 'fetch:tixstockvenues {page}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $page=$this->arguments()['page'];
        $this->processData($page);
    }
    
    public function processData(){
        
        $events = Events::with('venue')->where('venue_id','!=','')->wherenull('venue_address_id')->select('venue_id')->groupBy('venue_id')->get();
        $venues = DB::select('select id,tixstock_id from venues where id not in (select venue_id from venue_additional_info)');
        
        
        foreach($venues as $venue){
            if(!empty($venue->tixstock_id)){
                $sources=Sources::where('name','Tixstock')->first();
                $url= env('TIXSTOCK_URL')."/v1/venues/get?mapped_status=true&id=".$venue->tixstock_id;
                $response=$this->fetchData($url,$sources->token);
                if(!empty($response['data'])){
                    foreach($response['data'] as $data)
                    {
                        //check venue details 
                        $venue_address_id = null;
                        $venueAddress = [
                                        'venue_id'=>$venue->id,
                                        'address_line_1'=>isset($data['address_line_1']) && !empty($data['address_line_1']) ? $data['address_line_1'] : NULL,
                                        'address_line_2'=>isset($data['address_line_2']) && !empty($data['address_line_2']) ? $data['address_line_2'] : NULL,
                                        'post_code'=>isset($data['postcode']) && !empty($data['state']) ? str_replace(' ','',$data['postcode']) : NULL,
                                        'source_id'=>$sources->id
                                    ];
                                      
                        $checkVenueAddress= new VenueAdditionalInfo();
                        if(isset($data['address_line_1']) && !empty($data['address_line_1'])){
                             $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $data['address_line_1']);
                        }
                    
                        if(isset($data['address_line_2']) && !empty($data['address_line_2']) && $data['address_line_2'] != NULL){
                            $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_2) AGAINST(? IN BOOLEAN MODE)", $data['address_line_2']);
                        }
                        
                    
                        $checkVenueAddress= $checkVenueAddress->first();
                        if(!$checkVenueAddress){
                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                            if(!$checkwithParam){
                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueadditionalinfo->id;
                            }else{ 
                                $venue_address_id = $checkwithParam->id;
                            }  
                        }else{
                            if($checkVenueAddress->venue_id == $venue->id){
                            $checkVenueAddress->address_line_1 = isset($data['address_line_1']) && !empty($data['address_line_1']) ? $data['address_line_1'] : $checkVenueAddress->address_line_1;
                            $checkVenueAddress->address_line_2 = isset($data['address_line_2']) && !empty($data['address_line_2']) ? $data['address_line_2'] : $checkVenueAddress->address_line_2;
                            $checkVenueAddress->post_code = isset($data['postcode']) && !empty($data['postcode']) ? str_replace(' ','',$data['postcode']) : $checkVenueAddress->post_code;
                            $checkVenueAddress->save();
                            $venue_address_id = $checkVenueAddress->id;
                            }else{
                                $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                if(!$checkwithParam){
                                    $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                    $venue_address_id = $venueadditionalinfo->id;
                                }  
                            }
                        }
                    }
                    
                }
                echo $venue->tixstock_id."\n";
            }
        }
    }
    
    public function fetchData($url,$token)
    {

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 0,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "authorization: Bearer ".$token,
            "cache-control: no-cache",
            "content-type: application/x-www-form-urlencoded",
            "postman-token: 2dd383dd-1b1d-af80-96c9-c8ee7195cff0"
        ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return $err;
        } else {
            return json_decode($response,true);
        }
    }
}

?>