<?php

namespace App;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Support\Str;
use Laravel\Scout\Searchable;
class Events extends Model
{
    use SoftDeletes,Searchable;
    
    //
    const COUNTRY_CODES = [
        'GB' => 'United Kingdom',
        'AT' => 'Austria',
        'BE' => 'Belgium',
        'BG' => 'Bulgaria',
        'HR' => 'Croatia',
        'CY' => 'Cyprus',
        'CZ' => 'Czech Republic',
        'DK' => 'Denmark',
        'EE' => 'Estonia',
        'FI' => 'Finland',
        'FR' => 'France',
        'DE' => 'Germany',
        'GR' => 'Greece',
        'HU' => 'Hungary',
        'IE' => 'Ireland',
        'IM' => 'Isle Of Man',
        'IT' => 'Italy',
        'LV' => 'Latvia',
        'LT' => 'Lithuania',
        'LU' => 'Luxembourg',
        'MT' => 'Malta',
        'NL' => 'Netherlands',
        'PL' => 'Poland',
        'PT' => 'Portugal',
        'RO' => 'Romania',
        'SK' => 'Slovakia',
        'SI' => 'Slovenia',
        'ES' => 'Spain',
        'SE' => 'Sweden',
        'CH' => 'Switzerland',
        'TR' => 'Turkey',
        'IS' => 'Iceland',
        'NO' => 'Norway',
        'LI' => 'Liechtenstein',
        'BA' => 'Bosnia and Herzegovina',
        'ME' => 'Montenegro',
        'MD' => 'Moldova',
        'MK'   => 'North Macedonia',
        'AL'   => 'Albania',
        'RS'   => 'Serbia',
        'UA'   => 'Ukraine',
        'AE'   => 'UnitedArab Emirates',
        'US'  => 'United States',
        'BH'=> 'Bahrain'
    ];
    
    protected $fillable = ['id','name','search_type','url','external_id','performer_1_id','performer_2_id','event_date','event_date_utc','venue_id','tixstock_id','tixstock_map_status','is_run','currency_code',
    'mapped_status','source_id','status','use_tixstock_venue_details','update_requires','prev_name','prev_search_type','prev_url','prev_performer_1_id','prev_performer_2_id','prev_event_date','prev_event_date_utc','prev_venue_id',
    'prev_currency_code','parent_event_id','venue_address_id','prev_venue_address_id','current_version','map_id','update_map_flag'];
    
    
    public function toSearchableArray()
    {
        return [
            'id'=>$this->id,
            'name' => $this->name,
            'venue'=>$this->venue->name,
            'event_date'=>strtotime($this->event_date),
            'venue_address'=>$this->venueAddress->name ?? '',
            'external_id'=>$this->external_id,
            'category'=>$this->search_type,
            'performers'=>$this->performers,
            'source'=>$this->source->name ?? "Manual",
            'mapped_status'=>$this->mapped_status,
            'update_requires'=>$this->update_requires,
            'parent_event_id'=>$this->parent_event_id,
            'current_version'=>$this->current_version
        ];
    }
    
    public function mappedSources(){
        return $this->hasMany('App\EventsMappedSources','event_id');
    }
    public function maps(){
        return $this->belongsTo('App\Maps','map_id');
    }
    public function auditevents(){
        return $this->belongsTo('App\AuditEvents','event_id');
    }

    public function eventcategory(){
        return $this->hasOne('App\EventCategory','event_id');
    }
    
    public function performers(){
        return $this->belongsToMany('App\Performers','event_performers','event_id','performer_id');
    }
    
    public function prevperformers(){
        return $this->belongsToMany('App\Performers','event_performers','event_id','prev_performer_id');
    }

    public function performer1(){
        return $this->belongsTo('App\Performers','performer_1_id');
    }

    public function performer2(){
        return $this->belongsTo('App\Performers','performer_2_id');
    }

    public function venue(){
        return $this->belongsTo('App\Venues','venue_id');
    }
    
    public function venueAddress(){
         return $this->belongsTo('App\VenueAdditionalInfo','venue_address_id');
    }
    
    public function tixstocklog(){
        return $this->hasMany('App\TixstockLog','event_id');
    }
    
    public function source(){
        return $this->belongsTo('App\Sources','source_id');
    }
    
    public function prevPerformer1(){
         return $this->belongsTo('App\Performers','prev_performer_1_id');
    }
    
    public function prevPerformer2(){
         return $this->belongsTo('App\Performers','prev_performer_2_id');
    }
    
    public function prevVenue(){
        return $this->belongsTo('App\Venues','prev_venue_id');
    }
    
    public function eventVersion(){
        return $this->hasMany('App\Events','external_id','external_id')->select('id','external_id');
    }
    
    public function versions(){
        return $this->belongsTo('App\Events','id','parent_event_id');
    }
    
    public function latestVersion()
    {
        return $this->belongsTo('App\Events','id', 'parent_event_id')->with(['performers'])->orderByDesc('id');
    }
    
    public function generateUUID(){
        
        $externalId =  (string) str_replace('-','',Str::uuid()->toString());
        $check = Events::where('external_id',$externalId)->first();
        if(!$check){
            return $externalId;
        }else{
            $this->generateUUID();
        }
    }
}
