<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Carbon\Carbon;

use App\Http\Controllers\CategoryController;
use App\Traits\ApiTrait;
use App\Traits\StubhubTrait;

use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\Logs;
use App\VenueAdditionalInfo;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
use App\EventPerformers;
use App\EventLogs;

class StubhubController extends Controller
{
    use ApiTrait,StubhubTrait;
    //
    const STUBHUB_KEY = "e1JCPLipepG4ZijF2Gs2tCGot7kYLYQy";
    const STUBHUB_SECRET_KEY = "apnWAuRUSAxkV6GG";
    
    const URL_GET_VENUE = '/partners/search/venues/v3/';
    const URL_GET_TOKEN = "/sellers/oauth/accesstoken?grant_type=client_credentials";
    const URL_GET_EVENT = "/sellers/search/bulk/events/v3";
    const URL_GET_SINGLE_EVENT ="/sellers/search/events/v3";
    
    public $stubhubToken = "";
    
    public function index($slug=null){
        
        $resources = Sources::get();
        foreach($resources as $resource){
            switch($resource->name){
                case 'Stubhub':
                    $this->fetchStubhubData($resource,$slug);
                    break;
                default:
                    break;
            }
        }
    }
    
    /**
     * This function is used to create new token for accessing stubhub api
     * @array return token data
     */
    static function createToken(){
        $key=self::STUBHUB_KEY;
        $secret=self::STUBHUB_SECRET_KEY;
        
        $url = env('STUBHUB_API_URL').'/'.self::URL_GET_TOKEN;
        $curl = curl_init();
        $curl_headers = array(
                      'content-type: application/json',
                      'Authorization: Basic '. base64_encode("$key:$secret")
                     );  
       
        curl_setopt($curl, CURLOPT_HTTPHEADER, $curl_headers);
        curl_setopt($curl, CURLOPT_HEADER, false );
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING , 'gzip, deflate');
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST , false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER , false);
        curl_setopt($curl, CURLOPT_TIMEOUT , 500 );
        curl_setopt($curl, CURLINFO_HEADER_OUT, false);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
        
        
        $login_data = array(
                'username'=>env('STUBHUB_LOGIN'),
                'password'=>env('STUBHUB_PASSWORD')
                );

        $send_login_data = json_encode($login_data);
            
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $send_login_data);
        
        $output = curl_exec($curl);
        curl_close($curl);
        
        $response = json_decode($output,true);
        return $response['access_token'];
        
    }
    
    
    public function getVenue($venue_id){
        
        $url =  env('STUBHUB_API_URL').''.self::URL_GET_VENUE.'?id='.$venue_id;
        $token = $this->createToken();
        $output = $this->fetchData($url,$token);
        return $output;
    }
    
    public function getEvent($event_id){
        $url =  env('STUBHUB_API_URL').''.self::URL_GET_SINGLE_EVENT.'?id='.$event_id;
        $token = $this->createToken();
        $output = $this->fetchData($url,$token);
        return $output;
    }
    
    public function fetchStubhubData($resource,$slug){
       
        if(in_array($slug,['Sports','Music','UpdateData'])){
            $this->processData($resource,$slug);
        }
        
        if(in_array($slug,['Zones','UpdateEvents'])){
            //fetch zones
            if($slug=="Zones"){
                $this->fetchStuhubhubEventsZones($resource,$slug);
            }
            if($slug == "UpdateEvents"){
                $this->UpdateExistingEvents();
            }
        }
    }
    
    public function processData($resource,$slug,$start=null){
        
        date_default_timezone_set('Europe/London');
        $date="";
        $countResults = 0;
        $totalResults = 0;
        $createdEvents=[];
        $updatedEvents=[];
        $day = "";
        $categoriesArray= [];
        
        
        $sourceCategoryMapping = new CategoryController();
        $categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
        if($slug=="UpdateData"){
            $days = 200;
            $date= date('Y-m-d', strtotime(' - '.$days.' days'));
        }
        
        if($date==""){
            $date=date('Y-m-d');
            $start = Events::where('search_type','Music')->where('mapped_status',0)->get()->count();
            Log::info($start." fetched on ".$date);
            $url=env('STUBHUB_URL')."?categoryName=".$slug."&rows=500&start=".$start;
        }else{
            if($start == null){
                $start=0;
            }
            $url=env('STUBHUB_API_URL')."".self::URL_GET_EVENT."?dateLastModified=".$date."&rows=500&start=".$start;
        }
        
        //create stuhub  request to fetch events
        if($this->stubhubToken == ""){
            $this->stubhubToken = self::createToken();
        }
        
        $response=$this->fetchData($url,$this->stubhubToken);
        $totalResults=isset($response['numFound']) && !empty($response['numFound']) ? $response['numFound'] : 0;
        
        $log = Log::channel('cron');
        $log->info( "start: ".$start." of ".$totalResults." event");
        
        if(isset($response['events']) && count($response['events']) >= 1){
            foreach ($response['events'] as $key => $value) {
                $log->info( "Processing event: " .$value['id']);
                $event_category=array();
                $cat_arr=array();
                $event_arr = array();

                $performer_1_id = NULL;
                $performer_2_id = NULL;
                $venue_id = 0;
                $event_id = 0;
                
                
                //check country code if its not in europe skip it
                if(isset(Events::COUNTRY_CODES[$value['venue']['country']]) === false) {
                    echo 'Skipped event invalid country ' . $value['venue']['country'] . "\r\n";
                    $log->info($value['id']." event skipped.");
                    $log->info("=============================================");
                    continue;
                }
                
                
                //get currency code
                $currencyCode="";
                $eventWithCurrency = $this->getEvent($value['id']);
                if(!empty($eventWithCurrency) && !empty($eventWithCurrency['events'][0])){
                    $currencyCode = $eventWithCurrency['events'][0]['currencyCode'];
                }
               
                //performers data
                $performers = [];
                if(isset($value['performers']) && !empty($value['performers'])){
                    foreach($value['performers'] as $pk =>$performer){
                        $performer_arr=array();

                        //performer name
                        $performer_name = isset($performer['name']) ? $performer['name'] : '';
                        $performer_arr['name'] = str_replace("'", "",trim($performer_name));
                        $performer_arr['name_variation'] = str_replace("'", "",$performer_arr['name']);
                        $performer_arr['source_id']=$resource->id;
                        $performer_arr['external_id']=isset($performer['id']) && !empty($performer['id']) ? $performer['id'] : '';
                        
                        $checkPerformer = Performers::where('name',$performer_arr['name'])->first();
                        if(!$checkPerformer){   
                            $performer = Performers::create($performer_arr);
                            $performerId = $performer->id;
                            $performers[] = $performerId;
                            $log->info("Performer Id:".$performerId." created for event ".$value['id']);
                            
                        }
                        else{
                            if(!empty($checkPerformer->external_id) && !empty($perfomrer_arr['external_id']) && $checkPerformer->external_id != $performer_arr['external_id']){
                                $checkPerformer->external_id = $perfomrer_arr['external_id'] ;
                                $checkPerformer->save();
                                $log->info("Performer External id :".$perfomrer_arr['external_id'] ." saved for event ".$value['id']);
                            }
                            $performerId = $checkPerformer->id;
                             $performers[] = $performerId;
                        }
 
                    }
                }

                //venues data
                if(isset($value['venue']) && !empty($value['venue'])){
                   
                    $venue_arr = array();
                    $venue_name = isset($value['venue']['name']) ? str_replace("'","",$value['venue']['name']) : NULL;
                    $city = isset($value['venue']['city']) ? $value['venue']['city'] : NULL;
                    $state = isset($value['venue']['state']) ? $value['venue']['state'] : NULL;
                    $postalCode = isset($value['venue']['postalCode']) ? $value['venue']['postalCode'] : NULL;
                    $country = isset($value['venue']['country']) ? $value['venue']['country'] : NULL;

                    $venue_arr['name'] = !empty($venue_name) ? $venue_name : 'NA';
                    $venue_arr['city'] = !empty($city) ? str_replace("'", "",$city) : "NA";
                    $venue_arr['state'] = !empty($state) ? str_replace("'", "",$state) : "NA";
                    $venue_arr['country'] = !empty($country) ? str_replace("'", "",$country) : "NA";
                    $venue_arr['source_id'] = $resource->id;
                    
                    //check Venue
                    //1->check With Venue Id
                    $checkWithVenueId = Venues::where('tixstock_id',$value['venue']['id'])->first();
                    if(!$checkWithVenueId){
                        //2->check with available data
                        
                            $checkVenue = Venues::where('name','like','%'.$venue_arr['name'].'%');
                            if(!empty($venue_arr['city']) && $venue_arr['city'] != NULL){
                                $checkVenue = $checkVenue->where('city','like','%'.$venue_arr['city'].'%');
                            }
                            // if(!empty($venue_arr['state']) && $venue_arr['state'] != NULL){
                            //     $checkVenue = $checkVenue->where('state','like','%'.$venue_arr['state'].'%');
                            // }
                            if(!empty($venue_arr['country']) && $venue_arr['country'] != NULL){
                                $checkVenue = $checkVenue->where('country','like','%'.$venue_arr['country'].'%');
                            }
                            
                            $checkVenue = $checkVenue->first();
                            if(!$checkVenue){
                                //create venue  
                                $venue_arr['tixstock_id']=$value['venue']['id'];
                                $venue = Venues::create($venue_arr);
                                $venue_id = $venue->id;
                            }else{
                                Venues::where('id',$checkVenue->id)->update($venue_arr);
                                $venue_id = $checkVenue->id;
                            }
                    }else{
                        $venue_id = $checkWithVenueId->id;
                    }
                    
                    //3->check for venue address
                    $venue_address_id = null;
                    $venue_arr['post_code'] = str_replace("'", "",$postalCode);
                    $venue_arr['post_code'] = str_replace(" ","",$postalCode);
                    $venueDetails = $this->getVenue($value['venue']['id']);
                     
                    if(isset($venueDetails['venues'][0]) && !empty($venueDetails['venues'][0])){
                       $venue_arr['address_line_1'] = $venueDetails['venues'][0]['address']['address1'];
                     }
                     
                    $checkVenueAddress= new VenueAdditionalInfo();
                    if(isset($venue_arr['address_line_1']) && !empty($venue_arr['address_line_1'])){
                        $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $venue_arr['address_line_1']);
                        $checkVenueAddress = $checkVenueAddress->first();
                        if(!$checkVenueAddress){
                             //create Venue address
                             $venueAddress = [
                                                'venue_id'=>$venue_id,
                                                'address_line_1'=>$venue_arr['address_line_1'],
                                                'post_code'=>$venue_arr['post_code'],
                                                'source_id'=>$venue_arr['source_id']
                                            ];
                             $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                             if(!$checkwithParam){
                                $venueAddress = VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueAddress->id;
                             }else{
                                 $venue_address_id = $checkwithParam->id;
                             }
                             
                        }else{
                            //$venue_address_id = $checkVenueAddress->id;
                                if($checkVenueAddress->venue_id == $venue_id){
                                    $checkVenueAddress->post_code = !empty($venue_arr['post_code']) && $venue_arr['post_code'] != null ? $venue_arr['post_code'] : $checkVenueAddress->post_code; 
                                    $checkVenueAddress->address_line_1 = $venue_arr['address_line_1'];
                                    $checkVenueAddress->save();
                                    $venue_address_id = $checkVenueAddress->id;
                                }else{
                                    // if(!empty($venue_arr['post_code']) && $venue_arr['post_code'] != null){
                                    //     if(str_replace(' ','',$checkVenueAddress->post_code) != $venue_arr['post_code'])
                                    //     {
                                            $venueAddress=[
                                                        'venue_id'=>$venue_id,
                                                        'address_line_1'=>$venue_arr['address_line_1'],
                                                        'post_code'=>$venue_arr['post_code'],
                                                        'source_id'=>$venue_arr['source_id']
                                                        ];
                                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                            if(!$checkwithParam){
                                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                                $venue_address_id = $venueadditionalinfo->id;
                                            }else{
                                                 $venue_address_id = $checkwithParam->id;
                                            }
                                    //     }else{
                                    //         $venue_address_id = $checkVenueAddress->id;
                                    //     }
                                    // }
                            }
                        }
                    }
                }
                
                //check in category collection
                
                if(isset($eventWithCurrency['events'][0]['categoriesCollection']) && !empty($eventWithCurrency)){ 
                    $value['categoriesCollection'] = $eventWithCurrency['events'][0]['categoriesCollection'];
                }
                
                $primary_category = '';
                
                if(isset($value['categoriesCollection']) && !empty($value['categoriesCollection'])){
                    foreach($value['categoriesCollection']['categories'] as $category){
                        //check in tree
                        if(empty($cat_arr['primary_category'])){
                            if (false !== $key = array_search($category['name'], $categoriesArray)) {
                                //add category source mapping 
                                $primary_category = $category['name'];
                                
                            }else{
                                //check in category table
                                $check = SourceCategoryMapping::with('category')->where('source_category_name',$category['name'])->where('source_id',1)->first();
                                if($check){
                                    $primary_category = $check->category->name;
                                    
                                }
                            }
                        }
                            
                    }
                }
                
                // if(empty($primary_category)){
                    //check in groupings
                    if(isset($value['ancestors']['groupings']) && !empty($value['ancestors']['groupings'])){
                        foreach($value['ancestors']['groupings'] as $gk=>$gvalue){
                            $group_name = isset($gvalue['name']) ? $gvalue['name'] : '';
                            if(empty($cat_arr['primary_category'])){
                                if (false !== $key = array_search($group_name, $categoriesArray)) {
                                    //add category source mapping 
                                    $primary_category = $group_name;
                                
                                }else{
                                    //check in category table
                                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$group_name)->where('source_id',1)->first();
                                    if($check){
                                        $primary_category = $check->category->name;
                                      
                                    }
                                }
                            }
                        }
                    }
                // }
                
                if(empty($primary_category)){
                    if(isset($value['categoriesCollection']) && !empty($value['categoriesCollection'])){
                       $categoryCollection = end($value['categoriesCollection']['categories']);
                       $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                    if(isset($value['ancestors']['groupings']) && !empty($value['ancestors']['groupings'])){
                        $categoryCollection = end($value['ancestors']['groupings']);
                        $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                     $primary_category = 'No Category';
                 }
                 
                 
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryObj = new Categories();
                $categoryTree = $categoryObj->getParentCatgoryTree($primary_category);
                
                //get maps of venue
                
                $maps = VenueMaps::with('maps')->where('venue_id',$venue_id)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
                
                //event data
                $newEvent = 0;
                $new_event_id = '';
                $event_name = isset($value['name']) ? $value['name'] : '';
                $timezone = isset($event['timezone']) ? $event['timezone'] : date_default_timezone_get();
                
                $event_date = "";
                if(isset($value['eventDateLocal']) && !empty($value['eventDateLocal'])){
                    $carbonDate = new Carbon($value['eventDateLocal']);
                    $event_date = $carbonDate->format('Y-m-d H:i:s');
                }
               // $event_date = isset($value['eventDateLocal']) ? date('Y-m-d H:i:s',strtotime($value['eventDateLocal'])) : '';
                $event_date_utc = isset($value['eventDateUTC']) ? $value['eventDateUTC'] : '';  
                
                if(isset($value['ancestors']['categories'][0]['name']) && $value['ancestors']['categories'][0]['name'] == "Sports"){
                    $value['ancestors']['categories'][0]['name']= "Sport";
                }
                
                $event_arr['search_type'] = $primary_category;
                $event_arr['name'] = str_replace("'", "",$event_name);
                $event_arr['event_date'] = $event_date;
                $event_arr['event_date_utc'] = $event_date_utc;
                $event_arr['venue_id'] = $venue_id;
                $event_arr['source_id']=$resource->id;
                $event_arr['status'] = $value['status'];
                $event_arr['venue_address_id']=$venue_address_id;
                $eventArray['map_id'] = $map_id;
                $prev_arr=[];
                $logMsgArray = [];
                //check event external id exist or not 
                $checkExternalId= Events::where('external_id',$value['id'])->orderBy('id','desc')->first();
                
                if(!$checkExternalId){
                    
                    // if event with external id doesn't exist create event
                    $event_arr['external_id'] = isset($value['id']) ? $value['id'] : '';
                    // $checkEvent=Events::where($event_arr)->first();
                  
                   //check event exist or not with any of the resource
                    $event_date_array = explode(' ',$event_arr['event_date']);
                    $event_date = $event_date_array[0];
                    $checkEvent = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", str_replace("-"," ",$event_arr['name']))
                                        ->where('venue_id',$venue_id)
                                        ->whereBetween('event_date',[$event_date." 00:00:00",$event_date." 23:59:59"])
                                        ->first();
                    
                    if(!$checkEvent){
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $event_arr['status']=$value['status'];
                        $event_arr['currency_code']= isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        
                        $event=Events::create($event_arr);
                        
                        $event_id=$event->id;
                        
                        $new_event_id =$event->id;
                        $newEvent = 1;
                        $logMsgArray[] = [
                                           'event_id'=>$event_id,
                                           'message'=>"New Event Created.",
                                           'user_id'=>1,
                                           'date'=>date('Y-m-d h:i:s')
                                        ];
                                       
                        $log->info("created  event with external id : ".$value['id']);
                        $createdEvents[] = $event->external_id;
                    }else{
                         //check source of event(if matched then update event or create new event with resource)
                         $eventUpdateFlag = 0;
                        if($checkEvent->source_id == $resource->id){
                            
                            //if event is from same resource update the event
                            if($checkEvent->mapped_status == 1){
                                $excludeField = ['external_id','search_type','map_id','status','source_id','event_date_utc'];
                                $prevArray = $checkEvent->toArray();
                                $prev_arr=[];
                                foreach($event_arr as $key=>$val){
                                    if(!in_array($key,$excludeField) && !empty($val) && trim($prevArray[$key]) != trim($val)){
                                        $eventUpdateFlag=1;
                                        $prev_arr['prev_'.$key] = $val;
                                        $prev_arr['update_requires'] =1;
                                        //$log->info("Updated event's ".$key." to: ".$value['id']);
                                    }
                                }
                                if($eventUpdateFlag==1){
                                    //update flag of previous event
                                    //$event_arr['update_requires']=1;
                                     Events::where('id',$checkEvent->id)->update($prev_arr);
                                    //create new event 
                                    if($checkEvent->source_id == 0){
                                        //for manual events only.
                                        if($checkEvent->name != $event_arr['name']){
                                            $event_arr['name'] = $checkEvent->name;
                                        }
                                        
                                        if($checkEvent->search_type != $event_arr['search_type']){
                                            $event_arr['search_type'] = $checkEvent->search_type;
                                        }
                                    }
                                    
                                    $event_arr['update_requires'] = 0;
                                    $event_arr['current_version'] = 0;
                                    $event_arr['mapped_status'] = 0;
                                    $event_arr['parent_event_id'] = $checkEvent->id;
                                    $event = Events::create($event_arr);
                                    $event_id = $event->id;
                                    $new_event_id =$event->id;
                                    $newEvent = 1;
                                    $logMsgArray[] = ['event_id'=>$event_id,'message'=>"New Event Created.",'date'=>date('Y-m-d h:i:s')];
                                    $logMsgArray[] = ['event_id'=>$checkEvent->id,'message'=>"New Version from stubhub created.External Id:".$event->external_id,'date'=>date('Y-m-d h:i:s')];
                                }
                            }
                            else{
                                Events::where('id',$checkEvent->id)->update($event_arr);
                                $event_id=$checkEvent->id;
                                //update performers
                                EventPerformers::where('event_id',$event_id)->delete();
                                if(!empty($performers)){
                                    foreach($performers as $performer){
                                        EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                    }
                                }
                            } 
                        }else{
                            
                            //create versioning
                            if($checkEvent->source_id == 0){
                                //for manual events only.
                                if($checkEvent->name != $event_arr['name']){
                                    $event_arr['name'] = $checkEvent->name;
                                }
                                
                                if($checkEvent->search_type != $event_arr['search_type']){
                                    $event_arr['search_type'] = $checkEvent->search_type;
                                }
                                $primary_category = $checkEvent->search_type;
                            }
                            $event_arr['update_requires'] = 0;
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['parent_event_id'] = $checkEvent->id;
                            $event= Events::create($event_arr);
                            $event_id = $event->id;
                            $new_event_id =$event->id;
                            $newEvent = 1;
                            Events::where('id',$checkEvent->id)->update(['update_requires'=>1]);
                            $logMsgArray[] = ['event_id'=>$event_id,'message'=>"New Event Created.",'date'=>date('Y-m-d h:i:s')];
                            $logMsgArray[] = ['event_id'=>$checkEvent->id,'message'=>"New Version from stubhub created.External Id:".$event->external_id,'date'=>date('Y-m-d h:i:s')];
                        }
                    }  
                }else{
                    //check if same event exist or not 
                    $checkEventUpdate=Events::where($event_arr)->first();
                    if(!$checkEventUpdate){
                        //event is updated
                        $event_id = $checkExternalId->id;
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $eventUpdateFlag=0;
                        
                        if($checkExternalId->mapped_status == 1){
                            $excludeField = ['external_id','search_type','map_id','status','source_id','event_date_utc'];
                            $prevArray = $checkExternalId->toArray();
                            $prev_arr = [];
                            foreach($event_arr as $key=>$val){
                                if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] !== $val){
                                    $eventUpdateFlag=1;
                                    $prev_arr['prev_'.$key] = $val;
                                    
                                    $log->info("Updated event's ".$key." to: ".$value['id']);
                                }
                            }
                            if($eventUpdateFlag==1){
                                $prev_arr['update_requires'] =1;
                            }
                        }
                        
                        if($eventUpdateFlag==1){
                            //$event_arr['update_requires']=1;
                            Events::where('id',$checkExternalId->id)->update($prev_arr);
                            if($checkExternalId->source_id == 0){
                                //for manual events only.
                                if($checkExternalId->name != $event_arr['name']){
                                    $event_arr['name'] = $checkExternalId->name;
                                }
                                
                                if($checkExternalId->search_type != $event_arr['search_type']){
                                    $event_arr['search_type'] = $checkExternalId->search_type;
                                }
                            }
                           // Events::where('id',$checkExternalId->id)->update(['current_version'=>0]);
                            //create new event
                            $event_arr['external_id'] = $value['id'] ;
                            $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                            $event_arr['status']=$value['status'];
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                            $event_arr['update_requires']=0;
                            $event_arr['venue_address_id']=$venue_address_id;
                            $event_arr['parent_event_id']=$checkExternalId->id;
                            $event = Events::create($event_arr);
                            $event_id = $event->id;
                            $new_event_id =$event->id;
                            $newEvent = 1;
                            
                            $logMsgArray[] = ['event_id'=>$event_id,'message'=>"New Event Created.",'date'=>date('Y-m-d h:i:s')];
                            $logMsgArray[] = ['event_id'=>$checkExternalId->id,'message'=>"New Version from stubhub created.External Id:".$event->external_id,'date'=>date('Y-m-d h:i:s')];
                            
                            $log->info('New version of Event created for :'.$value['id']);
                            $updatedEvents[] = $checkExternalId->external_id;
                            $log->info("Event ".$value['id']." updated ");
                        }else{
                             $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                             Events::where('id',$checkExternalId->id)->update($event_arr);
                             $event_id = $checkExternalId->id;
                             EventPerformers::where('event_id',$event_id)->delete();
                             if(!empty($performers)){
                                foreach($performers as $performer){
                                    EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                }
                             }
                        }
                        
                    }else{
                        $checkEventUpdate->currency_code = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        $checkEventUpdate->save();
                        $event_id = $checkEventUpdate->id;
                        EventPerformers::where('event_id',$event_id)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                            }
                        }
                    }
                   
                }
                
                
                //event category data

                if(!empty($new_event_id)){
                    $event_id = $new_event_id;
                    $cat_arr['event_id'] = $new_event_id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                        }
                    }else{
                        $tbcperformer = Performers::where('name','TBC Performer')->first();
                        EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$tbcperformer->id]);
                    }
                }
                
                $checkEventCategory=EventCategory::where('event_id',$event_id)->first();
                if(!$checkEventCategory){
                    $cat_arr['primary_category'] = $primary_category;
                    $cat_arr['event_id'] = $event_id;
                    $cat_arr['venue_id']=$venue_id;
                    $cat_arr['venue_address_id']=$venue_address_id;
                    $cat_arr['source_id']=$resource->id;
                    EventCategory::create($cat_arr);
                    
                    $logMsgArray[] = ['event_id'=>$event_id,'message'=>"Event Category created. Category: ".$primary_category,'date'=>date('Y-m-d h:i:s')];
                    $log->info("Event category created for event :".$value['id']);
                }
                if(isset($logMsgArray) && !empty($logMsgArray)){
                    foreach($logMsgArray as $logMsg){
                        EventLogs::create($logMsg);
                    }
                }
                
                if($newEvent == 1){
                    $this->fetchStuhubhubEventsZones($resource,$event->external_id);
                    
                }
                
                $log->info("End event processing of event :".$value['id']);
                $log->info("=============================================");
            }
        }else{
            $logData=['type'=>"cron",'domain'=>"Stubhub:last ".$days." days data","message"=>"No events found."];
            Logs::create($logData);
            Log::info("No events found.");
            exit;
        }

        $countResults+=count($response['events']);
        $countResults=$start+$countResults;

        if($countResults != $totalResults){
            $this->processData($resource,$slug,$countResults);
        }
        
        $logMessage='';
        if(!empty($createdEvents)){
            $logMessage.='('.implode(',',$createdEvents).") events Created.";
        }
        if(!empty($updatedEvents)){
            $logMessage.='('.implode(',',$updatedEvents).") events Updated";
        }
        Log::info("last 10 day's data updated");
        if(!empty($logMessage)){
            $logData=['type'=>'cron','domain'=>"Stubhub:last ".$days." days data","message"=>$logMessage];
            Logs::create($logData);
        }

    }
    
    public function fetchStuhubhubEventsZones($resource,$external_id){

        $event=Events::where('is_run',0)->where('external_id',$external_id)->orderBy('id','desc')->first();
       
         $log=Log::channel('cron');
        $log->info("Fetching ".$external_id." Event's Zone thorugh stubhub Api");
        $eventsUpdated = [];
        if($event){
            // foreach($events as $event){
                $log->info('Processing event '.$event->external_id);
                $event_id=$event->external_id;
                $id=$event->id;
                $venue_id=$event->venue_id;
                
                $url = "https://api.stubcloudprod.com/partners/catalog/events/v3/".$event_id."/sectionZones";
                if(empty($this->stubhubToken)){ 
                    $stubhubToken = $this->createToken();
                }
                $response=$this->fetchData($url,$this->stubhubToken);
                
                if(!empty($response)){
                    if(count($response['zones']) >= 1){
                        $venue_category = 0;
                        $send_category_arr = array();
                        $send_section_arr = array();
                        $getCategory = EventCategory::where('event_id',$id)->where('venue_id',$venue_id)->where('source_id',$resource->id)->first();
    
                        $flag=0;
                        if(!empty($getCategory->secondary_category)){
                            $flag=1;
                            $venue_category = $getCategory->secondary_category;
                        }
                        if($flag==0){
                            if(!empty($getCategory->sub_category)){
                                $flag=1;
                                $venue_category = $getCategory->sub_category;
                            }
                        }
                        if($flag==0){
                            if(!empty($getCategory->primary_category)){
                                $flag=1;
                                $venue_category = $getCategory->primary_category;
                            }
                        }
                        $categoryArray= [];
                        $sectionArray=[];
                        
                        foreach ($response['zones'] as $key => $value) {
                            $cat_arr = array();
                            $category_id = 0;
                            $category_name = $value['name'];
    
                            $cat_arr['name'] = str_replace("'", "",trim($category_name));
                            $cat_arr['venue_id'] = $venue_id;
                           // $cat_arr['venue_category'] = $venue_category;
                            
    
                            $checkCategory=VenueCategory::where($cat_arr)->first();
                            
                            if(!$checkCategory){
                                $cat_arr['source_id'] = $resource->id;
                                $cat_arr['external_id']=$value['id'];
                                $cat_arr['venue_category'] = $venue_category;
                                $category = VenueCategory::create($cat_arr);
                                $category_id = $category->id;
                                $log->info('Created Category'.$category_id.' for event '.$event->external_id);
                                
                            }else{
                                $checkCategory->external_id=$value['id'];
                                $checkCategory->save();
                                $category_id = $checkCategory->id;
                            }
                            $categoryArray[]=$category_id;
                            //add sections of venue 
                            foreach ($value['sections'] as $sk => $section_value) {
                                $sec_arr = array();
                                $section_name = trim($section_value['name']);
                                $sec_arr['name'] = str_replace("'", "",$section_name);
                                $sec_arr['category_id'] = $category_id;
                                
                                
                                //check same section is available or not 
                                $checkVenueSection=VenueSection::where($sec_arr)->first();
                                if(!$checkVenueSection){
                                    $sec_arr['source_id'] = $resource->id;
                                    $sec_arr['external_id']=$section_value['id'];
                                    $venueSection = VenueSection::create($sec_arr);
                                    $section_id = $venueSection->id;
                                     $log->info('Created section'.$section_id.' for category '.$category_id);
                                }else{
                                    $checkVenueSection->external_id = $section_value['id'];
                                    $checkVenueSection->save();
                                    $section_id = $checkVenueSection->id;
                                }
                                
                                $sectionArray[] = $section_id;
                            }
                        }
                        
                        
                        $categoryArray = implode(',',$categoryArray);
                        $sectionArray = implode(',',$sectionArray);
                        
                        if(!empty($categoryArray) && !empty($sectionArray)){
                            $getCategory->event_categories = $categoryArray;
                            $getCategory->event_section = $sectionArray;
                            $getCategory->save();
                        }
                        
                    }
                    $update_data = array();	
                    $update_data['is_run'] = strtotime('now');
                    Events::where('id',$id)->update($update_data);
                    $eventsUpdated[] = $event->external_id;
                    
                    $log->info($event->id." Event's zone updated successfully.");
                    $log->info("==================================");
                }
            
            if(!empty($eventsUpdated)){
                $eventsUpdated = implode(',',$eventsUpdated);
                $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"(".$eventsUpdated.") Events zone updated successfully."];
                //Logs::create($logData);
            }
        }else{
            $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"No Events available to update zones"];
            //Logs::create($logData);
            Log::info("No Events available to update zones");
        }
    }
    
    public function UpdateExistingEvents(){
        $resource = Sources::where('name','Stubhub')->first();
        if($resource){
            $events = Events::where('source_id',$resource->id)->where('event_date','>=',date('Y-m-d'))->get();
            if($events->count() > 0){
                foreach($events as $event){
                    //fetch event from stubhub
                    echo "Processing Event :".$event->external_id."\r\n";
                    $fetchedEventData = $this->getEvent($event->external_id);
                    if(!empty($fetchedEventData['events'])){
                        $eventData = $fetchedEventData['events']['0'];
                        
                        $event_date = "";
                        if(isset($eventData['eventDateLocal']) && !empty($eventData['eventDateLocal'])){
                            $carbonDate = new Carbon($eventData['eventDateLocal']);
                            $event_date = $carbonDate->format('Y-m-d H:i:s');
                        }
                        
                        if($event_date == $event->event_date){
                            echo "Events Date matched for event : ".$event->external_id."\r\n";
                            continue;
                        }else{
                            //update the event
                            echo "Old event date - ".$event->event_date . " | "."New event date - ".$event_date."\r\n";
                            $event->event_date = $event_date;
                            if($event->mapped_status == 1){
                                $event->update_requires = 1;
                            }
                            
                            if($event->current_version == 0){
                                //update parent event
                                $parentEvent = Events::where('id',$event->parent_event_id)->first();
                                $parentEvent->event_date = $event_date;
                                if($parentEvent->mapped_status == 1){
                                    $parentEvent->update_requires = 1;
                                }
                                $parentEvent->save(); 
                            }
                            
                            $event->save();
                            echo "Event date is updated for event : ".$event->external_id."\r\n";
                        }
                    }else{
                        echo "Events data not found for event:".$event->external_id."\r\n";
                    }
                }
            }else{
                echo "No Events Found.\r\n";
            }
        }else{
            echo "Please check resource \r\n";
        }
        
    }
   
    
}
