<?php
namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
// use App\Logs;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\Traits\ApiTrait;
use App\Traits\StubhubTrait;
use App\VenueAdditionalInfo;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\CategoryController;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
use App\Maps;
use App\EventPerformers;
use App\TivoCategory;

use Aws\S3\S3Client;
class TicketEvolutionController extends Controller
{
    public $API_KEY;
    public $URL; 
    public $URL_GET_EVENT;
    public $SIGNATURE;
    public $page=1;
    public $totalPages;
    public $source = "Ticket Evolution";
    public $categoriesArray;
    public $manual_event = 0;
    public $parentCategory = [];
    public $updateEvent = false;
    public $mapUrl = "https://maps.ticketevolution.com/";
    
    
    public function __construct()
    {
        $this->API_KEY = env('TIVO_API_KEY');
        $this->URL = env('TICKET_EVOLUTION_URL');
        $this->SIGNATURE = env('TIVO_SIGNATURE');
    }
    
    public function fetchEvent(){
        
        $categories = TivoCategory::where('api_flag',1)->get();
        foreach($categories as $category){
            
            $this->totalPages=0;
            $this->page = 1;
            echo "Processing ".$category->name."\n";
            $startDate = date('Y-m-d');
            $endDate = date('Y-m-d',strtotime("+6 months"));
            $curl = curl_init();
         
            $url = $this->URL.'events/?occurs_at.gte='.$startDate.'&occurs_at.lt='.$endDate.'&page='.$this->page.'&country_code=US&category_id='.$category->category_id;
            echo $url."\n";
            curl_setopt_array($curl, array(
              CURLOPT_URL => $url,
              CURLOPT_RETURNTRANSFER => true,
              CURLOPT_ENCODING => "",
              CURLOPT_MAXREDIRS => 10,
              CURLOPT_TIMEOUT => 30,
              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
              CURLOPT_CUSTOMREQUEST => "GET",
              CURLOPT_HTTPHEADER => array(
                "cache-control: no-cache",
                "x-signature: ".$this->SIGNATURE."",
                "x-token: ".$this->API_KEY.""
              ),
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            } else {
                $response = json_decode($response,true);
              
                if(empty($response)){
                     echo "No Response Found.";
                     continue;
                }

                if(isset($response['errors']) && !empty($response['errors'])){
                     echo json_encode($response['errors']);
                     continue;
                }
                 
                if($this->totalPages == 0 && isset($response['total_entries']) && isset($response['per_page'])){
                    if($response['total_entries'] > 100){
                        $this->totalPages = ceil($response['total_entries']/$response['per_page']);
                    }else{
                        $this->totalPages = 1;
                    }
                }else{
                   continue;
                }
                
                
                if(isset($response['current_page']) && !empty($response['current_page'])){
                    $this->page = $response['current_page'];
                }else{
                    continue;
                }
                
                
                $this->processEvent($response);
                
                if( $this->totalPages != $this->page){
                      $this->page = $this->page+1;
                      echo $this->page."-".$this->totalPages."\r\n";
                     
                      $this->fetchRestEvent($category->category_id);
                }
            }
        }
    }
    
    public function fetchRestEvent($id){
        $startDate = date('Y-m-d');
        $endDate = date('Y-m-d',strtotime("+6 months"));
        $curl = curl_init();
        $url = $this->URL.'events/?occurs_at.gte='.$startDate.'&occurs_at.lt='.$endDate.'&page='.$this->page.'&country_code=US&category_id='.$id;
        
        curl_setopt_array($curl, array(
              CURLOPT_URL => $url,
              CURLOPT_RETURNTRANSFER => true,
              CURLOPT_ENCODING => "",
              CURLOPT_MAXREDIRS => 10,
              CURLOPT_TIMEOUT => 30,
              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
              CURLOPT_CUSTOMREQUEST => "GET",
              CURLOPT_HTTPHEADER => array(
                "cache-control: no-cache",
                "x-signature: ".$this->SIGNATURE."",
                "x-token: ".$this->API_KEY.""
              ),
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            } else {
                $response = json_decode($response,true);
              
                if(empty($response)){
                     return true;
                }
                if(isset($response['errors']) && !empty($response['errors'])){
                     echo json_encode($response['errors']);
                     return true;
                }
                 
                if( (isset($response['total_entries']) && !empty($response['total_entries'])) && (isset($response['per_page']) && !empty($response['per_page'])) ){
                    if($this->totalPages == 0){
                        if($response['total_entries'] > 100){
                            $this->totalPages = ceil($response['total_entries']/$response['per_page']);
                        }else{
                            $this->totalPages = 1;
                        }
                        
                    }
                }else{
                    return true;
                }
                $this->page = $response['current_page'] ?? false;
                if(!$this->page)
                    return true;
                
                $this->processEvent($response);
                
                if( $this->totalPages != $this->page){
                      $this->page = $this->page+1;
                      echo $this->page."-".$this->totalPages."\r\n";
                      
                      $this->fetchRestEvent($id);
                }else{
                    return true;
                }
            }
    }
    
    public function fetchUpdatedEvent(){
        $startDate = date('Y-m-d');
            // $endDate = date('Y-m-d',strtotime("+6 months"));
            $curl = curl_init();
            $url = $this->URL.'events/?updated_at='.$startDate.'&page='.$this->page.'&country_code=US';
            echo $url."\n";
            curl_setopt_array($curl, array(
              CURLOPT_URL => $url,
              CURLOPT_RETURNTRANSFER => true,
              CURLOPT_ENCODING => "",
              CURLOPT_MAXREDIRS => 10,
              CURLOPT_TIMEOUT => 30,
              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
              CURLOPT_CUSTOMREQUEST => "GET",
              CURLOPT_HTTPHEADER => array(
                "cache-control: no-cache",
                "x-signature: ".$this->SIGNATURE."",
                "x-token: ".$this->API_KEY.""
              ),
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            } else {
                $response = json_decode($response,true);
             
                if(empty($response)){
                     echo "No Response Found.";
                     exit;
                }
                if(isset($response['errors']) && !empty($response['errors'])){
                     echo json_encode($response['errors']);
                     exit;
                }
                if(!empty($response) && isset($response['events']) && empty($response['events'])){
                    echo "No events found.\n";
                    exit;
                }
                
                 
                if($this->totalPages == 0){
                    $this->totalPages = ceil($response['total_entries']/$response['per_page']);
                }
                
                $this->page = $response['current_page'];
                $this->updateEvent = true;
                $this->processEvent($response);
                
                if( $this->totalPages != $this->page){
                  $this->page = $this->page+1;
                  echo $this->page."-".$this->totalPages."\r\n";
                  $this->fetchUpdatedEvent();
                }
            }
    }
    
    public function processEvent($response){
        if(!empty($response) && isset($response['events']) && !empty($response['events'])){
            $sourceCategoryMapping = new CategoryController();
            $this->categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
            $resource = Sources::where('name',$this->source)->first();
            $log = Log::channel('cron');
            
            $s3 = new S3Client([
                        'version' => 'latest',
                        'region' => env('AWS_DEFAULT_REGION'),
                        'credentials' => [
                            'key' => env('AWS_ACCESS_KEY_ID'),
                            'secret' => env('AWS_SECRET_ACCESS_KEY'),
                        ],
                    ]);
            foreach($response['events'] as $event){
                
                $checkExistingEVent = "";
                if($this->updateEvent == false){
                //skipping the existing events.
                    $checkExistingEVent = Events::where('external_id',$event['id'])->where('source_id',$resource->id)->first();
                    if($checkExistingEVent){
                        echo $event['id']." exist skipping the event\r\n";
                        continue;
                    }
                }
                
                if($this->manual_event != 1){
                    echo "Processing Event - ".$event['id']." ";
                }
                
                $log->info("processing event - ".$event['id']);
                
                /*********** venue section ***************/
                
                if(isset($event['venue']) && !empty($event['venue']) && $event['venue'] != null){
                    $venue_id = "";
                    $venueDetails = $this->fetchVenue($event['venue']['id']);
                    $venue = [];
                    if(isset($venueDetails['name']) && !empty($venueDetails['name'])){
                        $venue['name'] = $venueDetails['name'];
                    }
                    if(isset($venueDetails['location']) && !empty($venueDetails['location'])){
                        $venue['city'] = $venueDetails['location'];
                    }
                    if(isset($venueDetails['country_code']) && !empty($venueDetails['country_code'])){
                        $venue['country'] = $venueDetails['country_code'];
                    }
                    if(isset($venueDetails['id']) && !empty($venueDetails['id'])){
                        $venue['tixstock_id'] = $venueDetails['id'];
                    }
                    $venue['source_id'] = $resource->id;

                    if(!empty($venue['name']) && !empty($venue['country'])){
                        $checkVenue = Venues::where("name","like","%".$venue['name']."%")
                                            ->where("country", $venue['country'])
                                            ->first();
                    }

                    if(!$checkVenue){
                        if(!empty($venue['name']) && !empty($venue['country'])){
                            $venue = Venues::create($venue);
                            $venue_id = $venue->id;
                        }
                    } else {
                        $venue_id = $checkVenue->id;
                    }
                  
                    //check for venue address
                    
                    $venue_address_id=NULL;
                    $checkVenueAddres = new VenueAdditionalInfo();
                    $venueAddress = [];
                    $venueAddress['venue_id'] = $venue_id;
                    $address = [];
                    if(!empty($venue_id)){
                        if(isset($venueDetails['address']['street_address']) && !empty($venueDetails['address']['street_address'])){
                       $address_line = $venueDetails['address']['street_address'];
                   
                       if(!empty($address_line)){
                           //$checkVenue = $checkVenue->where('address_line_1','like','%'.$address[0].'%');
                           $address_line = preg_replace('/[^A-Za-z0-9 ]/', '', $address_line);
                           $checkVenueAddres = $checkVenueAddres->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $address_line);
                           $venueAddress['address_line_1'] = $address_line;
                           if(isset($venueDetails['address']['extended_address']) && !empty($venueDetails['address']['extended_address'])){
                                $address_line_2 = preg_replace('/[^A-Za-z0-9 ]/', '', $venueDetails['address']['extended_address']);
                                $checkVenueAddres = $checkVenueAddres->where('address_line_2','like','%'.trim($address_line_2).'%');
                                $venueAddress['address_line_2'] = $address_line_2;
                           } 
                       }
                   }
                    }
                   
                    $postalCode = isset($venueDetails['address']['postal_code']) && !empty($venueDetails['address']['postal_code']) ? $venueDetails['address']['postal_code'] : '';
                    $venueAddress['post_code'] = $postalCode;
                  
            
                   $checkVenueAddres = $checkVenueAddres->first();

                    if(!$checkVenueAddres){
                        
                        if(gettype($venueAddress) == "array"){
                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                            if(!$checkwithParam){
                                $venueAddress['source_id']  = $resource->id;
                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueadditionalinfo->id;
                            }else{
                                $venue_address_id = $checkwithParam->id;
                            }
                           
                        }
                    }else{
                        //check venue id same or not
                        if($checkVenueAddres->venue_id == $venue_id){
                            $checkVenueAddres->address_line_1 = isset($venueAddress['address_line_1']) && !empty($venueAddress['address_line_1']) ? $venueAddress['address_line_1'] : $checkVenueAddres->address_line_1;
                            $checkVenueAddres->address_line_2 = isset($venueAddress['address_line_2']) && !empty($venueAddress['address_line_2']) ? $venueAddress['address_line_2'] : $checkVenueAddres->address_line_2;
                            $checkVenueAddres->post_code = isset($venueAddress['post_code']) && !empty($venueAddress['post_code']) ? str_replace(' ','',$venueAddress['post_code']) : $checkVenueAddres->post_code;
                            $checkVenueAddres->save();
                            $venue_address_id = $checkVenueAddres->id;
                        }else{
                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                            if(!$checkwithParam){
                                $venueAddress['source_id']  = $resource->id;
                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueadditionalinfo->id;
                            }else{
                                $venue_address_id = $checkwithParam->id;
                            }
                           
                        }
                        
                    }
                  /**** End Venue Section *****/
                }
                
                /*********** Venue category and section ***********/
               
                $eventcategoryArray=[];
                $eventsectionArray=[];
                if((isset($event['configuration']) && !empty($event['configuration']['id'])) && (isset($event['venue']['id']) && !empty($event['venue']['id'])) ){
                    $url = "https://maps.ticketevolution.com/".$event['venue']['id']."/".$event['configuration']['id']."/manifest.json";
                    $venueCatSectionJson = @file_get_contents($url);
                    $venueCatSectionArray = json_decode($venueCatSectionJson,true);
                    
                    if(isset($venueCatSectionArray['sections']) && !empty($venueCatSectionArray['sections'])){
                        foreach($venueCatSectionArray['sections'] as $section=>$category){
                           
                            $venueCategory="";
                            $categorySection="";
                            
                            if(empty($category) || isset($category['zone_name']) && empty($category['zone_name'])){
                                
                                //check if $section has any numeric number;
                                
                                preg_match('/[^ ]*$/', $section, $last_word);
                               
                                if (preg_match('~[0-9]+~', $last_word[0])) {
                                    $categorySection = $section;
                                    $venueCategory = trim(strstr($section,$last_word[0], true));
                                }else{
                                    $venueCategory = $section;
                                    $categorySection = $section;
                                }
                            }else{
                                if(isset($category['zone_name']) && !empty($category['zone_name'])){
                                   $venueCategory = $category['zone_name'];
                                   $categorySection = $section;
                                }else{
                                    $venueCategory = $section;
                                    $categorySection = $section;
                                }
                            }
                            
                            if(empty($venueCategory)){
                                $venueCategory = "NA";
                            } 
                            
                            $checkCategory = VenueCategory::where('name','like','%'.trim($venueCategory).'%')
                                                        ->where('venue_id',$venue_id)
                                                        ->first();
                               
                            if(!$checkCategory){
                                
                                $categoryArray=[
                                                'name'=> !empty($venueCategory) ? $venueCategory : "NA",
                                                'venue_id'=>$venue_id,
                                                'venue_category'=>$venueCategory,
                                                'source_id'=>$resource->id,
                                            ];
                               
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                                $categoryArray =[];
                                $eventcategoryArray[] = $categoryId;
                            }else{
                                $categoryId = $checkCategory->id;
                                $eventcategoryArray[] = $categoryId;
                            }
                            
                            $checkSection=VenueSection::where('name','like','%'.$categorySection.'%')
                                                            ->where('category_id',$categoryId)
                                                            ->first();
                            if(!$checkSection){
                                $sectionArray = [   
                                                    'name'=>$categorySection,
                                                    'category_id'=>$categoryId,
                                                    'source_id'=>$resource->id,
                                             ];
                                $section = VenueSection::create($sectionArray);
                                $eventsectionArray[] = $section->id;
                            }else{
                                $section_id= $checkSection->id;
                                $eventsectionArray[] = $section_id;
                            }
                        }
                    }
                }
                
             
                
                /*********** End Venue category and section *******/
                
                $primary_category = '';
                if(isset($event['category']) && !empty($event['category'])){
                    $primary_category = $this->getParentCategory($event['category'],$this->categoriesArray);
                    
                    if(empty($primary_category)){
                        $primary_category = $event['category']['name'];
                    }
                }
        
                //check for maps
                $map_id="";
                if( (isset($event['venue']['id']) && !empty($event['venue']['id'])) && ( isset($event['configuration']['id']) && !empty($event['configuration']['id']) )){
                    
                    $mapUrl = $this->mapUrl.$event['venue']['id']."/".$event['configuration']['id']."/map.svg";
                    $content = @file_get_contents($mapUrl);
                    $img_tag = [];
                    
                    if($content){
                        preg_match('/(<image .*?>)/', $content, $img_tag);
                        
                        if(empty($img_tag)){
                            $mapName = $event['venue']['slug'] ?? $event['venue']['name'];
                            $mapName = $event['venue']['slug']."-".$event['venue']['id']."-".$event['configuration']['id'];
                            $fileName = $mapName.".svg";
                            if($this->manual_event != 1){
                                echo $fileName."\r\n";
                            }
                            $checkMap = Maps::where('name','like',$fileName)->first();
                            
                            if(!$checkMap){
                                $fileName = $mapName.".svg";
                                $path = env('AWS_FOLDER').'maps/'.$fileName;
                                $s3->putObject([
                                    'Bucket' => env('AWS_BUCKET'),
                                    'Key' => $path,
                                    'Body' => $content,
                                    'ContentType' => 'image/svg+xml',
                                    'ACL' => 'public-read',
                                ]);
                                
                                $bucket = env('AWS_BUCKET');
                                
                                $url = $s3->getObjectUrl($bucket, $path);
                                $newMaps = Maps::create(['name'=>$fileName,'url'=>$url,'status'=>0,'source_id'=>$resource->id]);
                                $map_id = $newMaps->id;
                                echo $map_id."new Map Created\n"; 
                            }else{
                                $map_id = $checkMap->id;
                            } 
                        }
                    }
                }
                
                if(empty($map_id)){
                    $categoryTree = [];
                    $categoryObj = new Categories();
                    $categoryTree = $categoryObj->getParentCatgoryTree($primary_category);
                    
                    
                    $maps = VenueMaps::with('maps')->where('venue_id',$venue_id)->get();
                    if($maps->count() > 0){
                        if(!empty($categoryTree)){
                            foreach($maps as $map){
                                $mapCategory = explode(',',$map->maps->categories);
                                $commonCategory = array_intersect($categoryTree,$mapCategory);
                                if(!empty($commonCategory)){
                                    $map_id = $map->map_id;
                                }
                            }
                        }else{
                            $map_id = $maps[0]->map_id;
                        }
                    }else{
                        $map_id = null;
                    }
                }
                
                if(!empty($map_id) && $map_id != null){
                    //check in venueMaps
                    $checkVenueMap = VenueMaps::where('venue_id',$venue_id)->where('map_id',$map_id)->first();
                    if(!$checkVenueMap){
                        VenueMaps::create(['venue_id'=>$venue_id,'map_id'=>$map_id]);
                    }
                }
                
                /*************** Performer Section ***************/
                $performers = [];
                if(isset($event['performances']) && !empty($event['performances'])){
                    foreach($event['performances'] as $performer){
                        $performer_arr=array();

                        //performer name
                        $performer_name = isset($performer['performer']['name']) ? $performer['performer']['name'] : '';
                        $performer_arr['name'] = str_replace("'", "",trim($performer_name));
                        $performer_arr['name_variation'] = str_replace("'", "",$performer_arr['name']);
                        $performer_arr['source_id']=$resource->id;
                        $performer_arr['external_id']=isset($performer['performer']['id']) && !empty($performer['performer']['id']) ? $performer['performer']['id'] : '';
                        
                        $checkPerformer = Performers::where('name',$performer_arr['name'])->first();
                        if(!$checkPerformer){   
                            $performer = Performers::create($performer_arr);
                            $performerId = $performer->id;
                            $performers[] = $performerId;
                            $log->info("Performer Id:".$performerId." created for event ".$event['id']);
                            
                        }
                        else{
                            if(!empty($checkPerformer->external_id) && !empty($perfomrer_arr['external_id']) && $checkPerformer->external_id != $performer_arr['external_id']){
                                $checkPerformer->external_id = $perfomrer_arr['external_id'] ;
                                $checkPerformer->save();
                                $log->info("Performer External id :".$perfomrer_arr['external_id'] ." saved for event ".$event['id']);
                            }
                            $performerId = $checkPerformer->id;
                             $performers[] = $performerId;
                        }
                    }
                }
            
                
                /************* Event Section ******************/
                $newEvent = 0;
                $new_event_id = '';
                $newEventData = '';
                $event_name = isset($event['name']) ? $event['name'] : '';
                $event_date = isset($event['occurs_at_local']) ? date_format(date_create($event['occurs_at_local']),'Y-m-d H:i:s') : '';
                $event_date_utc = isset($event['occurs_at']) ? $event['occurs_at'] : '';
                
                $event_arr = []; 
                $event_arr['search_type'] = $primary_category;
                $event_arr['name'] = str_replace("'", "",$event_name);
                // $event_arr['performer_1_id'] = $performer_1_id;
                // $event_arr['performer_2_id'] = $performer_2_id;
                $event_arr['event_date'] = $event_date;
                $event_arr['event_date_utc'] = $event_date_utc;
                $event_arr['venue_id'] = $venue_id;
                $event_arr['source_id']=$resource->id;
                $event_arr['status'] = $event['state'];
                $event_arr['venue_address_id']=$venue_address_id;
                $event_arr['map_id'] = $map_id;
                
                //check event external id exist or not 
                $checkExternalId= Events::where('external_id',$event['id'])->orderBy('id','desc')->first();
                if(!$checkExternalId){
                
                    $event_arr['external_id'] = isset($event['id']) ? $event['id'] : '';
                    $event_date_array = explode(' ',$event_arr['event_date']);
                    $event_date = $event_date_array[0];
                    $event_name = '"'.str_replace("-"," ",$event_arr['name']).'"';
                    $checkEvent = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $event_name )
                                        ->where('venue_id',$venue_id)
                                        ->whereBetween('event_date',[$event_date." 00:00:00",$event_date." 23:59:59"])
                                        ->first();
                    
                    if(!$checkEvent){
                        $event_arr['url'] = isset($event['url']) ? $event['url'] : '';
                        $event_arr['status']=$event['state'];
                        
                        $newEventData=Events::create($event_arr);
                        
                        $event_id=$newEventData->id;
                        $new_event_id =$newEventData->id;
                        $newEvent = 1;
                        
                        $log->info("created  event with external id : ".$event['id']);
                        $createdEvents[] = $newEventData->external_id;
                    }else{
                        //check source of event(if matched then update event or create new event with resource)
                         $eventUpdateFlag = 0;
                         if($checkEvent->source_id = $resource->id){
                            //if event is from same resource update the event
                            if($checkEvent->mapped_status == 1){
                                $excludeField = ['external_id','search_type','map_id','status','source_id'];
                                $prevArray = $checkEvent->toArray();
                                $prev_arr=[];
                                foreach($event_arr as $key=>$val){
                                    if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                        $eventUpdateFlag=1;
                                        $prev_arr['prev_'.$key] = $val;
                                        $prev_arr['update_requires'] =1;
                                        //$log->info("Updated event's ".$key." to: ".$value['id']);
                                    }
                                }
                                if($eventUpdateFlag==1){
                                    //update flag of previous event
                                    //$event_arr['update_requires']=1;
                                     Events::where('id',$checkEvent->id)->update($prev_arr);
                                    //create new event 
                                    $event_arr['update_requires'] = 0;
                                    $event_arr['current_version'] = 0;
                                    $event_arr['mapped_status'] = 0;
                                    $event_arr['parent_event_id'] = $checkEvent->id;
                                    $newEventData = Events::create($event_arr);
                                    $event_id = $newEventData->id;
                                    $new_event_id =$newEventData->id;
                                    $newEvent = 1;
                                }
                            }else{
                                Events::where('id',$checkEvent->id)->update($event_arr);
                                $event_id=$checkEvent->id;
                                //update performers
                                EventPerformers::where('event_id',$event_id)->delete();
                                if(!empty($performers)){
                                    foreach($performers as $performer){
                                        EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                    }
                                }
                            }
                         }else{
                            // $checkEvent->current_version=0;
                            // $checkEvent->save();
                            //create versioning
                            $event_arr['update_requires'] = 0;
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['parent_event_id'] = $checkEvent->id;
                            $newEventData= Events::create($event_arr);
                            $event_id = $newEventData->id;
                            $new_event_id =$newEventData->id;
                            $newEvent = 1;
                         }
                    }
                }else{
                    //check if same event exist or not 
                    $checkEventUpdate=Events::where($event_arr)->first();
                    
                    if(!$checkEventUpdate){
                        //event is updated
                        $event_id = $checkExternalId->id;
                        $event_arr['url'] = isset($event['url']) ? $event['url'] : '';
                        $eventUpdateFlag=0;
                        
                        if($checkExternalId->mapped_status == 1){
                            $excludeField = ['external_id','search_type','map_id','status','source_id'];
                            $prevArray = $checkExternalId->toArray();
                            $prev_arr = [];
                            foreach($event_arr as $key=>$val){
                                if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                    $eventUpdateFlag=1;
                                    $prev_arr['prev_'.$key] = $val;
                                    
                                    $log->info("Updated event's ".$key." to: ".$event['id']);
                                }
                            }
                            if($eventUpdateFlag==1){
                                $prev_arr['update_requires'] =1;
                            }
                        }
                        
                        if($eventUpdateFlag==1){
                            //$event_arr['update_requires']=1;
                            Events::where('id',$checkExternalId->id)->update($prev_arr);
                            
                           // Events::where('id',$checkExternalId->id)->update(['current_version'=>0]);
                            //create new event
                            $event_arr['external_id'] = $event['id'] ;
                            $event_arr['url'] = isset($event['url']) ? $event['url'] : '';
                            $event_arr['status']=$event['state'];
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['update_requires']=0;
                            $event_arr['venue_address_id']=$venue_address_id;
                            $event_arr['parent_event_id']=$checkExternalId->id;
                            $newEventData = Events::create($event_arr);
                            $event_id = $newEventData->id;
                            $new_event_id =$newEventData->id;
                            $newEvent = 1;
                            $log->info('New version of Event created for :'.$event['id']);
                            $updatedEvents[] = $checkExternalId->external_id;
                            $log->info("Event ".$event['id']." updated ");
                        }else{
                            $event_id = $checkExternalId->id;
                            Events::where('id',$checkExternalId->id)->update($event_arr);
                            EventPerformers::where('event_id',$event_id)->delete();
                            if(!empty($performers)){
                                foreach($performers as $performer){
                                    EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                }
                            }
                        }   
                    }else{
                        
                        $event_id = $checkEventUpdate->id;
                        EventPerformers::where('event_id',$event_id)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                            }
                        }
                    }
                }
                
                
                if(!empty($new_event_id)){
                    $event_id = $new_event_id;
                    $cat_arr['event_id'] = $new_event_id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                        }
                    }
                }
                
                
                if(isset($event_id)){
                    $checkEventCategory=EventCategory::where('event_id',$event_id)->first();
                    if(!$checkEventCategory){
                        $cat_arr['primary_category'] = $primary_category;
                        $cat_arr['event_id'] = $event_id;
                        $cat_arr['venue_id']=$venue_id;
                        $cat_arr['venue_address_id']=$venue_address_id;
                        $cat_arr['event_categories']=!empty($eventcategoryArray) ? implode(',',$eventcategoryArray) : '';
                        $cat_arr['event_section']=!empty($eventsectionArray) ? implode(',',$eventsectionArray) : '';
                        $cat_arr['source_id']=$resource->id;
                        EventCategory::create($cat_arr);
                        
                        $log->info("Event category created for event :".$event['id']);}
                    else{
                        $checkEventCategory->primary_category = $primary_category;
                        $checkEventCategory->venue_id = $venue_id;
                        $checkEventCategory->venue_address_id = $venue_address_id;
                        $checkEventCategory->save();
                    }
                }
                
                $log->info("End event processing of event :".$event['id']);
                $log->info("=============================================");
                if($this->manual_event != 1){
                echo $event['id']."Event Processed.\r\n";
                }
            }
            if($this->manual_event == 1){
                return ['success'=>true];
            }
        }else{
            // $logData=['type'=>"cron",'domain'=>"Ticket Evolution","message"=>"No events found."];
            // Logs::create($logData);
            
            Log::info("No events found.");
            return true;
        }
    }
    
    public function fetchVenue($id){
        $curl = curl_init();
        $url = $this->URL.'venues/'.$id;
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "x-signature: ".$this->SIGNATURE."",
            "x-token: ".$this->API_KEY.""
          ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
            $response = json_decode($response,true);
            return $response;
        }
    }
    
    public function getParentCategory($category,$categoriesArray){
        $primary_category="";
        $resource = Sources::where('name',$this->source)->first();
        if(isset($category['name']) && !empty($category['name'])){
            
            $check = SourceCategoryMapping::with('category')->where('source_category_name',$category['name'])->where('source_id',$resource->id)->first();
            if($check){
                $primary_category = $check->category->name;
            }
            
            
            if (empty($primary_category) && false !== $key = array_search($category['name'], $categoriesArray)) {
                //add category source mapping 
                $primary_category = $category['name'];
            }
            
            
            if(!empty($primary_category)){
                return $primary_category;
            }
            
            if(empty($primary_category) && isset($category['parent']) && !empty($category['parent'])){
                $this->getParentCategory($category['parent'],$categoriesArray);
            }
        } 
    }
    
    public function fetchEventByExternalId($externalId){
        $curl = curl_init();
        $url = $this->URL.'events/'.$externalId;
       
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "x-signature: ".$this->SIGNATURE."",
            "x-token: ".$this->API_KEY.""
          ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
            
            $response = json_decode($response,true);
            return $response;
        }
    }
    
    public function fetchCategories(){
        $curl = curl_init();
        $url = $this->URL.'categories';
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "x-signature: ".$this->SIGNATURE."",
            "x-token: ".$this->API_KEY.""
          ),
        ));
        
        $response = curl_exec($curl);
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
            
            $response = json_decode($response,true);
           
            $categories = $response['categories'];
            foreach($categories as $category){
                
                if(empty($category['parent'])){
                    $checkCategory = TivoCategory::where('name',$category['name'])->first();
                    if(!$checkCategory){
                        TivoCategory::create(['name'=>$category['name'],'parent_id'=>0,'category_id'=>$category['id']]);
                    }
                }else{
                    // parent category exist or not 
                    $checkParentCategory = TivoCategory::where('name',$category['parent']['name'])->first();
                    if($checkParentCategory){
                        $parentId=$checkParentCategory->id;
                    }else{
                        $parentCategory = TivoCategory::create(['name'=>$category['parent']['name'],'parent_id'=>0,'category_id'=>$category['id']]);
                        $parentId =  $parentCategory->id;
                    }
                    
                    $checkduplicateCat = TivoCategory::where('name',$category['name'])->first();
                    if(!$checkduplicateCat){
                        TivoCategory::create(['name'=>$category['name'],'parent_id'=>$parentId,'category_id'=>$category['id']]);
                    }
                    $this->processCategory($category);
                }
                
               
              /* if(empty($category['parent'])){
                   $this->parentCategory[$category['name']] = [];
               }else{
                   if(!isset($this->parentCategory[$category['parent']['name']])){
                       $this->parentCategory[$category['parent']['name']]= [];
                   }
                  $this->parentCategory[$category['parent']['name']][] = $category['name'];
                   $this->getParentCategories($category['parent'],$this->parentCategory[$category['parent']['name']]);
               }*/
            }
             return response()->json(['success'=>true,'msg'=>"Ticket evolution categories updated"]);
        }
    }
    
    public function processCategory($category){
        if(empty($category['parent'])){
            $checkCategory = TivoCategory::where('name',$category['name'])->first();
            if(!$checkCategory){
                $parentCategory = TivoCategory::create(['name'=>$category['name'],'parent_id'=>0,'category_id'=>$category['id']]);
                $parentId = $parentCategory->id;
            }else{
                $parentId = $checkCategory->id;
            }
            
            // TivoCategory::where('name',$category['name'])->update(['parent_id'=>$parentId]);
        }else{
            //check for Parent Category
            $checkParentCategory = TivoCategory::where('name',$category['parent']['name'])->first();
            if($checkParentCategory){
                $parentId=$checkParentCategory->id;
            }else{
                $parentCategory = TivoCategory::create(['name'=>$category['parent']['name'],'parent_id'=>0,'category_id'=>$category['id']]);
                $parentId =  $parentCategory->id;
            }
            
            $checkduplicateCat = TivoCategory::where('name',$category['name'])->first();
            if(!$checkduplicateCat){
                TivoCategory::create(['name'=>$category['name'],'parent_id'=>$parentId,'category_id'=>$category['id']]);
            }else{
                $checkduplicateCat->parent_id = $parentId;
                $checkduplicateCat->save();
            }
            $this->processCategory($category['parent']);
        }
    }
    
    public function getParentCategories($category,$parentcategoryArray){
        
        if(empty($category['parent'])){
            $this->parentCategory[$category['name']]=$parentcategoryArray;
        }else{
            if(isset($this->parentCategory[$category['parent']['name']]) && in_array($category['name'],$this->parentCategory[$category['parent']['name']])){
                if (($key = array_search($category['name'],$this->parentCategory[$category['parent']['name']])) !== false) {
                    unset($this->parentCategory[$category['parent']['name']][$key]);
                }
            }
            
            $this->parentCategory[$category['parent']['name']][$category['name']][] = $parentcategoryArray;
            if(in_array($this->parentCategory[$category['name']],$this->parentCategory)){
                unset($this->parentCategory[$category['name']]);
            }
            $this->getParentCategories($category['parent'],$this->parentCategory[$category['parent']['name']]);
        }
    }
    
    public function updateEvents($type){
        
        $resource = Sources::where('name',$this->source)->first();
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        
        
        
         
        
        
        switch ($type) {
          case "updateMaps":
            //code block
            $events = Events::with('venue')->where('map_id',NULL)->where('created_at','<=',Date('Y-m-d'))->where('source_id',$resource->id)->get();
            if($events->count() <= 0){
                return response()->json(['msg'=>'No events found.']);
            }
            foreach($events as $event){
            //fetch events details
            
            $curl = curl_init();
            $url = $this->URL.$event->url;
          
            curl_setopt_array($curl, array(
                  CURLOPT_URL => $url,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => "",
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 30,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => "GET",
                  CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache",
                    "x-signature: ".$this->SIGNATURE."",
                    "x-token: ".$this->API_KEY.""
                  ),
            ));
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            }else{
               $eventDetails = json_decode($response,true);
               //check map
               if(!empty($eventDetails['venue']) && !empty($eventDetails['configuration'])){
                    $mapUrl = $this->mapUrl.$eventDetails['venue']['id']."/".$eventDetails['configuration']['id']."/map.svg";
                
                    $mapContent = @file_get_contents($mapUrl);
                    
                    if($mapContent){
                        $mapContent = preg_replace('/<(\s*)image[^<>]*>/i', "", $mapContent);
                        $mapContent = preg_replace('</image>', "", $mapContent);
                        $mapContent = preg_replace("<>", "", $mapContent);
                        $mapContent = preg_replace('/<(\s*)img[^<>]*>/i',"", $mapContent);
                        $mapContent = preg_replace('</img>', "", $mapContent);
                        $mapContent = preg_replace("<>", "", $mapContent);
                        preg_match('/(<image .*?>)/', $mapContent, $img_tag);
                                
                                if(empty($img_tag)){
                                    $mapName = $eventDetails['venue']['slug'] ?? $eventDetails['venue']['name'];
                                    $mapName = $eventDetails['venue']['slug']."-".$eventDetails['venue']['id']."-".$eventDetails['configuration']['id'];
                                    $fileName = $mapName.".svg";
                                    if($this->manual_event != 1){
                                        echo $fileName."\r\n";
                                    }
                                    $checkMap = Maps::where('name',$fileName)->first();
                                    
                                    if(!$checkMap){
                                        $fileName = $mapName.".svg";
                                        $path = env('AWS_FOLDER').'maps/'.$fileName;
                                        $s3->putObject([
                                            'Bucket' => env('AWS_BUCKET'),
                                            'Key' => $path,
                                            'Body' => $mapContent,
                                            'ContentType' => 'image/svg+xml',
                                            'ACL' => 'public-read',
                                        ]);
                                        
                                        $bucket = env('AWS_BUCKET');
                                        
                                        $url = $s3->getObjectUrl($bucket, $path);
                                        $newMaps = Maps::create(['name'=>$fileName,'url'=>$url,'status'=>0,'source_id'=>$resource->id]);
                                        $map_id = $newMaps->id;
                                        echo $map_id."new Map Created\n"; 
                                    }else{
                                        if($checkMap->status == 0){
                                            //update the map content if not grouped
                                            
                                            $path = env('AWS_FOLDER').'maps/'.$fileName;
                                            
                                            $bucket = env('AWS_BUCKET');
                                            
                                            
                                            $existingObject = $s3->headObject([
                                                    'Bucket' => $bucket,
                                                    'Key' => $path
                                            ]);
                                            
                                            $lastModifiedBefore = $existingObject['LastModified'];
                                            
                                            $eTag = $existingObject['ETag'];
                                            
                                            $s3->putObject([
                                                'Bucket' => $bucket,
                                                'Key' => $path,
                                                'Body' => $mapContent,
                                                'ContentLength' => strlen($mapContent),
                                                'ServerSideEncryption' => 'AES256',
                                                'ContentType' => 'image/svg+xml',
                                                'ACL' => 'public-read',
                                                'IfMatch' => $eTag
                                            ]);
                                            
                                            $object = $s3->headObject([
                                                    'Bucket' => $bucket,
                                                    'Key' => $path
                                            ]);

                                        }
                                        $map_id = $checkMap->id;
                                    } 
                                }
                            
                        
                                if(!empty($map_id) && $map_id != null){
                                    //check in venueMaps
                                    $checkVenueMap = VenueMaps::where('venue_id',$event->venue_id)->where('map_id',$map_id)->first();
                                    if(!$checkVenueMap){
                                        VenueMaps::create(['venue_id'=>$event->venue_id,'map_id'=>$map_id]);
                                    }
                                    $event->map_id = $map_id;
                                    if($event->mapped_status == 1){
                                        $event->update_requires = 1;
                                        $event->update_map_flag = 1;
                                    }
                                    $event->save();
                                    echo $event->id." map added to the event.\r\n";
                                }else{
                                    echo $event->id." event skipped.\r\n";
                                    continue;
                                }
                    }
                }
               
               else{
                   echo $event->id." event skipped.\r\n";
               }
            }
        }
            break;
          case "updateDates":
            //code block;
            
            break;
          case "updateCategory":
            //code block
            break;
          default:
            //code block
        }

    }
    
    public function updateEventMaps(){
       
        $resource = Sources::where('name',$this->source)->first();
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        
        $events = Events::with('venue')->where('map_id',NULL)->where('created_at','<=',Date('Y-m-d'))->where('source_id',$resource->id)->get();
        
        if($events->count() <= 0){
            return response()->json(['msg'=>'No events found.']);
        }
        
        foreach($events as $event){
            //fetch events details
            
            $curl = curl_init();
            $url = $this->URL.$event->url;
          
            curl_setopt_array($curl, array(
                  CURLOPT_URL => $url,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => "",
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 30,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => "GET",
                  CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache",
                    "x-signature: ".$this->SIGNATURE."",
                    "x-token: ".$this->API_KEY.""
                  ),
            ));
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            }else{
               $eventDetails = json_decode($response,true);
               //check map
               if(!empty($eventDetails['venue']) && !empty($eventDetails['configuration'])){
                    $mapUrl = $this->mapUrl.$eventDetails['venue']['id']."/".$eventDetails['configuration']['id']."/map.svg";
                
                    $mapContent = @file_get_contents($mapUrl);
                    
                    if($mapContent){
                        $mapContent = preg_replace('/<(\s*)image[^<>]*>/i', "", $mapContent);
                        $mapContent = preg_replace('</image>', "", $mapContent);
                        $mapContent = preg_replace("<>", "", $mapContent);
                        $mapContent = preg_replace('/<(\s*)img[^<>]*>/i',"", $mapContent);
                        $mapContent = preg_replace('</img>', "", $mapContent);
                        $mapContent = preg_replace("<>", "", $mapContent);
                        preg_match('/(<image .*?>)/', $mapContent, $img_tag);
                                
                                if(empty($img_tag)){
                                    $mapName = $eventDetails['venue']['slug'] ?? $eventDetails['venue']['name'];
                                    $mapName = $eventDetails['venue']['slug']."-".$eventDetails['venue']['id']."-".$eventDetails['configuration']['id'];
                                    $fileName = $mapName.".svg";
                                    if($this->manual_event != 1){
                                        echo $fileName."\r\n";
                                    }
                                    $checkMap = Maps::where('name',$fileName)->first();
                                    
                                    if(!$checkMap){
                                        $fileName = $mapName.".svg";
                                        $path = env('AWS_FOLDER').'maps/'.$fileName;
                                        $s3->putObject([
                                            'Bucket' => env('AWS_BUCKET'),
                                            'Key' => $path,
                                            'Body' => $mapContent,
                                            'ContentType' => 'image/svg+xml',
                                            'ACL' => 'public-read',
                                        ]);
                                        
                                        $bucket = env('AWS_BUCKET');
                                        
                                        $url = $s3->getObjectUrl($bucket, $path);
                                        $newMaps = Maps::create(['name'=>$fileName,'url'=>$url,'status'=>0,'source_id'=>$resource->id]);
                                        $map_id = $newMaps->id;
                                        echo $map_id."new Map Created\n"; 
                                    }else{
                                        if($checkMap->status == 0){
                                            //update the map content if not grouped
                                            
                                            $path = env('AWS_FOLDER').'maps/'.$fileName;
                                            
                                            $bucket = env('AWS_BUCKET');
                                            
                                            
                                            $existingObject = $s3->headObject([
                                                    'Bucket' => $bucket,
                                                    'Key' => $path
                                            ]);
                                            
                                            $lastModifiedBefore = $existingObject['LastModified'];
                                            
                                            $eTag = $existingObject['ETag'];
                                            
                                            $s3->putObject([
                                                'Bucket' => $bucket,
                                                'Key' => $path,
                                                'Body' => $mapContent,
                                                'ContentLength' => strlen($mapContent),
                                                'ServerSideEncryption' => 'AES256',
                                                'ContentType' => 'image/svg+xml',
                                                'ACL' => 'public-read',
                                                'IfMatch' => $eTag
                                            ]);
                                            
                                            $object = $s3->headObject([
                                                    'Bucket' => $bucket,
                                                    'Key' => $path
                                            ]);

                                        }
                                        $map_id = $checkMap->id;
                                    } 
                                }
                            
                        
                                if(!empty($map_id) && $map_id != null){
                                    //check in venueMaps
                                    $checkVenueMap = VenueMaps::where('venue_id',$event->venue_id)->where('map_id',$map_id)->first();
                                    if(!$checkVenueMap){
                                        VenueMaps::create(['venue_id'=>$event->venue_id,'map_id'=>$map_id]);
                                    }
                                    $event->map_id = $map_id;
                                    if($event->mapped_status == 1){
                                        $event->update_requires = 1;
                                        $event->update_map_flag = 1;
                                    }
                                    $event->save();
                                    echo $event->id." map added to the event.\r\n";
                                }else{
                                    echo $event->id." event skipped.\r\n";
                                    continue;
                                }
                    }
               }
               
               else{
                   echo $event->id." event skipped.\r\n";
               }
            }
            
           
        }
    }
    
    public function updateEventDates(){
        $resource = Sources::where('name',$this->source)->first();
        $events = Events::where('source_id',$resource->id)->where('event_date','>=',Date('Y-m-d'))->get();
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        if($events->count() <= 0){
            return response()->json(['msg'=>'No events found.']);
        }
        foreach($events as $event){
                
            
            //fetch events details
            $curl = curl_init();
            $url = $this->URL.$event->url;
          
            curl_setopt_array($curl, array(
                  CURLOPT_URL => $url,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => "",
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 30,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => "GET",
                  CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache",
                    "x-signature: ".$this->SIGNATURE."",
                    "x-token: ".$this->API_KEY.""
                  ),
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            }else{
                $eventDetails = json_decode($response,true);
                $eventDate = isset($eventDetails['occurs_at_local']) ? date_format(date_create($eventDetails['occurs_at_local']),'Y-m-d H:i:s') : '';
                $map_id = "";
                if($eventDate == $event->event_date){
                    echo "The event date is same.skipping the event.\r\n";
                    continue;
                }
                //check map
               
                if($event->map_id == null){
                    if(!empty($eventDetails['venue']) && !empty($eventDetails['configuration'])){
                        $mapUrl = $this->mapUrl.$eventDetails['venue']['id']."/".$eventDetails['configuration']['id']."/map.svg";
                    
                        $mapContent = @file_get_contents($mapUrl);
                        
                        if($mapContent){
                            $mapContent = preg_replace('/<(\s*)image[^<>]*>/i', "", $mapContent);
                            $mapContent = preg_replace('</image>', "", $mapContent);
                            $mapContent = preg_replace("<>", "", $mapContent);
                            $mapContent = preg_replace('/<(\s*)img[^<>]*>/i',"", $mapContent);
                            $mapContent = preg_replace('</img>', "", $mapContent);
                            $mapContent = preg_replace("<>", "", $mapContent);
                            preg_match('/(<image .*?>)/', $mapContent, $img_tag);
                                    
                            if(empty($img_tag)){
                                $mapName = $eventDetails['venue']['slug'] ?? $eventDetails['venue']['name'];
                                $mapName = $eventDetails['venue']['slug']."-".$eventDetails['venue']['id']."-".$eventDetails['configuration']['id'];
                                $fileName = $mapName.".svg";
                                echo $fileName."\r\n";
                                $checkMap = Maps::where('name',$fileName)->first();
                                
                                if(!$checkMap){
                                    $fileName = $mapName.".svg";
                                    $path = env('AWS_FOLDER').'maps/'.$fileName;
                                    $s3->putObject([
                                        'Bucket' => env('AWS_BUCKET'),
                                        'Key' => $path,
                                        'Body' => $mapContent,
                                        'ContentType' => 'image/svg+xml',
                                        'ACL' => 'public-read',
                                    ]);
                                    
                                    $bucket = env('AWS_BUCKET');
                                    
                                    $url = $s3->getObjectUrl($bucket, $path);
                                    $newMaps = Maps::create(['name'=>$fileName,'url'=>$url,'status'=>0,'source_id'=>$resource->id]);
                                    $map_id = $newMaps->id;
                                    echo $map_id."new Map Created\n"; 
                                }else{
                                    if($checkMap->status == 0){
                                        //update the map content if not grouped
                                        
                                        $path = env('AWS_FOLDER').'maps/'.$fileName;
                                        
                                        $bucket = env('AWS_BUCKET');
                                        
                                        
                                        $existingObject = $s3->headObject([
                                                'Bucket' => $bucket,
                                                'Key' => $path
                                        ]);
                                        
                                        $lastModifiedBefore = $existingObject['LastModified'];
                                        
                                        $eTag = $existingObject['ETag'];
                                        
                                        $s3->putObject([
                                            'Bucket' => $bucket,
                                            'Key' => $path,
                                            'Body' => $mapContent,
                                            'ContentLength' => strlen($mapContent),
                                            'ServerSideEncryption' => 'AES256',
                                            'ContentType' => 'image/svg+xml',
                                            'ACL' => 'public-read',
                                            'IfMatch' => $eTag
                                        ]);
                                        
                                        $object = $s3->headObject([
                                                'Bucket' => $bucket,
                                                'Key' => $path
                                        ]);
    
                                    }
                                    $map_id = $checkMap->id;
                                } 
                        }
                            //check in venueMaps
                            if(!empty($map_id) && $map_id != null){
                                $checkVenueMap = VenueMaps::where('venue_id',$event->venue_id)->where('map_id',$map_id)->first();
                                if(!$checkVenueMap){
                                    VenueMaps::create(['venue_id'=>$event->venue_id,'map_id'=>$map_id]);
                                }
                                echo $event->id." map added to the event.\r\n";
                            } 
                        }
                    }
                }
                
                if(isset($map_id) && !empty($map_id)){
                    $event->map_id = $map_id; 
                }
            
                $event->event_date =  isset($eventDetails['occurs_at_local']) ? date_format(date_create($eventDetails['occurs_at_local']),'Y-m-d H:i:s') : '';
                $event->event_date_utc = isset($eventDetails['occurs_at']) ? $eventDetails['occurs_at'] : '';
                if($event->mapped_status == 1){
                    $event->update_requires = 1;
                    // $event->update_map_flag = 1;
                }
                
                $event->save();
                
                echo "The event id ".$event->external_id." prcessed.\r\n";

            } 
        }
    }
    
    public function updateEventCategory(){
        $resource = Sources::where('name',$this->source)->first();
        $events = Events::where('source_id',$resource->id)->where('event_date','>=',Date('Y-m-d'))->get();
        
        if($events->count() <= 0){
            return response()->json(['msg'=>'No events found.']);
        }
        foreach($events as $event){
                
            
            //fetch events details
            $curl = curl_init();
            $url = $this->URL.$event->url;
          
            curl_setopt_array($curl, array(
                  CURLOPT_URL => $url,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => "",
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 30,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => "GET",
                  CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache",
                    "x-signature: ".$this->SIGNATURE."",
                    "x-token: ".$this->API_KEY.""
                  ),
            ));
            
            $response = curl_exec($curl);
            $err = curl_error($curl);
            
            curl_close($curl);
            
            if ($err) {
              echo "cURL Error #:" . $err;
            }else{
                $eventDetails = json_decode($response,true);
                
                $primary_category = '';
                if(isset($eventDetails['category']) && !empty($eventDetails['category'])){
                    $primary_category = $this->getParentCategory($eventDetails['category'],$this->categoriesArray);
                    
                    if(empty($primary_category)){
                        $primary_category = $eventDetails['category']['name'];
                    }
                }
                
                if($event->search_type == $primary_category){
                    echo "Event Category is same.Skipping the event :".$event->external_id."\r\n";
                    continue;
                }
                
                
                if(!empty($primary_category)){
                
                    $event->search_type = $primary_category;
                    if($event->mapped_status == 1){
                        $event->update_requires = 1;
                    }
                    
                    $event->save();
                
                    EventCategory::where('event_id',$event->id)->update(['primary_category'=>$primary_category]);
                }
                echo "The event id ".$event->external_id." processed.\r\n";

            } 
        }
    }
    
}