<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
// use App\Logs;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\Traits\ApiTrait;
use App\Traits\StubhubTrait;
use App\VenueAdditionalInfo;
use Illuminate\Support\Facades\Log;
use App\Http\Controllers\CategoryController;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
use App\EventPerformers;
class TicketmasterController extends Controller
{
    // use ApiTrait,StubhubTrait;
    
    public $API_KEY; 
    public $URL_GET_EVENT; 
    public $URL; 
    public $EUROPE_MARKET_ID = ['202','203','204','205','206','207','208','209','210','211','500','501','502','503','600'];
    public $TotalEvent = 0;
    public $page = 0;
    public $totalPages = 0;
    public $source = "TicketMaster";
    public $manual_event;
    public $categoriesArray = [];
    
    
    public function __construct(){
        $this->API_KEY = env('TICKET_MASTER_TOKEN');
        $this->URL_GET_EVENT = env('TICKET_MASTER_API_GET_EVENT');
        $this->URL = env('TICKET_MASTER_API_URL');
    }
    
    public function fetchEvent(){
        
        
        $curl = curl_init();
        
        $startDate = Date('Y-m-d 00:00:00');
        $startDate = date(DATE_ISO8601, strtotime($startDate));
        $startDate = substr($startDate, 0, strpos($startDate, "+"));
        
        
        $lastEventByDate = Events::orderBy('event_date','desc')->first();
        $endDate = date('Y-m-d 23:59:59',strtotime($lastEventByDate->event_date));
        $endDate = date(DATE_ISO8601, strtotime($endDate));
        $endDate = substr($endDate, 0, strpos($endDate, "+"));
        
        //fetch all future event and find version from tixstock for 
        
        // $url = $this->URL.''.$this->URL_GET_EVENT;
        $url = $this->URL.''.$this->URL_GET_EVENT;
        $params = ['apikey'=>$this->API_KEY,'locale'=>"*",'marketId'=>implode(',',$this->EUROPE_MARKET_ID),'size'=>20,'page'=>$this->page,'startDateTime'=>$startDate,'endDateTime'=>$endDate];
        $url = $url .'?'.http_build_query($params);
        
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "content-type: application/json",
            // "postman-token: 15455206-d06f-fcbc-fc93-757d82023564"
          ),
        ));
        
        $response = curl_exec($curl);
        
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
            $response = json_decode($response,true);
            
            if(empty($response)){
                 echo "No Response Found.";
                 exit;
            }
            if(isset($response['errors']) && !empty($response['errors'])){
                 echo json_encode($response['errors']);
                 $log = Log::channel('cron');
                 $log->info(json_encode($response['errors']));
                 exit;
            }
             
            if($this->totalPages == 0){
                $this->totalPages = $response['page']['totalPages'];
            }
            
            $this->processEvent($response);
            
            if( $this->totalPages != $this->page){
                  $this->page = $this->page+1;
                  echo $this->page."\r\n";;
                  $this->fetchEvent();
            }
        }
    } 
    
    public function fetchExistingEventVersion(){
        $events = Events::with(['performers'])->Where('event_date','>=',date('Y-m-d 00:00:00'))->where('current_version',1)->get();
        
        if($events){
            foreach($events as $event){
                echo "processing event ".$event->external_id."\r\n";
                $startDate = Date('Y-m-d 00:00:00',strtotime($event->event_date));
                $startDate = date(DATE_ISO8601, strtotime($startDate));
                $startDate = substr($startDate, 0, strpos($startDate, "+")).'Z';
                
                
                $endDate = Date('Y-m-d 23:59:59',strtotime($event->event_date));
                $endDate = date(DATE_ISO8601, strtotime($endDate));
                $endDate = substr($endDate, 0, strpos($endDate, "+")).'Z';
                
                $keyword=$event->performers[0]->name;
                if(isset($event->performers[1])){
                    $keyword.= ",".$event->performers[1]->name;
                }
                
                // $url = $this->URL.''.$this->URL_GET_EVENT;
                $url = $this->URL.''.$this->URL_GET_EVENT;
                $params = [ 
                            'apikey'=>$this->API_KEY,
                            'locale'=>"*",'marketId'=>implode(',',$this->EUROPE_MARKET_ID),
                            // 'startDateTime'=>$startDate,
                            // 'endDateTime'=>$endDate,
                            'keyword'=>$keyword
                        ];
                $url = $url .'?'.http_build_query($params);
               
                $curl = curl_init();
                curl_setopt_array($curl, array(
                  CURLOPT_URL => $url,
                  CURLOPT_RETURNTRANSFER => true,
                  CURLOPT_ENCODING => "",
                  CURLOPT_MAXREDIRS => 10,
                  CURLOPT_TIMEOUT => 30,
                  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                  CURLOPT_CUSTOMREQUEST => "GET",
                  CURLOPT_HTTPHEADER => array(
                    "cache-control: no-cache",
                    "content-type: application/json",
                    // "postman-token: 15455206-d06f-fcbc-fc93-757d82023564"
                  ),
                ));
        
                $response = curl_exec($curl);
                
                $err = curl_error($curl);
        
                curl_close($curl);
                
                $response = json_decode($response,true);
                
                if(!empty($response['_embedded']['events'])){
                    
                    $this->processEvent($response);
                }
                
            }
        }
    }
    
    public function processEvent($response){
     
        $sourceCategoryMapping = new CategoryController();
        $this->categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
        $resource = Sources::where('name',$this->source)->first();
        $log = Log::channel('cron');
        if(!empty($response['_embedded']['events'])){
            foreach($response['_embedded']['events'] as $event){
                
            $log->info("processing event - ".$event['id']);
             
                   /***** Venue section *****/
                   
                   //check for venue
                   $venue = [];
                   $venue['name'] = $event['_embedded']['venues'][0]['name'];
                   $venue['city'] = isset($event['_embedded']['venues'][0]['city']['name']) && !empty($event['_embedded']['venues'][0]['city']['name']) ? $event['_embedded']['venues'][0]['city']['name'] : "";
                   $venue['country'] = isset($event['_embedded']['venues'][0]['country']['countryCode']) && !empty($event['_embedded']['venues'][0]['country']['countryCode']) ? $event['_embedded']['venues'][0]['country']['countryCode'] : "";
                   $venue['source_id'] = $resource->id;
                   $venue['tixstock_id'] = $event['_embedded']['venues'][0]['id'];
                   
                   if(isset($venue['name']) && !empty($venue['name'])){
                        //$checkVenue = Venues::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $venue['name']);
                        $checkVenue = Venues::where("name","like","%".$venue['name']."%");
                   }
                   if(isset($venue['city']) && !empty($venue['city'])){
                       $checkVenue = $checkVenue->where("city","like","%".$venue['city']."%"); 
                   }
                   if(isset($venue['country']) && !empty($venue['country'])){
                       $checkVenue = $checkVenue->where("country", $venue['country']); 
                   }
                   
                   $checkVenue = $checkVenue->first();
                   //$checkVenue= $checkVenue->where('country',$event['_embedded']['venues'][0]['country']['countryCode']);
                   if(!$checkVenue){
                       $venue=Venues::create($venue);
                       $venue_id = $venue->id;
                   }else{
                       $venue_id = $checkVenue->id;
                   }
                  
                  //check for venue address
                  //$checkVenueAddres = VenueAdditionalInfo::where('venue_id',$venue_id);
                  $venue_address_id=NULL;
                  $checkVenueAddres = new VenueAdditionalInfo();
                  $venueAddress = [];
                  $venueAddress['venue_id'] = $venue_id;
                   $address = [];
                   if(isset($event['_embedded']['venues'][0]['address']['line1']) && !empty($event['_embedded']['venues'][0]['address']['line1'])){
                       $address_line = $event['_embedded']['venues'][0]['address']['line1'];
                       $address = explode(',',trim($address_line));
                   
                       if(!empty($address)){
                           //$checkVenue = $checkVenue->where('address_line_1','like','%'.$address[0].'%');
                           $checkVenueAddres = $checkVenueAddres->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $address[0]);
                           $venueAddress['address_line_1'] = $address[0];
                           if(isset($address[1]) && !empty($address[1])){
                                $checkVenueAddres = $checkVenueAddres->where('address_line_2','like','%'.trim($address[1]).'%');
                                $venueAddress['address_line_2'] = $address[1];
                           } 
                       }
                   }
                   
                   $postalCode = "";
                   //$postalCode = isset($event['_embedded']['venues'][0]['postalCode']) && !empty($event['_embedded']['venues'][0]['postalCode']) ? $event['_embedded']['venues'][0]['postalCode'] : '';
                   
                   if(isset($event['_embedded'])){
                        $postalCode = isset($event['_embedded']['venues'][0]['postalCode']) && !empty($event['_embedded']['venues'][0]['postalCode']) ? str_replace(' ','',$event['_embedded']['venues'][0]['postalCode']) : '';
                        $venueAddress['post_code'] = str_replace(' ','',$postalCode);
                   }
            
                       $checkVenueAddres = $checkVenueAddres->first();

                    if(!$checkVenueAddres){
                        
                        if(gettype($venueAddress) == "array"){
                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                            if(!$checkwithParam){
                                $venueAddress['source_id']  = $resource->id;
                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueadditionalinfo->id;
                            }else{
                                $venue_address_id = $checkwithParam->id;
                            }
                           
                        }
                    }else{
                        //check venue id same or not
                        if($checkVenueAddres->venue_id == $venue_id){
                            $checkVenueAddres->address_line_1 = isset($venueAddress['address_line_1']) && !empty($venueAddress['address_line_1']) ? $venueAddress['address_line_1'] : $checkVenueAddres->address_line_1;
                            $checkVenueAddres->address_line_2 = isset($venueAddress['address_line_2']) && !empty($venueAddress['address_line_2']) ? $venueAddress['address_line_2'] : $checkVenueAddres->address_line_2;
                            $checkVenueAddres->post_code = isset($venueAddress['post_code']) && !empty($venueAddress['post_code']) ? str_replace(' ','',$venueAddress['post_code']) : $checkVenueAddres->post_code;
                            $checkVenueAddres->save();
                            $venue_address_id = $checkVenueAddres->id;
                        }else{
                            // if(!empty($venueAddress['post_code']) && $venueAddress['post_code'] != NULL){
                            //     if($checkVenueAddres->post_code != $venueAddress['post_code'])
                            //     {
                                    
                                    $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                    if(!$checkwithParam){
                                        $venueAddress['source_id']  = $resource->id;
                                        $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                        $venue_address_id = $venueadditionalinfo->id;
                                    }else{
                                         $venue_address_id = $checkwithParam->id;
                                    }
                            //     }else{
                            //         $venue_address_id = $checkVenueAddres->id;
                            //     }
                            // }
                        }
                        
                    }
                  /**** End Venue Section *****/
                  
                  
                $primaryCategory = isset($event['classifications'][0]['genre']['name']) && !empty($event['classifications'][0]['genre']['name']) ? $event['classifications'][0]['genre']['name'] : "No Category";
               
                if (false !== $key = array_search($primaryCategory, $this->categoriesArray)) {
                    $primaryCategory = $primaryCategory;
                }else{
                    //check in category table
                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$primaryCategory)->where('source_id',3)->first();
                    if($check){
                        $primaryCategory = $check->category->name;
                    }
                }
                
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryObj = new Categories();
                $categoryTree = $categoryObj->getParentCatgoryTree($primaryCategory);
                
                //get maps of venue
                
                $maps = VenueMaps::with('maps')->where('venue_id',$venue_id)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
                  
                  /***** Event Section *****/
                   //create versioning or new event
                  $localDate = isset($event['dates']['start']['localDate']) && !empty($event['dates']['start']['localDate']) ? $event['dates']['start']['localDate'] : "0000-00-00";
                  $localTime = isset($event['dates']['start']['localTime']) && !empty($event['dates']['start']['localTime']) ? $event['dates']['start']['localTime'] : "00:00:00";
                  $event_date = $localDate;
                  
                  $eventArray=[];
                  $eventArray['search_type'] = $primaryCategory;
                 // $eventArray['search_type'] = $event['classifications'][]['segment']['name'] ;
                  $eventArray['name'] = $event['name'];
                  $eventArray['external_id'] = $event['id'];
                  $eventArray['event_date'] = $localDate." ".$localTime;
                  $eventArray['url'] = $event['url'];
                  $eventArray['event_date_utc'] = isset($event['dates']['start']['dateTime']) && !empty($event['dates']['start']['dateTime']) ? $event['dates']['start']['dateTime'] : "0000-00-00 00:00:00";
                  
                  $eventArray['venue_id'] = $venue_id;
                  $eventArray['venue_address_id'] = $venue_address_id;
                  $eventArray['map_id'] = $map_id;
                  //1->first check for event external id
                  
                  $checkWithExternalId = Events::where('external_id',$event['id'])->first();
                  $event['name'] = $new_string = preg_replace('/[^a-zA-Z0-9 ]/m', ' ',$event['name']);
                  
                  
                  if(!$checkWithExternalId){
                      //check with event data 
                        $checkEvent = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $event['name'])
                                        // ->where('venue_id',$venue_id)
                                        ->whereBetween('event_date',[$event_date." 00:00:00",$event_date." 23:59:59"])
                                        ->first();
                        
                        if(!$checkEvent){
                            $eventArray['source_id'] = $resource->id;
                            $events = Events::create($eventArray);
                            echo "new event ". $events->id . "created."."\n";
                     
                        }else{
                            //if event exist then check with source id
                            if($checkEvent->source_id != $resource->id){
                                //check event is mapped or not
                                
                                if($checkEvent->mapped_status==1){
                                    if($checkEvent->name != $eventArray['name']){
                                        $checkEvent->prev_name=$eventArray['name'];
                                    }
                                    if($checkEvent->event_date != $eventArray['event_date']){
                                        $checkEvent->prev_event_date=$eventArray['event_date'];
                                    }
                                    // if($checkEvent->performer_1_id != $eventArray['performer_1_id']){
                                    //     $checkEvent->prev_performer_1_id=$eventArray['performer_1_id'];
                                    // }
                                    // if($checkEvent->performer_2_id != $eventArray['performer_2_id']){
                                    //     $checkEvent->prev_performer_2_id=$eventArray['performer_2_id'];
                                    // }
                                    if($checkEvent->venue_id != $eventArray['venue_id']){
                                        $checkEvent->prev_venue_id=$eventArray['venue_id'];
                                    }
                                    // if($checkEvent->venue_address_id != $eventArray['venue_address_id']){
                                    //     $checkEvent->prev_venue_address_id=$eventArray['venue_address_id'];
                                    // }
                                    $checkEvent->update_requires=1;
                                    $checkEvent->save(); 
                                }
                                
                                //create new event with parent event id 
                                /*
                                $checkEvent->current_version = 0;
                                $checkEvent->update_requires = 0;
                                $checkEvent->save();
                                */
                                
                                $eventArray['current_version'] = 0;
                                $eventArray['parent_event_id'] = $checkEvent->id;
                                $eventArray['source_id'] = $resource->id;
                                $events = Events::create($eventArray);
                                if($this->manual_event != 1){
                                    echo $events->id ." new event existing in resource 1 "."\n";
                                }
                                
                            }else{
                                //rare case that external id is not there and sources are matching
                                if($checkEvent->mapped_status==1){
                                    //check what is update
                                    $prevArray = $checkEvent->toArray();
                                    $prev_arr = [];
                                    foreach($eventArray as $key=>$val){
                                        if($key != "external_id" && $key != "search_type" && $key != "map_id" && $prevArray[$key] != $val){
                                            $eventUpdateFlag=1;
                                            //$eventArray['prev_'.$key] = $prevArray[$key];
                                            $prev_arr['prev_'.$key] = $val;
                                        }
                                        if($eventUpdateFlag==1){
                                            $prev_arr['update_requires'] =1;
                                        }
                                    }
                                    //create new events 
                                    if($eventUpdateFlag ==1){
                                        if(!empty($prev_arr)){
                                            Events::where('id',$checkEvent->id)->update($prev_arr);
                                        }
                                        //create new event
                                        $eventArray['current_version'] = 0;
                                        $eventArray['update_requires'] = 0;
                                        $eventArray['mapped_status'] = 0;
                                        $eventArray['parent_event_id'] = $checkEvent->id;
                                        $eventArray['source_id'] = $resource->id;
                                        $events =  Events::create($eventArray);
                                    }else{
                                        Events::where('id',$checkEvent->id)->update($eventArray);
                                        $events = $checkEvent;
                                    }
                                    
                                }else{
                                    Events::where('id',$checkEvent->id)->update($eventArray);
                                    $events = $checkEvent;
                                }
                                
                            }
                            $event_id = $events->id;
                        }
                  }else{
                        //check event is mapped or unmapped
                        if($checkWithExternalId->mapped_status==1){
                            //check for updates in fields
                            $prevArray = $checkWithExternalId->toArray();
                            $prev_arr = [];
                            foreach($eventArray as $key=>$val){
                                if($key != "external_id" && $key != "search_type" && $key != "map_id" && $prevArray[$key] != $val){
                                    $eventUpdateFlag=1;
                                    $prev_arr['prev_'.$key] = $val;
                                    $prev_arr['update_requires'] = 1;
                                   // $eventArray['prev_'.$key] = $prevArray[$key];
                                    //$log->info("Updated event's ".$key." to: ".$value['id']);
                                }
                            }
                            if($eventUpdateFlag==1){
                                
                                if(!empty($prev_arr)){
                                    Events::where('id',$checkWithExternalId->id)->update($prev_arr);
                                }
                                //create new event
                                $eventArray['current_version']=0;
                                $eventArray['update_requires']=0;
                                $eventArray['mapped_status']=0;
                                $eventArray['parent_event_id']=$checkWithExternalId->id;
                                $eventArray['source_id'] = $resource->id;
                                $events = Events::create($eventArray);
                                
                                //$event_arr['update_requires']=1;
                                //$events =   Events::where('id',$checkExternalId->id)->update($eventArray);
                            }else{
                               $events =  Events::where('id',$checkExternalId->id)->update($eventArray);
                            }
                            $event_id = $checkWithExternalId->id;
                        }else{
                           $events =  Events::where('id',$checkWithExternalId->id)->update($eventArray);
                        }
                        
                        $events = $checkWithExternalId;
                  }
                      
                
                  /***** End Event Section *****/
                  
                  /***** Event Category Section *****/
                  $checkEventCategory = EventCategory::where('event_id',$events->id)->first();
                  
                  $venueCategory = VenueCategory::select('id')->where('venue_id',$venue_id)->get()->toArray();
                  $venueCategories = [];
                  $venueSection = [];   
                  if(!empty($venueCateogry)){
                      foreach($venueCategory as $cat){
                          $venueCategories[] =$cat;
                          $section = VenueSection::select('id')->where('category_id',$cat)->get()->toArray();
                          if(!empty($section)){
                              foreach($section as $s){
                                  $venueSection[]=$s;
                              }
                          }
                      }
                  }
                
                // $primaryCategory = isset($event['classifications'][0]['segment']['name']) && !empty($event['classifications'][0]['segment']['name']) ? $event['classifications'][0]['segment']['name'] : "No Category";
                
                
                  
                  $venueCategories = implode(',',$venueCategories);
                  $venueSection = implode(',',$venueSection);
                  
                  if(!$checkEventCategory){
                      $eventCategory = [
                                        'primary_category'=>$primaryCategory,
                                        // 'secondary_category'=>$secondaryCategory,
                                        'event_id' =>$events->id, 
                                        'venue_id' => $venue_id,
                                        'venue_address_id' =>$venue_address_id,
                                        'event_category'=>$venueCategories,
                                        'event_section'=>$venueSection,
                                        'source_id' =>$resource->id,
                                       ];
                      EventCategory::create($eventCategory);
                  }
                  /***** End Event Category Section *****/
            }
            if($this->manual_event==1){
                return ['success'=>true]; 
            }
        }
    }
    
    public function fetchEventByExternalId($externalId){
        $curl = curl_init();
        
        //fetch all future event and find version from tixstock for 
        
        // $url = $this->URL.''.$this->URL_GET_EVENT;
        $url = $this->URL.''.$this->URL_GET_EVENT;
        $params = ['apikey'=>$this->API_KEY,'locale'=>"*",'marketId'=>implode(',',$this->EUROPE_MARKET_ID),'size'=>1,'id'=>$externalId];
        $url = $url .'?'.http_build_query($params);
        
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "GET",
          CURLOPT_HTTPHEADER => array(
            "cache-control: no-cache",
            "content-type: application/json",
            // "postman-token: 15455206-d06f-fcbc-fc93-757d82023564"
          ),
        ));
        
        $response = curl_exec($curl);
        
        $err = curl_error($curl);
        
        curl_close($curl);
        
        if ($err) {
          echo "cURL Error #:" . $err;
        } else {
            
            $response = json_decode($response,true);
            return $response;
            if(empty($response)){
                 echo "No Response Found.";
                 exit;
            }
            if(isset($response['errors']) && !empty($response['errors'])){
                 echo json_encode($response['errors']);
                 $log = Log::channel('cron');
                 $log->info(json_encode($response['errors']));
                 exit;
            }
             
            if($this->totalPages == 0){
                $this->totalPages = $response['page']['totalPages'];
            }
            
            $this->processEvent($response);
            
            if( $this->totalPages != $this->page){
                  $this->page = $this->page+1;
                  echo $this->page."\r\n";;
                  $this->fetchEvent();
            }
        }
    }
}
