<?php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DataTables;
use App\AuditEvents;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Performers;
use App\Events;
use App\TixstockLog;
use App\Categories;
use App\Venues;
use App\VenueAdditionalInfo;
use App\Sources; 
use App\EventPerformers;
use App\Exports\EventsExport;
use App\VenueMaps;
use App\Http\Controllers\API\StubhubController as StubhubApi;
use App\Http\Controllers\API\TixstockController as TixstockApi;
use App\Http\Controllers\API\TicketmasterController as TicketMasterApi;
use App\Http\Controllers\API\TicketEvolutionController as TivoApi;
use App\Traits\StubhubTrait;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Http;
use Carbon\Carbon;
use DB;
use App\Maps;
use Illuminate\Pagination\LengthAwarePaginator;
use Illuminate\Support\Facades\Storage;
// use App\CustomLengthAwarePaginator;
class EventsController extends Controller
{
    use StubhubTrait;
    public $parentEventIds = [];
    public $categoriesArray = [];
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $category="";
        $updatedEvents = "";
        $mappedStatus = "";
        $searchPerformer="";
        if(isset($request->performer) && !empty($request->performer)){
          $searchPerformer = $request->performer;  
        }
       if(isset($request->category) && !empty($request->category)){
           $category = $request->category;
       }
       if(isset($request->mappedStatus) && !empty($request->mappedStatus)){
           $mappedStatus = $request->mappedStatus;
       }
       if(isset($request->updatedEvents) && !empty($request->updatedEvents)){
           $updatedEvents = "update_events_from_stubhub";
       }
        $performers=Performers::where('name','!=','')->groupBy('name')->get();
        $searchType=DB::table('event_category')
                        ->join('events','events.id', '=', 'event_category.event_id')
                        ->distinct()
                        ->where('primary_category','!=','')
                        ->where('events.deleted_at','=',NULL)
                        ->orderBy('primary_category')
                        ->get(['primary_category']);       
        $categories = Categories::orderBy('name','asc')->get();
        $resource = Sources::get();
    
        $pageTitle = "View Events";
        
        $categoriesArray=[];
        
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        
        $mapsArray = Maps::where('status',1)->orderBy('name')->get();
        
        return view('admin.events.index',compact('performers','searchType','searchPerformer','categories','pageTitle','resource','category','mappedStatus','updatedEvents','categoriesArray','mapsArray'));
    }
    
    public function subCategories($categories){
        
        foreach($categories as $cat){
            
            $subCategories = Categories::where('parent_id',$cat->id)->get();
            if($subCategories->count() > 0){
                $this->subCategories($subCategories);
            }else{
                $this->categoriesArray[]=$cat->name;
            }
        }
    }

    public function data1(Request $request){
        
        ini_set('memory_limit','-1');
        $categoriesArray=[];
        $parentEventIds = Events::select('parent_event_id')->where('parent_event_id','!=',0)->get()->toArray();
        $parentEventIds = array_column($parentEventIds, 'parent_event_id');
        $eventDateFlag = 0;
        
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        
        $events= Events::with(['venue','venueAddress', 'prevperformers', 'performers', 'eventcategory','tixstocklog'=>function($query){
            return $query->select('event_id');
        },'eventVersion','source','latestVersion','mappedSources']);
        
        
        // if(empty($request->external_id) && empty($request->change_in_date) && empty($request->change_in_name) && empty($request->change_in_venue) && empty($request->change_in_performer)){
            
        // }
        if($request->update_requires==1){
                $events=$events->where('update_requires',1);
        }
        if(!empty($request->type)){
            $type=$request->type;
            $events=$events->whereHas('eventcategory',function($q) use ($type){
                $q->where('primary_category',$type);
            });
        }
        if($request->mappedStatus != '' && ($request->mappedStatus == 0 || $request->mappedStatus == 1)){
            $events=$events->where('mapped_status',$request->mappedStatus);
        }
          
        if(!empty($request->performer)){
           $performers = explode(',',$request->performer);
        //   $events =$events->where(function ($query) use ($performers) {
        //         $query->whereIn('performer_1_id', $performers)
        //               ->orWhereIn('performer_2_id', $performers);
        //     });
            $events = $events->whereHas('performers',function($query)use($performers){
                $query->whereIn('performer_id',$performers);
            });
        }
        if(!empty($request->event_date) && empty($request->external_id)){
                $eventDateFlag = 1;
                $event_date = trim($request->event_date);
                $event_date = explode('to',$event_date);
                $start_date = trim($event_date[0]);
                $end_date = trim($event_date[1]);
                $events->whereBetween('event_date',[$start_date.' 00:00:00',$end_date.' 23:59:59']);
           
        }
        
        if(!empty($request->event_created_date) && empty($request->external_id)){
                $eventDateFlag = 1;
                $event_date = trim($request->event_created_date);
                $event_date = explode('to',$event_date);
                $start_date = $event_date[0];
                $end_date = $event_date[1];
                $events = $events->whereBetween('created_at',[trim($start_date).' 00:00:00',trim($end_date).' 23:59:59']);
            
        }
        
        
        if(!empty($request->event_name)){
            // $events=$events->whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $request->event_name);
             $events = $events->where('name','like','%'.$request->event_name.'%');
        }
        if(!empty($request->external_id)){
            $externalId = explode(',',$request->external_id);

            $events=$events->whereIn('external_id',$externalId);
        }
        if(!empty($request->venue)){
            $venue = $request->venue;
            $events=$events->whereHas('venue', function($q) use ($venue){
                $q->where('name','like','%'.$venue.'%');
            });
        }
        if(!empty($request->venue_address)){
            $venueAddress = $request->venue_address;
            $events=$events->whereHas('venueAddress',function($q) use ($venueAddress){
                $q->where('address_line_1','like','%'.$venueAddress.'%');
            });
        }
        if(!empty($request->change_in_date)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('event_date', '!=', 'events.event_date');
            })->where('update_requires',1);
        }
        if(!empty($request->change_in_venue)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('venue_id', '!=', 'events.venue_id');
            })->where('update_requires',1);
        }
        if(!empty($request->change_in_name)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('name', '!=', 'events.name');
            })->where('update_requires',1);
        }
        
        if(!empty($request->change_in_performer)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereDoesntHave('performers', function ($subquery) {
                    $subquery->whereIn('performer_id', function ($innerSubquery) {
                    $innerSubquery->select('performer_id')
                        ->from('event_performers')
                        ->whereColumn('event_id', '=', 'events.id');
                    });
                });
            })->where('update_requires',1);
        }
        
        if(!empty($request->resource)){
            $events =  $events->whereIn('source_id',$request->resource);
        }
    
        if(!empty($request->has_multiple_version)){
            $events = $events->whereIn('id',array_unique($parentEventIds));
        }
        
        if($request->mapped_status != null || $request->mapped_status==0){
            $events = $events->where('current_version',1);
        }else{
            $events = $events->whereIn('current_version',[0,1]);
        }
        
        if($request->unmapped_with_tixstocklog==1){
            $events =$events->whereHas('tixstocklog')->where('mapped_status',0);
        }
        
        if($request->update_map_flag && $request->update_map_flag == 1){
            $events = $events->where('update_map_flag',1)->where('update_requires',1);
        }
        if($request->has_map_flag && $request->has_map_flag == 1){
            $events = $events->where('map_id','!=','');
        }
        
        if(empty($request->type) && $request->mapped_categories_filter==0){
            $events=$events->whereHas('eventcategory',function($q) use ($categoriesArray){
                $q->whereNotIn('primary_category',$categoriesArray);
            });
        }
        
        if(isset($request->mapped_resource) && !empty($request->mapped_resource)){
            $sourcesArray = $request->mapped_resource;
            $events = $events->whereHas('mappedSources',function($q) use ($sourcesArray){
                $q->whereIn('source_id',$sourcesArray);
            });
        }
        
        // $events = $events->where('event_date','>=',date('Y-m-d'));
        // $events = $events->orderBy('event_date');
        
        if($eventDateFlag == 0){
            $events = $events->where('event_date','>=',Date('Y-m-d'));
        }
        
        
        // dd($events->toSql());

        $this->parentEventIds = $parentEventIds;
        $this->categoriesArray= $categoriesArray;
        
        return Datatables::eloquent($events,$parentEventIds)
        ->rawColumns(['name','external_id','category','event_date','venue','venue_address_id','performers','event_venue_details','actions','created_at','search_type','source_name','map_id','mapped_status'])
        
        ->editColumn('map_id',function($events){
            if(!empty($events->map_id)){
                
                $mapName = $events->maps->name;
                return '<a target="_blank" href="'.$events->maps->url.'">'.$mapName.'</a>';
            }else{
                return '<span class="text-danger">NA</span>';
            }
        })
        ->editColumn('created_at',function($events){
        
            return date('Y-m-d h:i:s',strtotime($events->created_at));
        })
        
        ->addColumn('source_name', function ($events){
            if($events->source_id==0){
                return "Manual";
            }
            return $events->source->name;
        })
        
        ->editColumn('external_id', function ($events) {
            return $events->external_id;
        })
        
        ->editColumn('search_type', function ($events) {
            if($events->eventcategory){
                if(!empty($events->eventcategory->primary_category)){
                    
                    $class="";
                    if(!in_array($events->eventcategory->primary_category,$this->categoriesArray)){
                        $class="text-danger";
                    }
                    return '<span class="'.$class.'">'.$events->eventcategory->primary_category.'</span><a href="javascript:void(0);" search_type = "'.$events->eventcategory->primary_category.'"  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                }else{
                    
                    return '<a href="javascript:void(0);" search_type = ""  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                }
            }else{
                return '<a href="javascript:void(0);" search_type = ""  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
            }
        })
        
        ->editColumn('name',function($events){
            $name = $events->name.'<a class="ms-2" href="https://stubhub.ie/'.$events->url.'" target="_blank" ><i class="mdi mdi-link-variant font-18 text-success me-1 align-middle"></i></a>';
            $name .= '<div class="url_div show_url">Url: '.$events->url.'</div>';
            if(!empty($events->prev_name) && ($events->latestVersion) && $events->latestVersion->name != $events->name && $events->update_requires == 1){
                return $name.'<hr>'.'<p class="text-danger">'.$events->latestVersion->name.'</p>';
            }
            return $name;
        })
        
        ->editColumn('category',function($events){
            $category_html = '';
            if(empty($category_html)){
                $category_html = '<button type="button" original_name="" search_type="'.$events->search_type.'" category_name="" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-blue waves-effect waves-light sub_category_btn" title="Add Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-arrow="true" ><i class="mdi mdi-plus"></i></button>';
            }
            return $category_html;
        })
        
        ->editColumn('event_date',function($events){
            if($events && isset($events->event_date) && !empty($events->event_date)){
                if(!empty($events->prev_event_date) && ($events->latestVersion) && ($events->latestVersion->event_date != $events->event_date) && $events->update_requires==1){
                    return $events->event_date .'<hr>'.'<p class="text-danger">'.$events->latestVersion->event_date.'</p>';
                }else{
                     return $events->event_date;
                }
            }else{
                return '';
            }
            
        })
        
        ->editColumn('venue_address_id',function($events){
            $venue_address_html = "";
            
            if(!empty($events->venue_address_id)){
                if($events->venueAddress){
                    $venue_address_html = $events->venueAddress->address_line_1." ";
                    if($events->venue){
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }
                }
            }else{
                if($events->venue){
                    if($events->venue->venueAdditionalInfo->isNotEmpty()){
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        $venue_address_html = $events->venue->venueAdditionalInfo[0]->address_line_1." ";
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }else{
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        // $venue_address_html = $events->venue->venueAdditionalInfo[0]->address_line_1;
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }
                }
            }
            return $venue_address_html;
        })
        
        ->addColumn('venue',function($events){
        
            if(!empty($events->latestVersion) && $events->venue_id != $events->latestVersion->venue_id && $events->update_requires ==1){
                $venueName = isset($events->venue->name) && !empty($events->venue->name) ? $events->venue->name : '';
                $prevVenueName = isset($events->latestVersion->venue->name) && !empty($events->latestVersion->venue->name) ? $events->latestVersion->venue->name : '';
                return $venueName.'<hr>'.'<p class="text-danger">'.$prevVenueName.'</p>';
            }else{
                if($events->venue){
                    return isset($events->venue->name) && !empty($events->venue->name) ? $events->venue->name : '';
                }else{
                    return "";
                }
            }
        })
        
        ->addColumn('performers',function($events){
            $performers_html = '';
            $performer='';
            $performerId=[];
            $eventPerformers=[];
            if($events){
                if( $events->performers->count() > 0){
                    foreach($events->performers as $performers){
                        if(empty($performer)){
                            $performer = $performers->name;
                        }else{
                            $performer .= ','. $performers->name;
                        }
                        $performerId[]=$performers->id;
                        $eventPerformers[] = $performers->id;
                    }
                }
            }
            // if(!empty($events->performer1)){
            //     $performer = isset($events->performer1->name) && !empty($events->performer1->name) ? $events->performer1->name : '';
            //     $performerId[]=$events->performer_1_id;
            // }
            
            // if(!empty($events->performer2) && $events->performer2 != null){
            //     $performer.=isset($events->performer2->name) && !empty($events->performer2->name) ? ','.$events->performer2->name : '';
            //     $performerId[]=$events->performer_2_id;
            // }
                $performerId=implode(',',$performerId);
           
            if(!empty($performer)){
                $performer_arr = array();
                
                $performers_html .= '<ul class="perfomer_ul">';
                
                $get_performers = array_filter(explode(',',$performer));
                
                foreach($get_performers as $get_performer){
                    $performers_html .= '<li>'.$get_performer.'</li>';
                    $performer_arr[] = $get_performer;
                }
                
                $performers_html .= '<a href="javascript:void(0);" performer_name = "'.implode(',',$performer_arr).'" performer_id="'.$performerId.'" event_name="'.$events->name.'" id="'.$events->id.'" class="action-icon edit_performer" style="color: green;" title="Edit Performer" data-original-title="Edit Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                $performers_html .= '</ul>';
            }
            else{
                $performers_html = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-danger btn-lg waves-effect waves-light performer_btn" title="Add Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-arrow="true" ><i class="mdi mdi-plus"></i></button>';
            }
            
            $prev_performers_html = '';
            $prev_performer='';
            $latestPerformer = [];
            if(!empty($events->latestVersion) && $events->latestVersion->performers->count() > 0){
                
                foreach($events->latestVersion->performers as $performers){
                    if(empty($prev_performer)){
                        $prev_performer = $performers->name;
                    }else{
                        $prev_performer .= ','. $performers->name;
                    }
                    $latestPerformer[] = $performers->id;
                }
                
                if(!empty($prev_performer)){
                $performer_arr = array();
                
                $performers_html .= '<hr><ul class="perfomer_ul">';
                
                $get_performers = array_filter(explode(',',$prev_performer));
                
                foreach($get_performers as $get_performer){
                    $prev_performers_html .= '<li class="text-danger">'.$get_performer.'</li>';
                }
                
                // $performers_html .= '<a href="javascript:void(0);" performer_name = "'.implode(',',$performer_arr).'" performer_id="'.$performerId.'" event_name="'.$events->name.'" id="'.$events->id.'" class="action-icon edit_performer" style="color: green;" title="Edit Performer" data-original-title="Edit Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                $prev_performers_html .= '</ul>';
            }
                
            }
            
            if($events->update_requires == 1 && $eventPerformers!=$latestPerformer){
                return $performers_html.''.$prev_performers_html;
            }else{
                return $performers_html;
            }
            
        })
        
        ->addColumn('event_venue_details',function($events){
            if(empty($events->eventcategory->event_categories)){
                $event_venue_details_btn = '<span class="text text-danger">NA</span>';
                $event_venue_details_btn.= '<br><a type="button" event_name="'.$events->name.'" id="'.$events->id.'" href= "#" style="color: green;" class="action-icon event_venue_detail_btn"><i class="mdi mdi-pencil"></i></button>';
            }else{
                $event_venue_details_btn = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-lg btn-primary waves-effect waves-light event_venue_detail_btn" dir="ltr" data-style="expand-right" title="View Detail" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            }
            return $event_venue_details_btn;
        })
        
        ->editColumn('mapped_status',function($events){
            
            if($events->mapped_status==0){
                return "UnMapped";
            }
            
            if($events->mapped_status==1){
                if($events->mappedSources->count() > 0){
                    $sourcesHtml = "";
                    foreach($events->mappedSources as $sources){
                        $sourcesHtml.= '<p class="text-success">'.$sources->sources->name.'</p>';
                    }
                    return $sourcesHtml;
                }else{
                    return "Mapped";
                }
            }
        })
        
        ->editColumn('actions',function($events){
            $action = "";
            $tixstlog = "";
            $delete= "";
            $version = "";
            $version_source = "";
          
            if($events->tixstocklog->count() > 0){
                $tixstlog = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class=" btn btn-md btn-danger waves-effect waves-light tixstock_log" dir="ltr" data-style="expand-right" title="Tixstock Log" data-original-title="Tixstock log details" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            }
            if( in_array($events->id,$this->parentEventIds)){
                $parent_event_id = $events->parent_event_id;
                if($events->parent_event_id == 0){
                    $parent_event_id = $events->id;
                }
                $version_source =  '<button style="margin-top:5px" type="button" event_id = "'.$events->id.'" id="'. $parent_event_id.'"  data-src="'.route('get.event.source.versions').'" class=" waves-effect btn-primary waves-light btn btn-md bg-red btn-flat event_source_version" dir="ltr" data-style="expand-right" title="Event Version" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-solid fa-code-branch"></i></button>';
            }
            $delete = '<button style="margin-top:5px" type="button" id="'. $events->id.'"  data-src="'.route('event.destroy', $events->id).'" class=" waves-effect btn-primary waves-light btn btn-md bg-red btn-flat delete" dir="ltr" data-style="expand-right" title="Discard Event" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-trash"></i></button>';
            $view = '<a href="'.route('event.show', $events->id).'" target="_blank" style="margin-top:5px" type="button" id="'. $events->id.'"  data-src="'.route('event.show', $events->id).'" class=" waves-effect btn-warning waves-light btn btn-md bg-red btn-flat" dir="ltr" data-style="expand-right" title="View Event" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-eye"></i></a>';
            $action = $tixstlog .' '.$view.' '.$version.' '.$version_source.' '.$delete;
            return $action;
        })
        
        ->setRowClass(function($events){
            
            return $events->update_requires  == 1 ? 'alert-danger' : '';
        })
        
        ->make(true);
    
    }
    
    public function data(Request $request){
        
        $categoriesArray=[];
        $parentEventIds = Events::select('parent_event_id')->where('parent_event_id','!=',0)->get()->toArray();
        $parentEventIds = array_column($parentEventIds, 'parent_event_id');
        $eventDateFlag = 0;
        
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        
        $index = Events::search($request->input('event_name'),[
            'filters'=>['current_version'=>1,'unmapped_status'=>$request->mappedStatus]]);
        // dd($index->get());
        // Apply filters
        $filters = [];

        // Filter by update_requires
        if ($request->update_requires == 1) {
            $filters[] = 'update_requires:1';
        }
        
          if($request->mappedStatus != '' && ($request->mappedStatus == 0 || $request->mappedStatus == 1)){
            // $events=$events->where('mapped_status',$request->mappedStatus);
            $filters[] = 'mapped_status:'.$request->mappedStatus;
        }

        // Filter by performer
        if (!empty($request->performer)) {
            $performers = explode(',', $request->performer);
            $filters[] = 'performers.performer_id:(' . implode(' OR ', $performers) . ')';
        }

        // Filter by event date
        if (!empty($request->event_date) && empty($request->external_id)) {
            $event_date = explode('to', $request->event_date);
            $start_date = trim($event_date[0]) . ' 00:00:00';
            $end_date = trim($event_date[1]) . ' 23:59:59';
            $filters[] = 'event_date:[' . $start_date . ' TO ' . $end_date . ']';
        }

        // Filter by created date
        if (!empty($request->event_created_date) && empty($request->external_id)) {
            $event_date = explode('to', $request->event_created_date);
            $start_date = trim($event_date[0]) . ' 00:00:00';
            $end_date = trim($event_date[1]) . ' 23:59:59';
            $filters[] = 'created_at:[' . $start_date . ' TO ' . $end_date . ']';
        }
     
        // Add more filters as needed...
      
        // Apply filters to the search query
        foreach ($filters as $filter) {
            $index->raw($filter);
        }
        $filters[] = 'current_version:1';
        
        $page = ($request->start / $request->length)+1;
        
        $perPage = $request->length; // Adjust the pagination size as needed
       
        // $results = $index->get();
    
        // $results = $index->paginate($perPage, 'page', $page);
        
        
        // Convert paginated results to array
        $totalFilteredEvents = $results->total();
        dd($results);
        $eventIds = $results->pluck('id')->toArray();
        
        // $events = $results;
        $events= Events::with(['venue','venueAddress', 'prevperformers', 'performers', 'eventcategory','tixstocklog'=>function($query){
            return $query->select('event_id');
        },'eventVersion','source','latestVersion','mappedSources'])->where('current_version',1)->whereIn('id', $eventIds)->get();
        

        // Get total count of filtered events
         
       
        // Get search results
        
      
        // $events = $events->get();
        $this->parentEventIds = $parentEventIds;
        $this->categoriesArray= $categoriesArray;
        
        
        
        // Pass data to Yajra Datatables
        // return DataTables::of($results)
   
       
        return Datatables::of($events,$parentEventIds)
        ->rawColumns(['name','external_id','category','event_date','venue','venue_address_id','performers','event_venue_details','actions','created_at','search_type','source_name','map_id','mapped_status'])
        
        ->editColumn('map_id',function($events){
            if(!empty($events->map_id)){
                
                $mapName = $events->maps->name;
                return '<a target="_blank" href="'.$events->maps->url.'">'.$mapName.'</a>';
            }else{
                return '<span class="text-danger">NA</span>';
            }
        })
        ->editColumn('created_at',function($events){
        
            return date('Y-m-d h:i:s',strtotime($events->created_at));
        })
        
        ->addColumn('source_name', function ($events){
            if($events->source_id==0){
                return "Manual";
            }
            return $events->source->name;
        })
        
        ->editColumn('external_id', function ($events) {
            return $events->external_id;
        })
        
        ->editColumn('search_type', function ($events) {
            if($events->eventcategory){
                if(!empty($events->eventcategory->primary_category)){
                    
                    $class="";
                    if(!in_array($events->eventcategory->primary_category,$this->categoriesArray)){
                        $class="text-danger";
                    }
                    return '<span class="'.$class.'">'.$events->eventcategory->primary_category.'</span><a href="javascript:void(0);" search_type = "'.$events->eventcategory->primary_category.'"  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                }else{
                    
                    return '<a href="javascript:void(0);" search_type = ""  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                }
            }else{
                return '<a href="javascript:void(0);" search_type = ""  id="'.$events->id.'" class="action-icon edit_search_type" style="color: green;" title="Edit Category" data-original-title="Edit Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
            }
        })
        
        ->editColumn('name',function($events){
            $name = $events->name.'<a class="ms-2" href="https://stubhub.ie/'.$events->url.'" target="_blank" ><i class="mdi mdi-link-variant font-18 text-success me-1 align-middle"></i></a>';
            $name .= '<div class="url_div show_url">Url: '.$events->url.'</div>';
            if(!empty($events->prev_name) && ($events->latestVersion) && $events->latestVersion->name != $events->name && $events->update_requires == 1){
                return $name.'<hr>'.'<p class="text-danger">'.$events->latestVersion->name.'</p>';
            }
            return $name;
        })
        
        ->editColumn('category',function($events){
            $category_html = '';
            if(empty($category_html)){
                $category_html = '<button type="button" original_name="" search_type="'.$events->search_type.'" category_name="" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-blue waves-effect waves-light sub_category_btn" title="Add Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-arrow="true" ><i class="mdi mdi-plus"></i></button>';
            }
            return $category_html;
        })
        
        ->editColumn('event_date',function($events){
            if($events && isset($events->event_date) && !empty($events->event_date)){
                if(!empty($events->prev_event_date) && ($events->latestVersion) && ($events->latestVersion->event_date != $events->event_date) && $events->update_requires==1){
                    return $events->event_date .'<hr>'.'<p class="text-danger">'.$events->latestVersion->event_date.'</p>';
                }else{
                     return $events->event_date;
                }
            }else{
                return '';
            }
            
        })
        
        ->editColumn('venue_address_id',function($events){
            $venue_address_html = "";
            
            if(!empty($events->venue_address_id)){
                if($events->venueAddress){
                    $venue_address_html = $events->venueAddress->address_line_1." ";
                    if($events->venue){
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }
                }
            }else{
                if($events->venue){
                    if($events->venue->venueAdditionalInfo->isNotEmpty()){
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        $venue_address_html = $events->venue->venueAdditionalInfo[0]->address_line_1." ";
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }else{
                        $city = isset($events->venue->city) && !empty($events->venue->city) ? $events->venue->city : '';
                        $state = isset($events->venue->state) && !empty($events->venue->state) ? $events->venue->state : '';
                        $post_code = isset($events->venue->post_code) && !empty($events->venue->post_code) ? $events->venue->post_code : '';
                        $country = isset($events->venue->country) && !empty($events->venue->country) ? $events->venue->country : '';
                        // $venue_address_html = $events->venue->venueAdditionalInfo[0]->address_line_1;
                        $venue_address_html.= $city.' '.$state.' '.$post_code.' '.$country;
                    }
                }
            }
            return $venue_address_html;
        })
        
        ->addColumn('venue',function($events){
        
            if(!empty($events->latestVersion) && $events->venue_id != $events->latestVersion->venue_id && $events->update_requires ==1){
                $venueName = isset($events->venue->name) && !empty($events->venue->name) ? $events->venue->name : '';
                $prevVenueName = isset($events->latestVersion->venue->name) && !empty($events->latestVersion->venue->name) ? $events->latestVersion->venue->name : '';
                return $venueName.'<hr>'.'<p class="text-danger">'.$prevVenueName.'</p>';
            }else{
                if($events->venue){
                    return isset($events->venue->name) && !empty($events->venue->name) ? $events->venue->name : '';
                }else{
                    return "";
                }
            }
        })
        
        ->addColumn('performers',function($events){
            $performers_html = '';
            $performer='';
            $performerId=[];
            $eventPerformers=[];
            if($events){
                if( $events->performers->count() > 0){
                    foreach($events->performers as $performers){
                        if(empty($performer)){
                            $performer = $performers->name;
                        }else{
                            $performer .= ','. $performers->name;
                        }
                        $performerId[]=$performers->id;
                        $eventPerformers[] = $performers->id;
                    }
                }
            }
            // if(!empty($events->performer1)){
            //     $performer = isset($events->performer1->name) && !empty($events->performer1->name) ? $events->performer1->name : '';
            //     $performerId[]=$events->performer_1_id;
            // }
            
            // if(!empty($events->performer2) && $events->performer2 != null){
            //     $performer.=isset($events->performer2->name) && !empty($events->performer2->name) ? ','.$events->performer2->name : '';
            //     $performerId[]=$events->performer_2_id;
            // }
                $performerId=implode(',',$performerId);
           
            if(!empty($performer)){
                $performer_arr = array();
                
                $performers_html .= '<ul class="perfomer_ul">';
                
                $get_performers = array_filter(explode(',',$performer));
                
                foreach($get_performers as $get_performer){
                    $performers_html .= '<li>'.$get_performer.'</li>';
                    $performer_arr[] = $get_performer;
                }
                
                $performers_html .= '<a href="javascript:void(0);" performer_name = "'.implode(',',$performer_arr).'" performer_id="'.$performerId.'" event_name="'.$events->name.'" id="'.$events->id.'" class="action-icon edit_performer" style="color: green;" title="Edit Performer" data-original-title="Edit Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                $performers_html .= '</ul>';
            }
            else{
                $performers_html = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-danger btn-lg waves-effect waves-light performer_btn" title="Add Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-arrow="true" ><i class="mdi mdi-plus"></i></button>';
            }
            
            $prev_performers_html = '';
            $prev_performer='';
            $latestPerformer = [];
            if(!empty($events->latestVersion) && $events->latestVersion->performers->count() > 0){
                
                foreach($events->latestVersion->performers as $performers){
                    if(empty($prev_performer)){
                        $prev_performer = $performers->name;
                    }else{
                        $prev_performer .= ','. $performers->name;
                    }
                    $latestPerformer[] = $performers->id;
                }
                
                if(!empty($prev_performer)){
                $performer_arr = array();
                
                $performers_html .= '<hr><ul class="perfomer_ul">';
                
                $get_performers = array_filter(explode(',',$prev_performer));
                
                foreach($get_performers as $get_performer){
                    $prev_performers_html .= '<li class="text-danger">'.$get_performer.'</li>';
                }
                
                // $performers_html .= '<a href="javascript:void(0);" performer_name = "'.implode(',',$performer_arr).'" performer_id="'.$performerId.'" event_name="'.$events->name.'" id="'.$events->id.'" class="action-icon edit_performer" style="color: green;" title="Edit Performer" data-original-title="Edit Performer" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" > <i class="mdi mdi-pencil"></i></a>';
                $prev_performers_html .= '</ul>';
            }
                
            }
            
            if($events->update_requires == 1 && $eventPerformers!=$latestPerformer){
                return $performers_html.''.$prev_performers_html;
            }else{
                return $performers_html;
            }
            
        })
        
        ->addColumn('event_venue_details',function($events){
            if(empty($events->eventcategory->event_categories)){
                $event_venue_details_btn = '<span class="text text-danger">NA</span>';
                $event_venue_details_btn.= '<br><a type="button" event_name="'.$events->name.'" id="'.$events->id.'" href= "#" style="color: green;" class="action-icon event_venue_detail_btn"><i class="mdi mdi-pencil"></i></button>';
            }else{
                $event_venue_details_btn = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class="btn btn-lg btn-primary waves-effect waves-light event_venue_detail_btn" dir="ltr" data-style="expand-right" title="View Detail" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            }
            return $event_venue_details_btn;
        })
        
        ->editColumn('mapped_status',function($events){
            
            if($events->mapped_status==0){
                return "UnMapped";
            }
            
            if($events->mapped_status==1){
                if($events->mappedSources->count() > 0){
                    $sourcesHtml = "";
                    foreach($events->mappedSources as $sources){
                        $sourcesHtml.= '<p class="text-success">'.$sources->sources->name.'</p>';
                    }
                    return $sourcesHtml;
                }else{
                    return "Mapped";
                }
            }
        })
        
        ->editColumn('actions',function($events){
            $action = "";
            $tixstlog = "";
            $delete= "";
            $version = "";
            $version_source = "";
          
            if($events->tixstocklog->count() > 0){
                $tixstlog = '<button type="button" event_name="'.$events->name.'" id="'.$events->id.'" class=" btn btn-md btn-danger waves-effect waves-light tixstock_log" dir="ltr" data-style="expand-right" title="Tixstock Log" data-original-title="Tixstock log details" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            }
            if( in_array($events->id,$this->parentEventIds)){
                $parent_event_id = $events->parent_event_id;
                if($events->parent_event_id == 0){
                    $parent_event_id = $events->id;
                }
                $version_source =  '<button style="margin-top:5px" type="button" event_id = "'.$events->id.'" id="'. $parent_event_id.'"  data-src="'.route('get.event.source.versions').'" class=" waves-effect btn-primary waves-light btn btn-md bg-red btn-flat event_source_version" dir="ltr" data-style="expand-right" title="Event Version" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-solid fa-code-branch"></i></button>';
            }
            $delete = '<button style="margin-top:5px" type="button" id="'. $events->id.'"  data-src="'.route('event.destroy', $events->id).'" class=" waves-effect btn-primary waves-light btn btn-md bg-red btn-flat delete" dir="ltr" data-style="expand-right" title="Discard Event" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-trash"></i></button>';
            $view = '<a href="'.route('event.show', $events->id).'" target="_blank" style="margin-top:5px" type="button" id="'. $events->id.'"  data-src="'.route('event.show', $events->id).'" class=" waves-effect btn-warning waves-light btn btn-md bg-red btn-flat" dir="ltr" data-style="expand-right" title="View Event" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy="" ><i class="fa fa-eye"></i></a>';
            $action = $tixstlog .' '.$view.' '.$version.' '.$version_source.' '.$delete;
            return $action;
        })
        
        ->setRowClass(function($events){
            
            return $events->update_requires  == 1 ? 'alert-danger' : '';
        })
        
        // ->setTotalRecords($results->total())
       
        ->make(true);
    }
    
    public function getTixstockLog(Request $request)
    {
        $tixstockLog = TixstockLog::with(['event','source'])->where('event_id',$request->id)->orderBy('created_at','desc')->get();
        $html=view('admin.events.ajax.tixstock_log',compact('tixstockLog'))->render();
       
        return response()->json(['error'=>0, 'errmsg'=>'','html'=>$html,'id'=>$request->id]);
        
    }

    public function fetchEventVenueDetails(Request $request){
        $event=Events::where('id',$request->id)->first();
        $eventCategory = EventCategory::where('event_id',$request->id)->first();
        
        $get_event_id_category = '';
        $get_event_id_section = '';
        
        $category_arr = array();
        $section_arr = array();
        $category_details_html = '';
        $section_details_html = '';
        $main_result_rows = array();
        $otherCategory= [];
        
        $get_event_id_category = $eventCategory->event_categories;
        $get_event_id_section = $eventCategory->event_section;
        
        if(!empty($get_event_id_category) && $get_event_id_category != NULL){
            // =>function($query) use ($get_event_id_section){
            //      return $query->whereIn('id',explode(',',$get_event_id_section));
            //  }
            
            // =>function($query) use ($get_event_id_section){
            //      return $query->whereNotIn('id',explode(',',$get_event_id_section));
            //  }
            $category = VenueCategory::with(['section'])->whereIn('id',explode(',',$get_event_id_category))->where('venue_id',$event->venue_id)->get();
            
            $otherCategory =VenueCategory::with(['section'])->whereNotIn('id',explode(',',$get_event_id_category))->where('venue_id',$event->venue_id)->get();
        }else{
            $category = VenueCategory::with('section')->where('venue_id',$eventCategory->venue_id)->get();
        }
        
       $tixstockCategory = VenueCategory::with('section')->where('venue_id',$eventCategory->venue_id)->where('source_id',0)->get();
       
       $venueMaps =VenueMaps::with('maps')->where('venue_id',$event->venue_id)->get();
       
       $venueId=$eventCategory->venue_id;
       
       $selectedMap = "";
       if(!empty($event->map_id)){
            $selectedMap = Maps::where('id',$event->map_id)->first();
       }
      
       $manual_event=1;
       $html = view('admin.events.ajax.event_venue_section',compact('event','category','tixstockCategory','venueId','otherCategory','manual_event','get_event_id_category','get_event_id_section','venueMaps','selectedMap'))->render();

        return response()->json(['error'=>0, 'errmsg'=>'','html'=>$html,'id'=>$request->id]);

    }

    public function eventsVenueCategoryUpdate(Request $request){

        $input=$request->all();
        
        $category=VenueCategory::where('id',$input['pk'])->first();
        $response=[];
        //check default version category exist or not
        $checkCategory=VenueCategory::where('venue_id',$category->venue_id)
                                    ->where('name',$request->name)
                                    ->where('source_id',0)
                                    ->where('venue_category',$category->venue_category)
                                    ->first();
        if(!$checkCategory){
            $venueCategory=['name'=>$input['value'],'venue_id'=>$category->id,'venue_category'=>$category->venue_category,'source_id'=>0];
            $defaultCategory=VenueCategory::create($venueCategory);
            $response=['success'=>true,'msg'=>'Category added successfully.'];
        }else{
            $checkCategory->name=$input['name'];
            $checkCategory->save();
            $response=['success'=>true,'msg'=>'Category updated successfully.'];
        }
        return response()->json($response);
    }

    public function eventsDataUpdate(Request $request){
        
        $input=$request->all();

        if($input['method']="performer_save"){
            //
            $performers = $input['performers'];
            $performers=array_filter($performers);
            if(!empty($performers)){
                foreach($performers as $performer){
                    $check = Performers::where('name',$performer)->first();
                    if(!$check){
                        Performers::create(['name'=>$performer,'name_variation'=>$performer]);
                    }
                }
            }
        }
    }

    public function eventsUpdateCategorySection(Request $request){
            
            
            $id=$request->id;
            $venueId = $request->venue_id;
            if(!isset($request->category)){
                $eventCategoryArray=[
                                  'event_categories'=>"",
                                  'event_section'=>""
                                ];
                EventCategory::where('event_id',$request->id)->update($eventCategoryArray);
            }else{
                $newlyAddCategory=array_filter($request->category);
                $newlyAddSection=array_filter($request->sections);
                $eventCategoryArray=[
                                      'event_categories'=>implode(',',$newlyAddCategory),
                                      'event_section'=>implode(',',$newlyAddSection)
                                    ];
                EventCategory::where('event_id',$request->id)->update($eventCategoryArray);
                $event = Events::where('id',$request->id)->first();
                if($event->mapped_status==1){
                    $event->update_requires = 1;
                    $event->save();
                }
                // if(!empty($newlyAddCategory)){
                //     foreach($newlyAddCategory as $key=>$category){
                //         if (is_numeric($key)) {
                //                 if(isset($newlyAddSection[$key]) && !empty($newlyAddSection[$key])){
                //                     foreach($newlyAddSection[$key] as $section){
                //                         if(!is_numeric($section)){
                //                             $secArray=['name'=>$section,'category_id'=>$key,'source_id'=>0];
                //                             $checkSection=VenueSection::where($secArray)->first();
                //                             if(!$checkSection){
                //                                 VenueSection::create($secArray);
                //                             }
                //                         }
                //                     }
                //                 }
                //         }
                //         else{
                //             //create category
                //             $categoryArray= explode('/',$category);
                //             $catArray=['name'=>isset($categoryArray[0]) && !empty($categoryArray[0]) ? $categoryArray[0] : '','venue_id'=>$venueId,'source_id'=>0,'stubhub_name'=>isset($categoryArray[0]) && !empty($categoryArray[1]) ? $categoryArray[1] : ''];
                //             $checkCategory=VenueCategory::where($catArray)->first();
                //             if(!$checkCategory){
                //                 $newCategory=VenueCategory::create($catArray);
                //                 if(!empty($newlyAddSection[$key])){
                //                     foreach($newlyAddSection[$key] as $section){
                //                         $sectionArray= explode('/',$section);
                //                         $secArray=['name'=>isset($sectionArray[0]) && !empty($sectionArray[0]) ? $sectionArray[0] : '','category_id'=>$newCategory->id,'source_id'=>0,'stubhub_name'=>isset($sectionArray[1]) && !empty($sectionArray[1]) ? $sectionArray[1] : ''];
                //                         $checkSection=VenueSection::where($secArray)->first();
                //                         if(!$checkSection){
                //                             VenueSection::create($secArray);
                //                         }   
                //                     }
                //                 } 
                //             } 
                //         }
                //     }
                // }
            }
            $response=['success'=>true,'msg'=>"Category & Section updated successfully."];
            return response()->json($response);
    }
    
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $categoriesArray=[];
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        $eventCategory = $categoriesArray;
        $venues = Venues::where('name','<>','')->distinct(['name'])->orderBy('id','desc')->get();
        return view('admin.events.create',compact('eventCategory','venues'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        
        //
        $rules = [
            'event_name'=>"required",
            'event_date'=>"required",
            'event_category'=>"required",
            'performers'=>"required",
            'venue'=>"required"
        ];
        $this->validate($request,$rules);

        $input = $request->all();

        $event_date_utc = $input['event_date'];
        date_default_timezone_set("UTC");
        $event_date_utc = date("Y-d-mTG:i:sz", strtotime($input['event_date']));
        
        
        $eventData=[
             'name'=>$input['event_name'],
             'event_date'=>date('Y-m-d',strtotime($input['event_date'])),
             
        ];
        $performers=[];
        
       
       
        
        //check same event exist or not
        $objEvents = new Events();
        $uuidNo = $objEvents->generateUUID();
        $eventExists = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", str_replace("-"," ",$eventData['name']))
                                        ->where('venue_id',$input['venue'])
                                        ->whereBetween('event_date',[$eventData['event_date']." 00:00:00",$eventData['event_date']." 23:59:59"])
                                        ->first();
      
        if(!$eventExists){
            
            $eventData=[
                'name'=>$input['event_name'],
                'search_type'=>$input['event_category'],
                'external_id'=>$uuidNo,
                'event_date'=>$input['event_date'],
                'venue_id'=>$input['venue'],
                'venue_address_id'=>isset($input['venue_address']) && !empty($input['venue_address']) ? $input['venue_address'] : NULL,
                'event_date_utc'=>date(DATE_ISO8601,strtotime($input['event_date'])),
                
            ];
            if(isset($input['performers']) && !empty($input['performers'])){
                if(sizeof($input['performers']) > 1 ){
                    $eventData['performer_1_id'] = $input['performers'][0];
                    $eventData['performer_2_id'] = $input['performers'][1];
                }else{
                    $eventData['performer_1_id'] = $input['performers'][0];
                }
            }
            $eventData['source_id'] = 0;
            $eventData['manual_event'] = 1;
       
            $events = Events::create($eventData);
            $category= [];
            $sections = [];
            if(isset($input['category']) && !empty($input['category'])){
                $category = implode(',',$input['category']);
            }
            if(isset($input['sections']) && !empty($input['sections'])){
                $sections = implode(',',$input['sections']);
            }
            $eventCategory = [
                                'primary_category'=>$input['event_category'],
                                'event_id'=>$events->id,
                                'event_categories'=>isset($category) && !empty($category) ? $category: '',
                                'event_section'=>isset($sections) && !empty($sections) ? $sections : '',
                                'venue_id'=>$input['venue'],
                                'source_id'=>0
                            ];
          
            EventCategory::create($eventCategory);
            
            foreach($input['performers'] as $performer){
                EventPerformers::create(['event_id'=>$events->id,'performer_id'=>$performer]);
            }
            return response()->json(['success'=>true,'msg'=>'Event Created Successfully']);
            
        }else{
            return response()->json(['success'=>false,'msg'=>'Event already exists']);
        }
        
    }


    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
        $event = Events::with(['venue','source','eventcategory','venue.category','venue.category.section' ,'performer1', 'performer2','tixstocklog'])->where('id',$id)->first();
        
        $eventPerformers = EventPerformers::select('performer_id')->where('event_id',$id)->get()->toArray();
        if(!empty($eventPerformers)){
            $eventPerformers = array_column($eventPerformers,'performer_id');
        }
        
        $venueAddress = [];
        $venues = Venues::where('name','<>','')->distinct(['name'])->orderBy('id','desc')->get();
        if(!empty($event->venue_id)){
            $venueAddress= VenueAdditionalInfo::where('venue_id',$event->venue->id)->get();
        }
        
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        
        $eventCategory= $categoriesArray;
        $performers = Performers::where('name',"!=","")->where('name','!=',NULL)->get();
          return view('admin.events.view',compact('event','venues','performers','eventCategory','venueAddress','eventPerformers'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        
      
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        
        $input = $request->all();
        $event_date_utc = date("Y-d-mTG:i:sz", strtotime($input['event_date']));
        $eventArray =[
                        'name' => $input['name'],
                        'external_id'=>$input['external_id'],
                        'event_date'=>$input['event_date'],
                        // 'event_date_utc'=>$event_date_utc,
                        'venue_id'=>$input['venue'],
                        'venue_address_id'=>$input['venue_address'],
                    ];
       
        $event = Events::where('id',$input['event_id'])->first();
        $prevArray = $event->toArray();
    
        $eventUpdateFlag=0;
        foreach($eventArray as $key=>$val){
            if(!empty($val) && $prevArray[$key] != $val){
                $eventUpdateFlag=1;
            }
        }
        
        if($eventUpdateFlag==1 && $event->mapped_status ==1){
            $eventArray['update_requires'] = 1;
        }
        $eventArray['event_date_utc']=$event_date_utc;
        
        Events::where('id',$input['event_id'])->update($eventArray);
        
        if(isset($input['performers']) && !empty($input['performers'])){
            Eventperformers::where('event_id',$input['event_id'])->delete();
            foreach($input['performers'] as $performer){
                EventPerformers::create(['event_id'=>$input['event_id'],'performer_id'=>$performer]);
            }
        }
        
        
        EventCategory::where('event_id',$input['event_id'])->update(['primary_category'=>$request->event_category]);
        return response()->json(['success'=>true,'msg'=>"Event Updated Successfully"]);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
        $parentEventIds = Events::select('parent_event_id')->where('parent_event_id',$id)->get()->toArray();
        $parentEventIds = array_column($parentEventIds, 'parent_event_id');
        // if(!empty($parentEventIds)){
        //     EventCategory::whereIn('event_id',$parentEventIds)->delete();
        // }
        Events::where('parent_event_id',$id)->delete();
        // EventCategory::where('event_id',$id)->delete();
        Events::where('id',$id)->delete();
        return response()->json(['success'=>true,'msg'=>'Event Deleted Successfully']);
    }
    
    public function deleteBulkEvents(Request $request){
        
        $id = explode(',',$request->id);
        Events::whereIn('id',$id)->delete();
        return response()->json(['success'=>true,'msg'=>'Event Deleted Successfully']);
    }

    public function eventsPerformersUpdate(Request $request){

        $input=$request->all();
        
        $request->performers = array_filter($request->performers);
        $request->performers = array_unique($request->performers);
        $request->merge(['performers'=>$request->performers]);
        
        $rules = [
            'performers'=> 'required|array|distinct|min:1|max:2',
        ];
        $this->validate($request,$rules);

        $performer_1_id="";
        $performer_2_id="";

        $input = $request->all();
        $performer_id = [];
        array_unique($input['performers']);
        $input['performers'] = array_values($input['performers']);
        //
        
        foreach($input['performers'] as $performer){
            $checkPerformer = Performers::where('name',$performer)->first();
            if($checkPerformer){
                $performer_id[] = $checkPerformer->id;
            }else{
                $performersArray=['name'=>$performer,'source_id'=>1];
                $newperformer=Performers::create($performersArray);
                $performer_id[] = $newperformer->id;
            }
        }
        
        if(!empty($performer_id)){
            EventPerformers::where('event_id',$input['id'])->delete();
            foreach($performer_id as $performer){
                EventPerformers::create(['event_id'=>$input['id'],'performer_id'=>$performer]);
            }
        }
        
        $event = Events::where('id',$input['id'])->first();
        if($event->mapped_status == 1 ){
            $event->update_requires = 1;
            $event->save();
        }
        
        
        //check performer 1 & performer 2
        // if(isset($input['performers'][0]) && !empty($input['performers'][0])){
        //     $checkPerformer1 = Performers::where('name',$input['performers'][0])->first();
        //     if($checkPerformer1){
        //         $performer_1_id = $checkPerformer1->id;
        //     }else{
        //         $performersArray=['name'=>$input['performers'][0],'source_id'=>1];
        //         $performers=Performers::create($performersArray);
        //         $performer_1_id=$performers->id;
        //     }
        // }

        // if(isset($input['performers'][1]) && !empty($input['performers'][1])){
        //     $checkPerformer2 = Performers::where('name',$input['performers'][1])->first();
        //     if($checkPerformer2){
        //         $performer_2_id = $checkPerformer2->id;
        //     }else{
        //         $performersArray=['name'=>$input['performers'][1],'source_id'=>1];
        //         $performers=Performers::create($performersArray);
        //         $performer_2_id=$performers->id;
        //     }
        // }
        
        // $eventsArray=[];
        // if(!empty($performer_1_id)){
        //     $eventsArray['performer_1_id']=$performer_1_id;
        // }
        // if(!empty($performer_2_id)){
        //     $eventsArray['performer_2_id']=$performer_2_id;
        // }
        // $event = Events::where('id',$input['id'])->first();
       
        // if($event->mapped_status == 1 ){
        //     $eventsArray['update_requires'] = 1;
        // }
        
        // Events::where('id',$input['id'])->update($eventsArray);

        $response = ['success'=>true,'msg'=>"Performer updated successfully."];
        return response()->json($response);

    }
    
    public function bulkUpdateEventPerformer(Request $request){
        $input = $request->all();
        $request->performer = array_filter($request->performer);
        $request->performer = array_unique($request->performer);
        $request->merge(['performer'=>$request->performer]);
        $rules = [
            'performer'=> 'required|array|distinct|min:1|max:2',
        ];
        $this->validate($request,$rules);
        
        $events = explode(',',$input['id']);
        foreach($events as $event){
            
            EventPerformers::where('event_id',$event)->delete();
            
            foreach($input['performer'] as $p){
                EventPerformers::create(['event_id'=>$event,'performer_id'=>$p]);
            }
            
            $event = Events::where('id',$event)->first();
            if($event->mapped_status == 1 ){
                $event->update_requires = 1;
                $event->save();
            } 
        }
        
        $response = ['success'=>true,'msg'=>"Performer updated successfully."];
        return response()->json($response);
    }

    public function eventsPerformersDelete(Request $request){
        EventPerformers::where('event_id',$request->event_id)->where('performer_id',$request->id)->delete();
        $checkMappedstatus = Events::where('id',$request->event_id)->first();
        if($checkMappedstatus->mapped_status == 1 && $checkMappedstatus->update_requires == 0){
            $checkMappedstatus->update_requires = 1;
            $checkMappedstatus->save();
        }
        return response()->json(['success'=>true,'msg'=>'Performers deleted successfully.']);
        /*
            $event=Events::where('id',$request->event_id)->first();
            if($event->performer_1_id==$request->id){
                $event->performer_1_id=null;
                $event->save();
            }
            if($event->performer_2_id==$request->id){
                $event->performer_2_id=null;
                $event->save();
            }
        */
        
    }
    
    public function getEventVersion(Request $request){
        $eventVersions =  Events::with(['venue', 'eventcategory','venue.category','venue.category.section' ,'performer1', 'performer2','tixstocklog','performers'])->where('current_version',0)->where('external_id',$request->id)->orderBy('created_at','desc')->get();
       
        $html=view('admin.events.ajax.event_version',compact('eventVersions'))->render();
       
        return response()->json(['error'=>0, 'errmsg'=>'','html'=>$html,'id'=>$request->id]);
    }
    
    public function setEventDefault(Request $request)
    {
        
        $id =$request->event_id;
        $other_event = $request->other_event;
        $external_id = $request->external_id;
        Events::where('id',$id)->update(['current_version'=>1]);
        Events::whereIn('id',explode(',',$other_event))->update(['current_version'=>0]);
        
        return response()->json(['success'=>true,'msg'=>'Event is set to default.']);
    }
    
    public function setEventPrimary(Request $request)
    { 
        $id =$request->event_id;
        Events::where('id',$id)->update(['current_version'=>1,'parent_event_id'=>0]);
        return response()->json(['success'=>true,'msg'=>'Event is set to Primary.']);
    }
    
    public function updateEventCategory(Request $request){
        
         $rules = [
           'category'=>'required'
        
        ];
        $this->validate($request, $rules);
        $ids = explode(',',$request->id);
        foreach($ids as $id){
            $updateArray =[];
            $event = Events::where('id',$id)->first();
            $updateArray['search_type'] = $request->category;
            if($event->mapped_status==1){
                $updateArray['update_requires']=1;
            }
            Events::where('id',$id)->update($updateArray);
            EventCategory::where('event_id',$id)->update(['primary_category'=>$request->category]);
        }
        
        return response()->json(['success'=>true,'msg'=>'Category Updated Succesfully.']);
    }
    
    /**
     * Display the page for fetching events from resource
     * 
    */
     
    public function fetchSingleEvent(){
        $resources = Sources::get();
        return view('admin.events.fetch_from_resource',compact('resources'));
    }
    
    /**
     * Fetch event from resource 
     * @param   object $request contains source name and external id of event
     * @return  json containing event data
    */
    public function fetchSingleEventSubmit(Request $request){
        $rules = [
            'source'=>'required',
            'external_id'=>'required'
         ];

         $this->validate($request,$rules);
        $resource = "";
         $response = "";
            //fetch event from stubhub  
            switch($request->source){
                case 'Stubhub':
                //fetch from stubhub
                $stubhubapi = new StubhubApi();
                $response = $stubhubapi->getEvent($request->external_id);
                $response = $response;
                $resource = "Stubhub";
                break;

                case 'Tixstock':
                //fetch from tixstock  
                $tixstockApi = new TixstockApi();
                $response = $tixstockApi->getEvent($request->external_id);
                $resource = "Tixstock";
                break;
                
                case 'Ticket Evolution':
                //fetch from TicketEvolution
                
                $tivoapi = new TivoApi();
                $response = $tivoapi->fetchEventByExternalId($request->external_id);
                $resource = "TicketEvolution";
                break;
                
                case 'TicketMaster':
                //fetch from ticketmaster
                $ticketmasterapi = new TicketMasterApi();
                $response = $ticketmasterapi->fetchEventByExternalId($request->external_id);
                $resource = "TicketMaster";
                
                
            }
            $result = ['success'=>true,'response'=>$response,'resource'=>$resource];
            return response()->json($result);
    }
    
    public function fetchedSingleEventSubmit(Request $request){
        $input = $request->all();
        $eventData = json_decode($input['event_data'],true);
        
        if($input['resource']!="TicketMaster" && $input['resource'] != "Ticket Evolution"){
            $event = $eventData[0];
        }else if($input['resource'] == "Ticket Evolution"){
            $event['events'][] = $eventData;
        }
        else{
            $event = $eventData;
        }
        
        $resources = Sources::where('name',$input['resource'])->first();
        $response = $this->processSingleEvent($resources,$event);
        return $response;
    }
    
    //version from events page if event has any version
    public function getSourceEventVersion(Request $request){
      
        $input = $request->all();
        // $eventCategory = EventCategory::groupBy('primary_category')->pluck('primary_category');
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        
        $eventCategory= $categoriesArray;
        $venues = Venues::where('name','<>','')->distinct(['name'])->orderBy('name')->get();
       
        $events = Events::with(['performer1','performer2','venue','venue.venueAdditionalInfo','performers','maps'])->where('parent_event_id',$input['id'])->orWhere('id',$input['id'])->groupBy(['source_id','mapped_status'])->orderBy('created_at','desc')->get();
        
        $performers = Performers::where('name',"!=","")->where('name','!=',NULL)->get();
       
        $default_event_id = $request->event_id;
        
        $html=view('admin.events.ajax.event_versions',compact('events','eventCategory','venues','default_event_id','performers'))->render();
       
        return response()->json(['error'=>0, 'errmsg'=>'','html'=>$html,'id'=>$request->id]);
        dd($event);
    }
    
    public function bulkEventsUpdate(Request $request){
        // dd($request->all());
        $input = $request->all();
        $events=explode(',',$input['event_id']);
        $bulkUpdateOption=explode(',',$input['update_option']);
        
        foreach($events as $eventId){
            $event = Events::with(['latestVersion'])->where('id',$eventId)->first();
            
            if(in_array('change_in_date',$bulkUpdateOption)){
                if(!empty($event->latestVersion) && ($event->latestVersion->event_date != $event->event_date)){
                    $event->event_date = $event->latestVersion->event_date;
                }
            }
            
            if(in_array('change_in_venue',$bulkUpdateOption)){
                if($event->prev_venue_id != NULL && !empty($event->prev_venue_id) ){
                    $event->venue_id = $event->latestVersion->venue_id;
                }
            }
            
            if(in_array('change_in_name',$bulkUpdateOption)){
                // if(!empty($event->prev_event_name) && $event->prev_event_name != NULL){
                if(!empty($event->latestVersion)){
                    $event->name = $event->latestVersion->event_name;
                }
            }
            
            if(in_array('change_in_performer',$bulkUpdateOption)){
                if(!empty($event->latestVersion)){
                    $latestPerformers=[];
                    if($event->latestVersion->performers->count() > 0){
                        foreach($event->latestVersion->performers as $performer){
                            $latestPerformers[] = $performer->id;
                        }
                    }
                    
                    $eventPerformers=[];
                    foreach($event->performers as $performer){
                        $eventPerformers[] = $performer->id;
                    }
                    
                    // if($eventPerformers != $latestPerformers){
                        EventPerformers::where('event_id',$event->id)->delete();
                        foreach($latestPerformers as $performer){
                            EventPerformers::create(['event_id'=>$event->id,'performer_id'=>$performer]);
                        }
                    // }
                }
                // if(!empty($event->prev_performer_1_id) && $event->prev_performer_1_id != NULL && $event->prev_performer_1_id != $event->performer_1_id ){
                //     $event->performer_1_id = $event->prev_performer_1_id;
                // }
            }
            
            if(in_array('change_in_name',$bulkUpdateOption)){
                if(!empty($event->latestVersion)){
                    if($event->latestVersion->name != $event->name){
                        $event->name = $event->latestVersion->name;
                    } 
                }
                
                /*if(!empty($event->prev_event_name) && $event->prev_event_name != NULL){
                    $event->name = $event->prev_event_name;
                }*/
            }
            
            $event->save();
            
            $csrfToken = csrf_token();
            $response = Http::withToken($csrfToken)->post(route('tixstock.events.add'), [
                '_token' => $csrfToken,
                'event_id' => $eventId,
            ]);
            
        }
        return response()->json(['success'=>true,'successmsg'=>"Events are updated.Please check respective event's  tixstock log."]);
    }
    
    public function bulkEventsVersioning(Request $request){
        $input = $request->all();
        
        $events = explode(',',$input['event_id']);
        $checkedEvent = [];
        //check if any mapped events or not
        $checkMappedEvent = Events::whereIn('id',$events)->where('mapped_status',1)->first();
        
        if($checkMappedEvent){
            foreach($events as $eventId){
                
                $event = Events::where('id',$eventId)->first();
                //check if event is mapped or not 
                if($event->mapped_status == 1){
                    $versionEvent = Events::where('name','like','%'.$event->name.'%')
                                            ->where('venue_id',$event->venue_id)
                                            ->whereIn('id',$events)
                                            ->where('mapped_status',0)
                                            ->where('current_version',1)
                                            ->first();
                    
                    if($versionEvent){
                        //check Events performers
                        $versionEventPerformers = $versionEvent->performers->count();
                        $eventPerformers = $event->performers->count();
                        
                        if($versionEventPerformers == $eventPerformers){
                            //check date difference 
                            $eventDate = $event->event_date;
                            $versionEventDate = $versionEvent->event_date;
                            $difference = round((strtotime($versionEventDate) - strtotime($eventDate))/3600, 1);
                            
                            if($difference <= 48){
                                
                                $event->update_requires = 1;
                                $event->save();
                                
                                $versionEvent->parent_event_id = $event->id;
                                $versionEvent->current_version = 0;
                                $versionEvent->save();
                            }
                        }
                    }
                    $checkedEvent[] = $event->external_id;
                }else{
                    $parentEvent = Events::where('name','like','%'.$event->name.'%')
                                            ->where('venue_id',$event->venue_id)
                                            ->whereIn('id',$events)
                                            ->where('mapped_status',1)
                                            ->where('current_version',1)
                                            ->first();
                    if($parentEvent){
                         $parentEventPerformers = $parentEvent->performers->count();
                         $eventPerformers = $event->performers->count();
                        if($parentEventPerformers == $eventPerformers){
                            //check date difference 
                            $eventDate = $event->event_date;
                            $parentEventDate = $parentEvent->event_date;
                            $difference = round((strtotime($eventDate) - strtotime($parentEventDate))/3600, 1);
                            if($difference <= 48){
                                
                                $parentEvent->update_requires = 1;
                                $parentEvent->save();
                                
                                $event->parent_event_id = $parentEvent->id;
                                $event->current_version = 0;
                                $event->save();
                            }
                        }
                    }
                    $checkedEvent[] = $event->external_id;
                }
            }
            return response()->json(['success'=>true,'msg'=>implode(', ',array_unique($checkedEvent))." Events Processed"]);
        }else{
            return response()->json(['success'=>false,'msg'=>'No Mapped Events Found.']);
        }
        
        //     foreach($events as $event){
        //     $event = Events::where('id',$event)->first();
        //     $matchingEvent = Events::with('eventPerformers')->where('name','like','%'.$event->name.'%')->where('venue_id',$event->venue_id)->whereIn('id',$events)->where('id','!=',$event)->first();
            
        // }
    }
    
    public function tixstockLog(){
        
    }
    
    public function assignMap(Request $request){
        $events = Events::where('id',$request->event_id)->first();
        $events->map_id = $request->id;
        if($events->mapped_status == 1){
            $events->update_requires = 1;
        }
        $events->save();
        return response()->json(['success'=>true,'msg'=>'Map is assigned to the event.']);
    }
    
    public function removeMap(Request $request){
        $events = Events::where('id',$request->event_id)->first();
        $events->map_id = null;
        $events->save();
        return response()->json(['success'=>true,'msg'=>'Map is removed from the event.']);
    }
    
    public function importEvents(){
        return view('admin.events.import_events');
    }
    
    public function exportEvents(Request $request){
        
        $input = $request->all();
        $fileName = 'events.xlsx';
        
        $categoriesArray=[];
        
        
        $categories = Categories::with('parent')->where('parent_id',0)->get();
        if($categories->count() > 0){
            $subCategories = $this->subCategories($categories);
            $categoriesArray = $this->categoriesArray;
            sort($categoriesArray);
        }
        $events= Events::with(['venue','venueAddress', 'prevperformers', 'performers', 'eventcategory','tixstocklog'=>function($query){
            return $query->select('event_id');
        },'eventVersion','source','latestVersion','maps']);
        
        if($request->update_requires==1){
                $events=$events->where('update_requires',1);
        }
        if(!empty($request->type)){
            $type=$request->type;
            $events=$events->whereHas('eventcategory',function($q) use ($type){
                $q->where('primary_category',$type);
            });
        }
        if($request->mappedStatus != '' && ($request->mappedStatus == 0 || $request->mappedStatus == 1)){
            $events=$events->where('mapped_status',$request->mappedStatus);
        }
          
        if(!empty($request->performer)){
           $performers = explode(',',$request->performer);
            $events = $events->whereHas('performers',function($query)use($performers){
                $query->whereIn('performer_id',$performers);
            });
        }
        if(!empty($request->event_date) && empty($request->external_id)){
            
                $event_date = trim($request->event_date);
                $event_date = explode('to',$event_date);
                $start_date = trim($event_date[0]);
                $end_date = trim($event_date[1]);
                $events->whereBetween('event_date',[$start_date.' 00:00:00',$end_date.' 23:59:59']);
           
        }
        
        if(!empty($request->event_created_date) && empty($request->external_id)){
            
                $event_date = trim($request->event_created_date);
                $event_date = explode('to',$event_date);
                $start_date = $event_date[0];
                $end_date = $event_date[1];
                $events = $events->whereBetween('created_at',[trim($start_date).' 00:00:00',trim($end_date).' 23:59:59']);
            
        }
        if(!empty($request->event_name)){
            // $events=$events->whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $request->event_name);
             $events = $events->where('name','like','%'.$request->event_name.'%');
        }
        if(!empty($request->external_id)){
            $externalId = explode(',',$request->external_id);

            $events=$events->whereIn('external_id',$externalId);
        }
        if(!empty($request->venue)){
            $venue = $request->venue;
            $events=$events->whereHas('venue', function($q) use ($venue){
                $q->where('name','like','%'.$venue.'%');
            });
        }
        if(!empty($request->venue_address)){
            $venueAddress = $request->venue_address;
            $events=$events->whereHas('venueAddress',function($q) use ($venueAddress){
                $q->where('address_line_1','like','%'.$venueAddress.'%');
            });
        }
        if(!empty($request->change_in_date)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('event_date', '!=', 'events.event_date');
            })->where('update_requires',1);
        }
        if(!empty($request->change_in_venue)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('venue_id', '!=', 'events.venue_id');
            })->where('update_requires',1);
        }
        if(!empty($request->change_in_name)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereColumn('name', '!=', 'events.name');
            })->where('update_requires',1);
        }
        
        if(!empty($request->change_in_performer)){
            $events = $events->whereHas('latestVersion', function ($query) {
                $query->whereDoesntHave('performers', function ($subquery) {
                    $subquery->whereIn('performer_id', function ($innerSubquery) {
                    $innerSubquery->select('performer_id')
                        ->from('event_performers')
                        ->whereColumn('event_id', '=', 'events.id');
                    });
                });
            })->where('update_requires',1);
        }
        
        if(!empty($request->resource)){
            $events =  $events->whereIn('source_id',$request->resource);
        }
    
        if(!empty($request->has_multiple_version)){
            $events = $events->whereIn('id',array_unique($parentEventIds));
        }
        
        
        if($request->mapped_status != null || $request->mapped_status==0){
            $events = $events->where('current_version',1);
        }else{
            $events = $events->whereIn('current_version',[0,1]);
        }
        
        if($request->unmapped_with_tixstocklog==1){
            $events =$events->whereHas('tixstocklog')->where('mapped_status',0);
        }
        
        if(empty($request->type) && $request->mapped_categories_filter==0){
            $events=$events->whereHas('eventcategory',function($q) use ($categoriesArray){
                $q->whereNotIn('primary_category',$categoriesArray);
            });
        }
        
        $events = $events->where('event_date','>=',date('Y-m-d'));
        $events = $events->get();
        
        if($events->count() > 0){
            $filePath = Excel::store(new EventsExport($events), $fileName);
            $file = env('APP_URL').'/storage/app/'.$fileName;
            return response()->json(['success'=>true,'data' => [ 'file_name' => $file ] ]);
        }else{
            return response()->json(['success'=>false,'msg'=>"No Events Found."]);
        }
    }
    
    public function importEventsSubmit(Request $request){
        
        
        
        $request->validate([
            'events_file' => 'required|mimes:xls,xlsx',
        ]);

        // Get the file from the request
        $file = $request->file('events_file');

        // Import the Excel file and get the first sheet data
        $data = Excel::toArray([], $file)[0];
        $header = array_shift($data);
        $columnIndexes = array_flip($header);
       
        if(!empty($data)){
            foreach ($data as $row) {
                // Create a new Event instance
                $event = new Events();
                $date =Carbon::createFromTimestamp(floor(($row[$columnIndexes['event_datetime']] - 25569) * 86400));
                $objEvents = new Events();
                $uuidNo = $objEvents->generateUUID();
                //checkEvent 
                $checkEvent = Events::where('name',trim($row[$columnIndexes['event_name']]))->where('event_date',$date)->first();
                
                if(!$checkEvent){
                    // Set the event attributes using the column indexes
                    $event->name = trim($row[$columnIndexes['event_name']]);
                    $event->external_id = $uuidNo;
                    $event->search_type = $row[$columnIndexes['event_category']];
                    $event->event_date = Carbon::createFromTimestamp(floor(($row[$columnIndexes['event_datetime']] - 25569) * 86400));
                    $event->venue_id = $row[$columnIndexes['venue_id']];
                    $event->source_id = 0;
                    
                    //get venue address_id
                    $venueAddress = VenueAdditionalInfo::where('venue_id',$row[$columnIndexes['venue_id']])->get();
                    $venueAddressId = "";
                    if(!empty($venueAddress) && $venueAddress->count() > 0){
                        $venueAddressId = $venueAddress[0]->id;
                    }
                    $event->venue_address_id =$venueAddressId;
                    // Save the event to the database
                    $event->save();
                    $eventId = $event->id;
                    $performers = $row[$columnIndexes['performers']];
                    $performers = explode(',',$performers);
                    
                    $venueCategory = VenueCategory::with(['sections'])->where('venue_id',$row[$columnIndexes['venue_id']])->get();
                    $categoryArray=[];
                    $sectionArray=[];
                    if($venueCategory->count() > 0){
                        foreach($venueCategory as $category){
                        $categoryArray[] = $category->id;
                        if(!empty($category->sections) && $category->sections->count() > 0){
                            foreach($category->sections as $section){
                                $sectionArray[]= $section->id;
                            }
                        }
                    }
                    }
                    $categories="";
                    if(!empty($categoryArray)){
                        $categories = implode(',',$categoryArray);
                    }
                    $sections="";
                    if(!empty($sectionArray)){
                        $sections = implode(',',$sectionArray);  
                    }
                
                    //add event performers;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            $performer = trim($performer);
                            //check peformers
                            $checkPerformer = Performers::where('name',$performer)->first();
                            if(!$checkPerformer){
                                $performers = Performers::create(['name'=>$performer]);
                                $performerId = $performers->id;
                            }else{
                                $performerId = $checkPerformer->id;
                            }
                            
                            EventPerformers::create(['event_id'=>$eventId,'performer_id'=>$performerId]);
                        }
                    }
                    EventCategory::create(['event_id'=>$eventId,'primary_category'=>$row[$columnIndexes['event_category']],'source_id'=>0,'event_categories'=>$categories,'event_section'=>$sections]);
                }
            }
        }
        
        return response(['success'=>true,'msg'=>"Events Added Successfully."]);
    }
    
    public function bulkUpdateEventsCatSection(Request $request){
        $input = $request->all();
        $venueId = Events::select('venue_id')->whereIn('id',explode(',',$input['event_id']))->groupBy('venue_id')->get()->toArray();
        if(!empty($venueId) && sizeof($venueId) > 1){
            return response()->json(['success'=>false,'msg'=>"Different Venues.Can't change categorty and section."]);
        }
        
        $venueId = array_shift($venueId);
        $venueId = $venueId['venue_id'];
        
        $otherCategory = VenueCategory::with('section')->where('venue_id',$venueId)->where('source_id',0)->get();
        $category = VenueCategory::with('section')->where('venue_id',$venueId)->where('source_id','!=',0)->get();
        $events = $input['event_id'];
        $manual_event = 1;
        $catSectionModal = view('admin.events.ajax.update_category_section',compact('category','otherCategory','events','venueId','manual_event'))->render();
        
        return response()->json(['success'=>true,'html'=>$catSectionModal]);
    }
    
    public function submitBulkUpdateEventsCatSection(Request $request){
        $input = $request->all();
    
        if(!isset($input['category']) ||  empty($input['category'])){
            return response()->json(['success'=>false,'msg'=>"Please select category."]);
            exit;
        }
        $eventCategories = EventCategory::with('event')->whereIn('event_id',explode(',',$input['id']))->get();
        
        if($eventCategories->count()>0){
            foreach($eventCategories as $eventCategory){
                
                // $category = explode(',',$eventCategory->event_categories);
                // $section  = explode(',',$eventCategory->event_section);
                
                $category = [];
                $section = [];
                
                foreach($input['category'] as $cat){
                        $category[] = $cat;
                }
                foreach($input['sections'] as $sec){
                        $section[] = $sec;
                }
                
                $category = implode(',',$category);
                $section = implode(',',$section);
                $eventCategory->event_categories = $category;
                $eventCategory->event_section = $section;
                $eventCategory->save();
                
                if($eventCategory->event->mapped_status==1){
                    Events::where('id',$eventCategory->event_id)->update(['update_requires'=>1]);
                }
            }
            
            return response()->json(['success'=>true,'msg'=>'Events updated successfully.']);
        }
        
    }
    
    public function bulkUpdateEventsMap(Request $request){
        $input = $request->all();
        $updateArray = ['map_id'=>$input['maps']];
        
        Events::whereIn('id',explode(',',$input['event_id']))->update($updateArray);
        Events::whereIn('id',explode(',',$input['event_id']))->where('mapped_status',1)->update(['update_requires'=>1]);
        
        return response(['success'=>true,'msg'=>'Map is updated to all selected events']);
    }
    
}
