<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\VenueCategory;
use App\VenueSection;
use App\Categories;
use App\Maps;
use App\Sources;
use App\VenueMaps;
use Aws\S3\S3Client;
use Barryvdh\Snappy\Facades\SnappyImage;
use File;
use DataTables;
class MapsController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
        return view('admin.maps.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $venueId ="";
        $categories = VenueCategory::orderBy('name')->groupBy('name')->get();
        $categoriesTree = Categories::get();
        return view('admin.maps.create',compact('categories','categoriesTree','venueId'));
    }
    
    public function data(Request $request){
        
        $maps=Maps::with(['source']);
        
        if($request->name && !empty($request->name)){
            $maps->where('name','like','%'.$request->name.'%');
        }
        
        if($request->status != ""){
            $maps->where('status',$request->status);
        }
        
        return Datatables::eloquent($maps)
        ->rawColumns(['action','name','preview_url','categories','status','source'])
        ->editColumn('name', function ($maps) {
            return $maps->name;
        })
        ->editColumn('status', function ($maps) {
            $html = '';
            if($maps->status == 1){
               $html = '<span class="badge badge-soft-warning">Grouped</span>';
            }else{
               $html = '<span class="badge badge-soft-danger">Un-Grouped</span>';
            }
            return $html;
        })
       ->addColumn('source', function ($maps) {
           return $maps->source->name;
        }) 
        ->editColumn('categories', function ($maps) {
            $html = "";
            if(!empty($maps->categories)){
                $categories = explode(',',$maps->categories);
                foreach($categories as $category){
                    $html.='<span class="badge badge-soft-primary">'.$category.'</span>';
                }
            }
            return $html;
        })
        ->addColumn('preview_url', function ($maps) {
           return '<a class="ms-2" href="'.$maps->url.'" target="_blank"><i class="mdi mdi-link-variant font-18 text-success me-1 align-middle"></i></a>';
        })
        ->addColumn('action', function ($maps) {
            $actionHtml = '';
            $actionHtml.= '<button type="button" class="btn btn-sm btn-light delete" data-src="'.route('maps.destroy',$maps->id).'" data-id="'.$maps->id.'"><i class="fa fa-trash"></i></button>';
            if($maps->status == 0) {
                $actionHtml .= '<a type="button" class="btn btn-sm btn-light" href="'.route('map.tivo.edit',$maps->id).'" data-id="'.$maps->id.'"><i class="fa fa-edit"></i></a>';
            }else{
            $actionHtml .= '<a type="button" class="btn btn-sm btn-light" href="'.route('maps.edit',$maps->id).'" data-id="'.$maps->id.'"><i class="fa fa-edit"></i></a>';
            }
            return $actionHtml;   
        })
        ->make(true);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        
        //create svg file in localdirectory
        $input=$request->all();
        //move code to s3bucket
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        $fileName = $input['map-name'].".svg";
        $path = env('AWS_FOLDER').'maps/'.$fileName;
        $s3->putObject([
            'Bucket' => env('AWS_BUCKET'),
            'Key' => $path,
            'Body' => $input['svg-data'],
            'ContentType' => 'image/svg+xml',
            'ACL' => 'public-read',
        ]);
        
        $bucket = env('AWS_BUCKET');
        $key = $path;

        $url = $s3->getObjectUrl($bucket, $key);
        $resouce = Sources::where('name','Tixstock')->first();
        $source_id = "";
        if($resource){
            $source_id = $resource->id;
        }
        //store url and map details
        if(!empty($url)){
            $maps = Maps::create(['name'=>$fileName,'url'=>$url,'categories'=>implode(',',$input['category']),'source_id'=>1,'status'=>1]);
            if(isset($input['venue_id']) && !empty($input['venue_id'])){
                //assign map to venue
                VenueMaps::create(['venue_id'=>$input['venue_id'],'map_id'=>$maps->id]);
            }
            return response()->json(['success'=>true,"msg"=>"Map Created Successfully."]);
        }else{
            return response()->json(['success'=>false,"msg"=>"Something Wrong.Map is not created."]);
        }
        
        
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
         
         $map = Maps::where('id',$id)->first();
         $content = file_get_contents($map->url);
         $categories = VenueCategory::orderBy('name')->groupBy('name')->get();
         $categoriesTree = Categories::get();
         return view('admin.maps.edit',compact('map','content','categories','categoriesTree'));
    }
    
    public function tivo_edit($id){
        
         $map = Maps::where('id',$id)->first();
         $content = file_get_contents($map->url);
         $categories = VenueCategory::orderBy('name')->groupBy('name')->get();
         $categoriesTree = Categories::get();
         $venueId ="";
         $catSection = [];
         $content = file_get_contents($map->url); 
         return view('admin.maps.tivo_maps',compact('map','content','categories','categoriesTree','venueId','content','catSection'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        
        
        $input=$request->all();
        //move code to s3bucket
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        
        $fileName = $input['map-name'];
        $path = env('AWS_FOLDER').'maps/'.$fileName;
        
        $bucket = env('AWS_BUCKET');
        
        
        $existingObject = $s3->headObject([
                'Bucket' => $bucket,
                'Key' => $path
        ]);
        
        $lastModifiedBefore = $existingObject['LastModified'];
        
        $eTag = $existingObject['ETag'];
        
        $s3->putObject([
            'Bucket' => $bucket,
            'Key' => $path,
            'Body' => $input['svg-data'],
            'ContentLength' => strlen($input['svg-data']),
            'ServerSideEncryption' => 'AES256',
            'ContentType' => 'image/svg+xml',
            'ACL' => 'public-read',
            'IfMatch' => $eTag
        ]);
        
        $object = $s3->headObject([
                'Bucket' => $bucket,
                'Key' => $path
        ]);
        $lastModifiedAfter = $object['LastModified'];
        //store url and map details
        if ($lastModifiedBefore != $lastModifiedAfter) {
            $maps = Maps::where('id',$id)->update(['name'=>$fileName,'categories'=>implode(',',$input['category']),'status'=>1]);
            return response()->json(['success'=>true,"msg"=>"Map updated Successfully."]);
        }else{
            return response()->json(['success'=>false,"msg"=>"Something Wrong.Map is not created."]);
        }
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        
        $map = Maps::where('id',$id)->first();
        
        
        $s3 = new S3Client([
            'version' => 'latest',
            'region' => env('AWS_DEFAULT_REGION'),
            'credentials' => [
                'key' => env('AWS_ACCESS_KEY_ID'),
                'secret' => env('AWS_SECRET_ACCESS_KEY'),
            ],
        ]);
        
        $result = $s3->deleteObject([
            'Bucket' => env('AWS_BUCKET'),
            'Key' => env('AWS_FOLDER').'maps/'.$map->name,
        ]);
        
        // if ($result['DeleteMarker']) {
             Events::where('map_id',$map->id)->update(['map_id'=>NULL]);
             VenueMaps::where('map_id',$map->id)->delete();
             $map->delete();
             return response()->json(['success'=>true,'msg'=>"Map deleted successfully."]);
        // } else {
        //     return response()->json(['success'=>false,'msg'=>"Something went wrong! Map isn't deleted."]);
        // }
        
        
        
        /*
        $objectKey = env('AWS_FOLDER').$fileName;
        $bucketName = env('AWS_BUCKET');
        
        try {
            $result = $s3->headObject([
                'Bucket' => $bucketName,
                'Key' => $objectKey,
            ]);
        } catch (S3Exception $e) {
            // The object has been deleted
            if ($e->getStatusCode() == '404') {
                echo "The object has been deleted.";
            } else {
                // Other error occurred
                echo "An error occurred while checking the object status: " . $e->getMessage();
            }
        }
        */
    }
    
    public function getSection(Request $request){
        $category = VenueCategory::where('name','like','%'.$request->category.'%')->select('id')->get()->toArray();
        $ids = array_column($category, 'id');
        $venueSection = VenueSection::whereIn('category_id',$ids)->select('name')->get()->toArray();
        $venueSections = array_unique(array_column($venueSection,'name'));
        
        $html = '';
        foreach($venueSections as $section){
            $html .='<option value="'.$section.'">'.$section.'</option>';
        }
        
        
        return response()->json(['success'=>true,'html'=>$html]);
    }
}
