<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Performers;
use App\Events;
use DataTables;
use App\EventPerformers;
class PerformerController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //

        return view('admin.performers.index');
    }

    public function data(Request $request){
        $performers = Performers::with('events');
        return Datatables::eloquent($performers)
        ->rawColumns(['name','events','action'])
        ->editColumn('name', function ($performers) {
            $route = route("events.index",['performer'=>$performers->name]);
            $name = "<a href= ".$route.">".$performers->name."</a>";
            return $name;
        })
        ->addColumn('events', function ($performers) {
            $totalCount= EventPerformers::where('performer_id',$performers->id)->get()->count();
            // $totalCount = $performers->events->count()+$awayEvents;
            return $totalCount;
        })
        
        ->addColumn('action',function($performers){
            $totalEvents = EventPerformers::where('performer_id',$performers->id)->get()->count();
            if($totalEvents > 0){
                return '<a type="button" class="btn btn-sm btn-light" href="'.route('performer.edit',$performers->id).'" data-id="'.$performers->id.'"><i class="fa fa-edit"></i></a>';
            }else{
                return '<button type="button" class="btn btn-sm btn-light delete" data-src="'.route('performer.destroy',$performers->id).'" data-id="'.$performers->id.'"><i class="fa fa-trash"></i></button>
                    <a type="button" class="btn btn-sm btn-light" href="'.route('performer.edit',$performers->id).'" data-id="'.$performers->id.'"><i class="fa fa-edit"></i></a>';
            }
            
            
        })
        
       ->make(true);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        
        return view('admin.performers.create');
    }

    public function getPerformers(Request $request){
        $search= isset($request->q) ? $request->q : '';
        $page= isset($request->page) ? $request->page: 1;
        $resultCount = 10;
        $end = ($page - 1) * $resultCount;       
        $start = $end + $resultCount;
        
        $performers = Performers::query();

        if(!empty($search)){
            $performers = $performers->where('name','like','%'.$search.'%');
        }

        $performers = $performers->get();
        
        $count = 10;
        $data = [];
        if($performers){
            foreach($performers as $performer){
                $data[]=['id'=>$performer->id,'name'=>$performer->name];
            }
        }
       
            
        // IF SEARCH TERM IS NOT FOUND DATA WILL BE EMPTY SO
        if (empty($data)){
            $empty[] = ['id'=>'', 'name'=>'',"q" => $q]; 
            return response()->json($empty);
        }
        else{
            $results = array(
              "items" => $data,
              "total_count" => $count,
            ); 
            return response()->json($results);
        }
    
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        // 
        $rules = [
            'name'         => 'required',
        ];
        $this->validate($request, $rules);
        $input = $request->all();
        
        $checkPerformer = Performers::where('name','like','%'.$input['name'].'%')->first();
        if(!$checkPerformer){
            Performers::Create(['name'=>trim($input['name']),'name_variation'=>$input['name'],'external_id'=>trim($input['external_id'])]);
            return response()->json(['success'=>true,'msg'=>'Performer created successfully.']);
        }else{
            return response()->json(['success'=>false,'msg'=>'Performer already exist.']);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *r
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $performer  =  Performers::where('id',$id)->first();
        return view('admin.performers.edit',compact('performer'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        $rules = [
            'name'         => 'required',
        ];
        $this->validate($request, $rules);
        $input = $request->all();
        Performers::where('id',$id)->update(['name'=>trim($input['name']),'name_variation'=>$input['name'],'external_id'=>trim($input['external_id'])]);
        return response()->json(['success'=>true,'msg'=>'Performer updated successfully.']);
        
        
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
        $performer = Performers::where('id',$id)->delete();
        return response()->json(['success'=>true,'msg'=>"Performer deleted successfully."]);
    }

    
}
