<?php

namespace App\Http\Controllers\Tixstock;

use App\Http\Controllers\Controller;

use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\TixstockLog;
use App\Http\Controllers\API\StubhubController as StubhubApi;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use App\OldCategories;
use App\SubCategories;
use App\VenueMaps;
use App\Maps;
use App\Jobs\ProcessEventsJob;
use Carbon\Carbon;
use App\EventsMappedSources;
class EventsController extends Controller
{

    public function addEvent(Request $request) {
        
        
        $addedEvents = [];
        $updatedEvents = [];
        $errorEvents = [];
        $skipGaEvent = 0;
        
        
        if (!$request->filled('event_id')) {
            return response()->json(['success' => false, 'errors' => ['event_id is required.'] ]);
        }
        
        $eventIds=explode(',',$request->event_id);
        
        if(isset($request->gaEvent) && !empty($request->gaEvent)){
            $skipGaEvent = $request->gaEvent;
        }
       
        //check if any event is with no cat/section
        
        if($skipGaEvent == 0){
            $checkEventsCatSection = EventCategory::whereIn('event_id',$eventIds)
                                                    ->where(function ($query) {
                                                        $query->where('event_categories', '')
                                                              ->orWhereNull('event_categories');
                                                    })->get();
            
            if($checkEventsCatSection->count() > 0){
                $eventArray = [];
                foreach($checkEventsCatSection as $event){
                    $eventArray[] = $event->event->external_id;
                }
                return response()->json(['success'=>false,'successmsg'=>"","errormsg"=>"Events (".implode(',',$eventArray).") are missing with venuedetails(Catgory/Section)."]);
            }
        }
        
        if(!empty($eventIds) && sizeof($eventIds) > 5){
            ProcessEventsJob::dispatch($eventIds);
            return response()->json(['success'=>true,'successmsg'=>"Events are processing in background.Event logs can be tracked on tixstocklogs.","errormsg"=>""]);
        }
        
        if(!empty($eventIds)){
            
             $date = Carbon::today()->format('dmY');

            // Count the number of logs for today with the same batch ID
            $lastLog = TixstockLog::whereDate('created_at', today())
                ->where('batch_id', 'like', $date . '%')
                ->orderBy('id','desc')
                ->first();
            
            if($lastLog){
            $lastBatchId = explode('-',$lastLog->batch_id);
            $count = $lastBatchId[1]+1;
            }else{
                $count= 1;
            }

            // Set the batch ID
            $batch_id = $date . '-' . $count;
            foreach($eventIds as $eventId){
                
                $sources=Sources::where('name','Tixstock')->first();
                $api = Http::withToken($sources->token);
                
                $event = "";
                $event = Events::with(['venue','performers','venueAddress', 'eventcategory' ,'performer1', 'performer2','source','latestVersion'])->where('id', $eventId)->first();
                
                $eventCategory = isset($event->eventcategory->event_categories) && !empty($event->eventcategory->event_categories) ? $event->eventcategory->event_categories : [];
                $eventSection = isset($event->eventcategory->event_section) && !empty($event->eventcategory->event_section) ? $event->eventcategory->event_section : [];
              
                $category = [];
                if(!empty($eventCategory)){
                    $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                      return $query->whereIn('id',explode(',',$eventSection));
                    }])->whereIn('id',explode(',',$eventCategory))->where('venue_id',$event->venue_id)->get(['id','external_id','name','stubhub_name'])->toArray();
                }
                
                $removeId = function ($item) use (&$removeId) {
                    if($item['external_id'] != ""){
                        $item['id']=(string)$item['external_id'];
                    }else{
                        $item['id'] = (string)$item['id'];
                    }
                    
                    if (isset($item['sections'])) {
                        $item['sections'] = array_map($removeId, $item['sections']);
                    }
                    unset($item['external_id']);
                    return $item;
                };
                
                $category = array_map($removeId, $category);
                
                if ($event) {
                        
                        $address_line_1="";
                        $address_line_2="";
                        $post_code="";
                        
                        if(!empty($event->venue_address_id)){
                            $address_line_1=$event->venueAddress->address_line_1;
                            $address_line_2=$event->venueAddress->address_line_2;
                            $post_code= $event->venueAddress->post_code;
                        }else{
                            if($event->venue->venueAdditionalInfo->count() > 0){
                                $address_line_1 = $event->venue->venueAdditionalInfo[0]->address_line_1;
                                $address_line_2 = $event->venue->venueAdditionalInfo[0]->address_line_2;
                                $post_code = $event->venue->venueAdditionalInfo[0]->post_code;
                            }
                        }
                        
                        //maps logic
                        $map_url="";
                        if(!empty($event->map_id)){
                            $map = Maps::where('id',$event->map_id)->first();
                            $map_url = $map->url;
                        }
                        
                        if(empty($map_url)){
                            $maps = VenueMaps::with('maps')->where('venue_id',$event->venue_id)->first();
                            if($maps){
                                $map_url = $maps->maps->url;
                            }
                        }
                    
                        $form_data = [];
                        $form_data['id'] = (string)$event->external_id;
                        $form_data['name'] = $event->name;
                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->event_date));
                        $form_data['currency'] = $event->currency_code!=NULL ? $event->currency_code : "EUR";
                        $form_data['use_tixstock_venue_details'] = 0;
                        $form_data['venue']['id'] = isset($event->venue->z) && !empty($event->venue->tixstock_id) ? (string)$event->venue->tixstock_id : (string)$event->venue->id;
                        $form_data['venue']['name'] = $event->venue->name;
                        $form_data['venue']['address_line_1'] = $address_line_1;//isset($event->venueAddress->address_line_1) && !empty($event->venueAddress->address_line_1) ? $event->venueAddress->address_line_1: "" ;
                        $form_data['venue']['address_line_2'] = $address_line_2;//$event->venue->address_line_2;
                        $form_data['venue']['city'] = $event->venue->city;
                        $form_data['venue']['country_code'] = $event->venue->country;
                        $form_data['venue']['postcode'] = $post_code;//isset($event->venueAddress->post_code) && !empty($event->venueAddress->post_code) ? $event->venueAddress->post_code: ''; //$event->venue->post_code;
                        $form_data['map_url'] = $map_url;
                        
                        $venue_details=$category;//VenueCategory::with('sections')->whereIn('id',explode(',',$event->eventcategory->event_categories))->where('venue_id',$event->venue->id)->get(['id','name'])->toArray();
                        
                        if(!empty($venue_details) && $venue_details != NULL){
                            $form_data['venue_details']=$venue_details;
                        }
                        
                        if($event->performers->count() > 0){
                            $k=0;
                            foreach($event->performers as $performer){
                                $form_data['performers'][$k]['id'] = isset($performer->external_id) && !empty($performer->external_id) ? (string)$performer->external_id : (string)$performer->id;
                                $form_data['performers'][$k]['name'] = isset($performer->name) && !empty($performer->name)? $performer->name : '';
                                $k++;
                            } 
                        }
                        
                        $form_data['category'] = $event->eventcategory->primary_category;
                        
                        // $form_data['category']=!empty($event->eventcategory->secondary_category) && $event->eventcategory->secondary_category != NULL ? $event->eventcategory->secondary_category : $event->eventcategory->primary_category ;
                        $form_data['status'] = "Active";
            
                        // if (!empty($event->performer2)) {
                        //     $form_data['performers'][1]['id'] = isset($event->performer2->external_id) && !empty($event->performer2->external_id) ? (string)$event->performer2->external_id : (string)$event->performer2->id;
                        //     $form_data['performers'][1]['name'] = isset($event->performer2->name) && !empty($event->performer2->name) ? $event->performer2->name : '';
                        // }
                  
                    if($event->mapped_status == 0 && $event->update_requires == 0){
                        
                        $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                        $response_arr = $response->json();
                        $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->external_id;
                        
                        $sourcesId = $sources->id;
                        if($sourcesId == 0){
                            $sources = Sources::where('name','Tixstock')->first();
                            $sourcesId = $sources->id;
                        }
                        
                        if(!empty($response_arr['data'])){
                            //store success log into database
                            $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sourcesId,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $addedEvents [] = $event->external_id;
                            $event->mapped_status=1;
                            $event->save();
                            //mapped with resource
                            EventsMappedSources::create(['event_id'=>$event->id,'source_id'=>$sourcesId]);
                            
                        }else{
                           // store error log in to database
                            $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sourcesId,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $errorEvents[] = $event->external_id;
                        }
                        
                            //send using stubhub,ticket evolution token
                            
                            if(!empty($eventCategory)){
                                $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                                  return $query->whereIn('id',explode(',',$eventSection));
                                }])->whereIn('id',explode(',',$eventCategory))->where('stubhub_name',NULL)->where('parent_category_id',NULL)->where('venue_id',$event->venue_id)->get(['id','name','stubhub_name'])->toArray();
                            }
                            if(!empty($category) && $category != NULL){
                                $form_data['venue_details']=$category;
                            }
                            $form_data['use_tixstock_venue_details'] = 1;
                            unset($form_data['venue_details']);
                            if($event->source_id==0){
                                    continue;
                                }
                                
                            $sources=Sources::where('id',$event->source->id)->first();
                            $api = Http::withToken($sources->token);
                            unset($form_data['url']);
                            $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                            $response_arr = $response->json();
                            
                            if(!empty($response_arr['data'])){
                                //store success log into database
                                $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->external_id;
                                $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                                TixstockLog::create($tixstockLog);
                                //mapped with resource
                                EventsMappedSources::create(['event_id'=>$event->id,'source_id'=>$sources->id]);
                            }else{
                                //store error log into database
                                $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                                TixstockLog::create($tixstockLog);
                            } 
                    }
                    
                    //update event 
                    if($event->update_requires == 1){
                        
                         $response = $api->post(env('TIXSTOCK_URL').'/events/update/'.$event->external_id, $form_data);
                         $response_arr = $response->json();
                         $form_data['url'] = env('TIXSTOCK_URL').'/events/update/'.$event->external_id;
                         if(!empty($response_arr['data'])){
                             //store log in to database
                             $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                             TixstockLog::create($tixstockLog);
                             $updatedEvents[] = $event->external_id;
                             $event->update_requires = 0;
                             $event->mapped_status =1;
                             $event->save(); 
                         }else{
                             //store error log into database
                             $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                             TixstockLog::create($tixstockLog);
                             $errorEvents[] =$event->external_id; 
                         }
                         
                        //check if event is manual then add to stubhub if latest version is from stubhub.
                        if( $event->source_id == 0 || (!empty($event->source) && $event->source->name=="Tixstock")){ 
                                
                                if(!empty($event->latestVersion) && $event->latestVersion->mapped_status==0 && $event->latestVersion->source->name == "Stubhub"){
                                    //move event to tixstock with stubhub token
                                    $eventCategory = isset($event->latestVersion->eventcategory->event_categories) && !empty($event->latestVersion->eventcategory->event_categories) ? $event->latestVersion->eventcategory->event_categories : [];
                                    $eventSection = isset($event->latestVersion->eventcategory->event_section) && !empty($event->latestVersion->eventcategory->event_section) ? $event->latestVersion->eventcategory->event_section : [];
                                    $category = [];
                                    if(!empty($eventCategory)){
                                        $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                                          return $query->whereIn('id',explode(',',$eventSection));
                                        }])->whereIn('id',explode(',',$eventCategory))->where('venue_id',$event->latestVersion->venue_id)->get(['id','external_id','name','stubhub_name'])->toArray();
                                    }
                                    
                                    $removeId = function ($item) use (&$removeId) {
                                        if($item['external_id'] != ""){
                                            $item['id']=(string)$item['external_id'];
                                        }else{
                                            $item['id'] = (string)$item['id'];
                                        }
                                        
                                        if (isset($item['sections'])) {
                                            $item['sections'] = array_map($removeId, $item['sections']);
                                        }
                                        unset($item['external_id']);
                                        return $item;
                                    };
                                    
                                    $category = array_map($removeId, $category);
                                    
                                    $address_line_1="";
                                    $address_line_2="";
                                    $post_code="";
                        
                                    if(!empty($event->latestVersion->venue_address_id)){
                                        $address_line_1=$event->latestVersion->venueAddress->address_line_1;
                                        $address_line_2=$event->latestVersion->venueAddress->address_line_2;
                                        $post_code= $event->latestVersion->venueAddress->post_code;
                                    }else{
                                        if($event->latestVersion->venue->venueAdditionalInfo->count() > 0){
                                            $address_line_1 = $event->latestVersion->venue->venueAdditionalInfo[0]->address_line_1;
                                            $address_line_2 = $event->latestVersion->venue->venueAdditionalInfo[0]->address_line_2;
                                            $post_code = $event->latestVersion->venue->venueAdditionalInfo[0]->post_code;
                                        }
                                    }
                                    $form_data = [];
                                    $form_data['id'] = (string)$event->latestVersion->external_id;
                                    
                                    if($event->name != $event->latestVersion->name){
                                        $event->latestVersion->name = $event->name;
                                        $form_data['name'] = $event->name;
                                    }else{
                                        $form_data['name'] = $event->latestVersion->name;
                                    }
                                    
                                    if($event->event_date != $event->latestVersion->event_date){
                                        $event->latestVersion->event_date = $event->event_date;
                                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->event_date));
                                    }else{
                                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->latestVersion->event_date));
                                    }
                                    
                                    $form_data['currency'] = $event->latestVersion->currency_code!=NULL ? $event->latestVersion->currency_code : "EUR";
                                    $form_data['use_tixstock_venue_details'] = 1;
                                   
                                    $form_data['venue']['id'] = isset($event->latestVersion->venue->tixstock_id) && !empty($event->latestVersion->venue->tixstock_id) ? (string)$event->latestVersion->venue->tixstock_id : (string)$event->latestVersion->venue->id;
                                    $form_data['venue']['name'] = $event->latestVersion->venue->name;
                                    $form_data['venue']['address_line_1'] = $address_line_1;//isset($event->venueAddress->address_line_1) && !empty($event->venueAddress->address_line_1) ? $event->venueAddress->address_line_1: "" ;
                                    $form_data['venue']['address_line_2'] = $address_line_2;//$event->venue->address_line_2;
                                    $form_data['venue']['city'] = $event->latestVersion->venue->city;
                                    $form_data['venue']['country_code'] = $event->latestVersion->venue->country;
                                    $form_data['venue']['postcode'] = $post_code;//isset($event->venueAddress->post_code) && !empty($event->venueAddress->post_code) ? $event->venueAddress->post_code: ''; //$event->venue->post_code;
                                   
                                    $venue_details=$category;//VenueCategory::with('sections')->whereIn('id',explode(',',$event->eventcategory->event_categories))->where('venue_id',$event->venue->id)->get(['id','name'])->toArray();
                                
                                    if(!empty($venue_details) && $venue_details != NULL){
                                        //$form_data['venue_details']=$venue_details;
                                    }
                        
                                    if($event->latestVersion->performers->count() > 0){
                                        $k=0;
                                        foreach($event->latestVersion->performers as $performer){
                                            $form_data['performers'][$k]['id'] = isset($performer->external_id) && !empty($performer->external_id) ? (string)$performer->external_id : (string)$performer->id;
                                            $form_data['performers'][$k]['name'] = isset($performer->name) && !empty($performer->name)? $performer->name : '';
                                            $k++;
                                        } 
                                    }
                        
                                    if($event->eventcategory->primary_category != $event->latestVersion->eventcategory->primary_category){
                                        $event->latestVersion->eventcategory->primary_category = $event->eventcategory->primary_category;
                                        $event->latestVersion->search_type = $event->eventcategory->primary_category;
                                        $form_data['category'] = $event->eventcategory->primary_category;
                                    }else{
                                        $form_data['category'] = $event->latestVersion->eventcategory->primary_category;
                                    }
                                    
                                    $form_data['status'] = "Active";
                                    $sources=Sources::where('id',$event->latestVersion->source->id)->first();
                                    $api = Http::withToken($sources->token);
                                    
                                    $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->latestVersion->external_id, $form_data);
                                    $response_arr = $response->json();
                                    
                                    if(!empty($response_arr['data'])){
                                        //store success log into database
                                        $event->latestVersion->mapped_status =1;
                                        $event->latestVersion->save();
                                        
                                        $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->latestVersion->external_id;
                                        $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                                        TixstockLog::create($tixstockLog);
                                        //mapped with resource
                                        EventsMappedSources::create(['event_id'=>$event->id,'source_id'=>$sources->id]);
                                    }else{
                                        $event->latestVersion->save();
                                        //store error log into database
                                        $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->latestVersion->external_id;
                                        $tixstockLog=['event_id'=>$event->id,'batch_id'=>$batch_id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                                        TixstockLog::create($tixstockLog);
                                    }
                                }
                            }
                    }
                }
            }
            
            $successmessage = "";
            $errormessage = "";
            if(!empty($addedEvents)){
                $successmessage  .= "(".implode(',',$addedEvents).") Events Added Successfully. ";
            }
            if(!empty($updatedEvents)){
                $successmessage  .= "(".implode(',',$updatedEvents).") Events Updated Successfully. ";
            }
            if(!empty($errorEvents)){
                $errormessage  .= "(".implode(',',$errorEvents).") Something went wrong with mentioned events.";
            }
            return response()->json(['success'=>true,"successmsg"=>$successmessage,"errormsg"=>$errormessage]);
        }else{
            return response()->json(['success'=>false,"msg"=>"events not found."]);
        }
    }

    public function getEvents($page=null){

        $sources=Sources::where('name','Stubhub')->first();
        if($page==null){
            $page=33;
        }
        echo $page;
        $perPage=50;
        $url="https://sandbox-api.tixstock.com/v1/events/get?mapped_status=false&per_page=50&page=".$page;
        $response=$this->fetchData($url,$sources->token);
        dd($response);
        if(!empty($response['data'])){
            foreach($response['data'] as $events)
            {
                //check venue details insert if needed
                $checkVenue=Venues::where('name',$events['venue']['name'])
                                    ->where('city',$events['venue']['city'])
                                    ->where('post_code',$events['venue']['postcode']);
                                if(!empty($events['venue']['id'])){
                                    $checkVenue=$checkVenue->where('tixstock_id',$events['venue']['id']);
                                }
                $checkVenue=$checkVenue->first();
                if(!$checkVenue){
                    $venueArray=[
                                'tixstock_id'=>$events['venue']['id'],
                                'name'=>$events['venue']['name'],
                                'address_line_1'=>$events['venue']['address_line_1'],
                                'address_line_2'=>$events['venue']['address_line_2'],
                                'city'=>$events['venue']['city'],
                                'state'=>$events['venue']['state'],
                                'post_code'=>$events['venue']['postcode'],
                                'country'=>$events['venue']['country_code'],
                                'source_id'=>$sources->id
                    ];
                    $venue=Venues::create($venueArray);
                    $venueId=$venue->id;
                }else{
                    $venueId=$checkVenue->id;
                }
                //check venue section and category
                if(!empty($events['venue_details'])){
                    foreach($events['venue_details'] as $venueDetail){
                        //check venue category
                        $checkCategory = VenueCategory::where('name',$venueDetail['name'])
                                                        ->where('venue_id',$venueId)
                                                        ->where('source_id',$sources->id)
                                                        ->first();
                            if(!$checkCategory){
                                $categoryArray=[
                                                'name'=>$venueDetail['name'],
                                                'venue_id'=>$venueId,
                                                'venue_category'=>$venueDetail['name'],
                                                'source_id'=>$sources->id
                                            ];
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                            }else{
                                $categoryId = $checkCategory->id;
                            }
                            //check section
                            foreach($venueDetail['sections'] as $section){
                                    $checkSection=VenueSection::where('name',$section['name'])->where('source_id',$sources->id)->first();
                                    if(!$checkSection){
                                        $sectionArray = [
                                                            'name'=>$section['name'],
                                                            'category_id'=>$categoryId,
                                                            'source_id'=>$sources->id
                                                        ];
                                        $section = VenueSection::create($sectionArray);
                                    }
                            }
                    }
                }
                
                //check performers 
                $count=1;
                $performer_1_id=null;
                $performer_2_id=null;
                foreach($events['performers'] as $performer){
                    $checkPerformer=Performers::where('name',$performer['name']);
                    if(!empty($performer['id'])){
                        $checkPerformer=$checkPerformer->where('external_id',$performer['id']);
                    }
                                               
                    $checkPerformer=$checkPerformer->where('source_id',$sources->id)->first();
                    if(!$checkPerformer){
                        $performerArray=[
                                            'name'=>$performer['name'],
                                            'external_id'=>$performer['id'],
                                            'source_id'=>$sources->id,
                        ];
                        $performer = Performers::create($performerArray);
                        if($count==1){
                            $performer_1_id = $performer->id;
                        }
                        else{
                            $performer_2_id= $performer->id;
                        }
                    }else{
                        if($count==1){
                            $performer_1_id=$checkPerformer->id;
                        }else{
                            $performer_2_id = $checkPerformer->id;
                        }
                    }
                    $count++;
                }

                //check events 
                $event_name = isset($events['name']) ? $events['name'] : '';
                $event_date = isset($events['datetime']) ? date ('Y-m-d H:i:s',strtotime($events['datetime'])) : '';
                $event_date_utc = isset($events['datetime']) ? $events['datetime'] : '';

                $eventArray['search_type'] = $events['category']['name'];
                $eventArray['name'] = str_replace("'", "",$event_name);
                $eventArray['performer_1_id'] = $performer_1_id;
                $eventArray['performer_2_id'] = $performer_2_id;
                $eventArray['event_date'] = $event_date;
                $eventArray['event_date_utc'] = $event_date_utc;
                $eventArray['venue_id'] = $venueId;
                $eventArray['source_id']=$sources->id;
                $eventArray['mapped_status']=true;
                $checkEvents=Events::where($eventArray)->first();
               
                if(!$checkEvents){
                    $eventArray['external_id'] = isset($events['id']) ? $events['id'] : '';
                    $newevent = Events::create($eventArray);
                    $eventId = $newevent->id;
                }else{
                    $eventId = $checkEvents->id;
                }

                //check event category
                $checkEventCategory = EventCategory::where('primary_category',$events['category']['name'])->where('event_id',$eventId)->where('source_id',$sources->id)->first();
                if(!$checkEventCategory){
                    $eventCategoryArray=['primary_category'=>$events['category']['name'],'event_id'=>$eventId,'venue_id'=>$venueId,'source_id'=>$sources->id];
                    EventCategory::create($eventCategoryArray);
                }
            }
        $page++;
        $this->getEvents($page);
        }else{
            echo "all data received";
            exit;
        }
        
    }

    public function fetchData($url,$token)
    {

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "authorization: Bearer ".$token,
            "cache-control: no-cache",
            "content-type: application/x-www-form-urlencoded",
            "postman-token: 2dd383dd-1b1d-af80-96c9-c8ee7195cff0"
        ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return $err;
        } else {
            return json_decode($response,true);
        }
    }
}
