<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DataTables;
use App\User;
use App\Roles;
use Hash;
class UserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
        return view('admin.users.index');
    }

    public function data(){

        $users=User::get();
        
        return Datatables::of($users)
        ->rawColumns(['actions','status','role'])
        ->editColumn('role', function ($users) {
            return isset($users->role) ? $users->role:'';
        })
        ->editColumn('status', function ($users) {
             if($users->status == 0){
                 return '<label class="label text-success" >Active</label>';
             } else {
                 return '<label class="label text-danger">InActive</label>';
             }
         })
        ->addColumn('actions', function (User $users) {

         $edit='<a href="'.route('user.edit',$users->id).'" class="btn btn-sm edit-role bg-green btn-flat" title="Edit Role"><i class="fa fa-edit"></i></a>';
         $delete='<a href="#" id="'. $users->id.'"  data-src="'.route('user.destroy', $users->id).'" class="btn btn-sm bg-red btn-flat delete" title="Delete Role"><i class="fa fa-trash"></i></a>';
         $action = $edit.' '.$delete.' ';
         return $action;
        })
        ->make(true);
     }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        $roles=Roles::where('status',0)->get();
        
        return view('admin.users.create',compact('roles'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $rules = [
            'name'            => 'required',
            'email'           => 'required|email|unique:users,email',
            'password'        => 'required|same:confirm_password',
            'user_status'     => 'required',
            'user_role'       => 'required'
        
        ];
        $this->validate($request, $rules);
        $input= [
            'name'=>$request->name,
            'email'=>$request->email,
            'password'=>Hash::make($request->password),
            'role_id'=>$request->user_role,
            'status'=>$request->user_status
        ];

        User::create($input);
        return response()->json(['success'=>true,'msg'=>'User Created Successfully']);

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $user=User::where('id',$id)->first();
        $roles=Roles::where('status',0)->get();
        return view('admin.users.edit',compact('user','roles'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
       
        $rules = [
            'name'            => 'required',
            'email'           => 'required|email',
            'user_status'     => 'required',
            'user_role'       => 'required'
        
        ];
        if($request->password && !empty($request->password)){
            $rules['password']=['required|same:confirm_password'];
        }
        $this->validate($request, $rules);
        $input= [
            'name'=>$request->name,
            'email'=>$request->email,
            'password'=>Hash::make($request->password),
            'role_id'=>$request->user_role,
            'status'=>$request->user_status
        ];
        if($request->password && !empty($request->password)){
            $input['password']=Hash::make($request->password);
        }

        User::where('id',$id)->update($input);
        return response()->json(['success'=>true,'msg'=>'User Updated Successfully']);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
        User::where('id',$id)->delete();
        return response()->json(['success'=>true,'msg'=>'User Deleted Successfully']);
    }
}
