<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DataTables;
use App\Venues;
use App\VenueCategory;
use App\VenueSection;
use App\VenueAdditionalInfo;
use App\Maps;
use App\VenueMaps;
use App\Categories;
use App\Events;
class VenueController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
        $countryCodes = Events::COUNTRY_CODES;
        return view('admin.venue.index',compact('countryCodes'));
    }

    public function data(Request $request){
        $venues = Venues::where('name','!=','');
        
        if(isset($request->name) && !empty($request->name)){
            $venues = Venues::where('name','like','%'.$request->name.'%');
        }
        
        if(isset($request->country) && !empty($request->country)){
            $venues = Venues::where('country',$request->country);
        }
        
        
        
        return Datatables::eloquent($venues)
        ->rawColumns(['action','category_section','edit','address','maps'])
        ->addColumn('address', function ($venues) {
            // return $venues->address_line_1.' '.$venues->address_line_2;
             $html='<button type="button" id="'.$venues->id.'" data-source="'.$venues->source_id.'" class="ladda-button btn btn-warning waves-effect waves-light address_detail_btn" dir="ltr" data-style="expand-right" title="View Detail" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            return $html;
        })
        ->editColumn('postcode', function ($venues) {
            return $venues->post_code;
        })
        ->addColumn('category_section', function ($venues) {
            $html='<button type="button" id="'.$venues->id.'" data-source="'.$venues->source_id.'" class="ladda-button btn btn-danger waves-effect waves-light category_section_detail_btn" dir="ltr" data-style="expand-right" title="View Detail" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            return $html;
        })
         ->addColumn('edit', function (Venues $venues) {
            $html = '';//'<a href="'.route('venue.edit',$venues->id).' " class="action-icon edit_icon"> <i class="mdi mdi-square-edit-outline"></i></a>';
            return $html;
        })
        ->addColumn('action', function (Venues $venues) {

            $html = '<button id="'.$venues->id.'" title="Add Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" class="btn btn-blue mb-2"><i class="mdi mdi-plus-circle"></i></a>';
            $action = $html;
            $action = '';
            return $action;
        })
        
        ->addColumn('maps', function (Venues $venues) {

            $html='<button type="button" id="'.$venues->id.'" data-source="'.$venues->source_id.'" class="ladda-button btn btn-primary waves-effect waves-light map_btn" dir="ltr" data-style="expand-right" title="View Maps" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-map"></i></button>';
            return $html;
        })
        ->make(true);
    }

    public function getVenueCategorySectionData(Request $request){
        
        // $venueCategorySection = VenueCategory::with('section')->where('venue_id',$request->id)->get();
        $venueId=$request->id;
        $input = $request->all();
        $category = VenueCategory::with('section')->where('venue_id',$request->id)->where('source_id','!=',0)->get();
        $tixstockCategory = VenueCategory::with('section')->where('venue_id',$request->id)->where('source_id',0)->get();
        $manual_event = isset($input['manual_event']) && !empty($input['manual_event']) ? $input['manual_event'] : '';
        $manual_venue = isset($input['manual_venue']) && !empty($input['manual_venue']) ? $input['manual_venue'] : '';
        $html = view('admin.venue.ajax.venue_category_section',compact('category','tixstockCategory','venueId','manual_event','manual_venue'))->render();
        // $venueCategorySection =  $category;
       
        // $html= view('admin.category.ajax.category_section',compact('venueCategorySection','manual_event'))->render();
        return response()->json(['html'=>$html,'id'=>$request->id]);

    }
    
    
    public function getSingleCategorySection(Request $request){
        if(isset($request->edit) && !empty($request->edit)){
               $tixstockCategory = VenueCategory::with('section')->where('id',$request->id)->get();
        }else{
            $tixstockCategory = VenueCategory::with('section')->where('parent_category_id',$request->id)->where('source_id',0)->get();
            if($tixstockCategory->count() <= 0){
                $tixstockCategory = VenueCategory::with('section')->where('id',$request->id)->where('source_id','!=',0)->get();
            }
        }
 
       $html = view('admin.venue.ajax.single_category_section',compact('tixstockCategory'))->render();
        return response()->json(['html'=>$html,'id'=>$request->id]);
    }
    
    public function storeSingleCategorySection(Request $request){
        $parent_category_id= $request->parent_category_id;
        $parentCategory=VenueCategory::where('id',$parent_category_id)->first();
       
        foreach($request->newly_add_category as $key=>$category){
            if($key == $parent_category_id){
                $catArray = explode('/',$category);
                $categoryArray= [
                        'name'=>isset($catArray[0]) && !empty($catArray[0]) ? $catArray[0]:'',
                        'venue_id'=>$parentCategory->venue_id,
                        'venue_category'=>$category,
                        'source_id'=>0,
                        'parent_category_id'=>$parent_category_id,
                        'stubhub_name'=>isset($catArray[1]) && !empty($catArray[1]) ? $catArray[1]:'',
                    ];
                $storedCategory = VenueCategory::create($categoryArray);
            }else{
                $catArray = explode('/',$category);
                $categoryArray=[
                    'name'=>isset($catArray[0]) && !empty($catArray[0]) ? $catArray[0]:'',
                    'venue_id'=>$parentCategory->venue_id,
                    'stubhub_name'=>isset($catArray[1]) && !empty($catArray[1]) ? $catArray[1]:''
                    ];
                VenueCategory::where('id',$key)->update($categoryArray);
            }
            
            foreach($request->newly_add_section[$key] as $section){
                if($key == $parent_category_id){
                    $secArray=explode('/',$section);
                    $sectionArray=[
                        'name'=>isset($secArray[0]) && !empty($secArray[0]) ? $secArray[0]:'',
                        'category_id'=>$storedCategory->id,
                        'source_id'=>0,
                        'stubhub_name'=>isset($secArray[1]) && !empty($secArray[1]) ? $secArray[1]:''
                    ];
                    VenueSection::create($sectionArray);
                }else{
                    VenueSection::where('category_id',$key)->delete();
                    foreach($request->newly_add_section[$key] as $section){
                        $secArray=explode('/',$section);
                         $sectionArray=[
                        'name'=>isset($secArray[0]) && !empty($secArray[0]) ? $secArray[0]:'',
                        'category_id'=>$key,
                        'source_id'=>0,
                        'stubhub_name'=>isset($secArray[1]) && !empty($secArray[1]) ? $secArray[1]:'',
                    ];
                    VenueSection::create($sectionArray);
                    }
                }
            }
        }
        return response()->json(['success'=>true,'msg'=>"Tixstock's version for category saved."]);
    }
    
    public function editSingleCategorySection(Request $request){
        $parent_category_id= $request->parent_category_id;
        $parentCategory=VenueCategory::where('id',$parent_category_id)->first();
        
        foreach($request->newly_add_category as $key=>$category){
            
            $catArray = explode('/',$category);
            $categoryDetail=VenueCategory::where('id',$key)->first();
            $categoryDetail->name=isset($catArray[0]) && !empty($catArray[0]) ? $catArray[0]:'';
            $categoryDetail->stubhub_name=isset($catArray[1]) && !empty($catArray[1]) ? $catArray[1]:'';
            $categoryDetail->save();
    
            foreach($request->newly_add_section[$key] as $key1=>$section){
                
                $secArray = explode('/',trim($section));
                if(!isset($secArray[0]) && empty($secArray[0])){
                    continue;
                }
                
                $sectionDetail = VenueSection::where('id',$key1)->where('category_id',$key)->first();
              
                if($sectionDetail){
                    $sectionDetail->name = isset($secArray[0]) && !empty($secArray[0]) ? $secArray[0]:'';
                    $sectionDetail->stubhub_name = isset($secArray[1]) && !empty($secArray[1]) ? $secArray[1]:'';
                    $sectionDetail->save();
                }else{
                    $sectionArray = [
                            'name'=> isset($secArray[0]) && !empty($secArray[0]) ? $secArray[0]:'',
                            'stubhub_name'=>isset($secArray[1]) && !empty($secArray[1]) ? $secArray[1]: '',
                            'category_id'=>$key
                        ];
                    VenueSection::create($sectionArray);
                    
                }
                
            }
        }
        return response()->json(['success'=>true,'msg'=>"Tixstock's version for category saved."]);
    }
    
    
    public function getAddressDetails(Request $request){
         $venueId=$request->id;
         $input = $request->all();
         $countryCodes = Events::COUNTRY_CODES;
         $address = VenueAdditionalInfo::where('venue_id',$venueId)->get();
         $venue = Venues::where('id',$venueId)->first();
         
         $html = view('admin.venue.ajax.address',compact('address','venueId','venue','countryCodes'))->render();
         return response()->json(['html'=>$html,'id'=>$request->id]);
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        return view('admin.venue.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        //
        $rules = [
            'name'         => 'required',
            'city'         => 'required',
            // 'postcode'     => 'required',
            // 'address'      => 'required',
            'state'        => 'required',
            'country'      => 'required'
        
        ];
        $this->validate($request, $rules);

        /*        $category = strip_tags(trim($request->category));
        $category = explode('\n',$category);
        $category = array_filter($category,'trim');
        $sendCategoryId = [];
        foreach($category as $c){
            $check = VenueCategory::where('name',trim($c))->first();
            if(!$check){
                $venueCategory=VenueCategory::create(['name'=>$c]);
                $venueCategoryId = $venueCategory->id;
            }
            else{
                $venueCategoryId = $check->id;
            }
            $sendCategoryId[]= $venueCategoryId;
        }

        if(!empty($sendCategoryId)){
            $sendCategoryId = implode(',',array_unique($sendCategoryId));
        }

        $sendSectionId = [];
        $section = trim($request->section);
        $section = explode('\n',$section);
        $section = array_filter($section,'trim');

        foreach($section as $s){
            
            $venueSection = VenueSection::where('name',$s)->first();
            if(!$venueSection){
                $venueSection = VenueSection::create(['name'=>$s]);
                $venueSectionId = $venueSection->id;
            }else{
                $venueSectionId = $venueSection->id;
            }

            $sendSectionId[] = $venueSectionId;
        }

        if(!empty($sendSectionId)){
            $sendSectionId = implode(',',array_unique($sendSectionId));
        }
        */
        $input= [
            'name'=>$request->name,
            'city'=>$request->city,
            'state'=>$request->state,
            // 'address_line_1'=>$request->address,
            'country'=>$request->country,
            'source_id'=>1
            // 'postcode'=>$request->postcode,
        ];

        Venues::create($input);
        return response()->json(['success'=>true,'msg'=>'Venue Created Successfully']);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $venue = Venues::with(['category','category.section'])->where('id',$id)->first();
        return view('admin.venue.edit',compact('venue'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
       
        //
        $rules = [
            // 'category'     => 'required',
            // 'section'      => 'required',
            'name'         => 'required',
            // 'city'         => 'required',
            // 'postcode'     => 'required',
            // 'address'      => 'required',
            // 'state'        => 'required',
            // 'country'      => 'required'
        
        ];
        $this->validate($request, $rules);

        //Check category 
        /*
        $category = strip_tags(trim($request->category));
        $category = explode('\n',$category);
        $category = array_filter($category,'trim');
        $sendCategoryId = [];
        foreach($category as $c){
            $check = VenueCategory::where('name',trim($c))->first();
            if(!$check){
                $venueCategory=VenueCategory::create(['name'=>$c]);
                $venueCategoryId = $venueCategory->id;
            }
            else{
                $venueCategoryId = $check->id;
            }
            $sendCategoryId[]= $venueCategoryId;
        }

        if(!empty($sendCategoryId)){
            $sendCategoryId = implode(',',array_unique($sendCategoryId));
        }

        //check section 
        $sendSectionId = [];
        $section = trim($request->section);
        $section = explode('\n',$section);
        $section = array_filter($section,'trim');

        foreach($section as $s){
            
            $venueSection = VenueSection::where('name',$s)->first();
            if(!$venueSection){
                $venueSection = VenueSection::create(['name'=>$s]);
                $venueSectionId = $venueSection->id;
            }else{
                $venueSectionId = $venueSection->id;
            }

            $sendSectionId[] = $venueSectionId;
        }

        if(!empty($sendSectionId)){
            $sendSectionId = implode(',',array_unique($sendSectionId));
        }
        */
        $input= [
            'name'=>$request->name,
            'city'=>$request->city,
            'state'=>$request->state,
            'country'=>$request->country,
            
        ];

        Venues::where('id',$id)->update($input);
        if(!empty($request->venue_address)){
            foreach($request->venue_address as $key=>$value){
                $venueUpdate =[
                        'address_line_1'=>$value['address_line_1'],
                        'address_line_2'=>$value['address_line_2'],
                        'post_code'=>$value['post_code']
                    ];
                VenueAdditionalInfo::where('id',$key)->update($venueUpdate);
            }
        }
        
        if(!empty($request->new_venue_address)){
            for($i=0;$i<sizeof($request->new_venue_address['address_line_1']);$i++){
                $venueAddress = [
                        'address_line_1'=>$request->new_venue_address['address_line_1'][$i],
                        'address_line_2'=>$request->new_venue_address['address_line_2'][$i],
                        'post_code'=>$request->new_venue_address['post_code'][$i],
                        'venue_id'=>$id
                    ];
                    
                VenueAdditionalInfo::create($venueAddress);
            }
        }
        return response()->json(['success'=>true,'msg'=>'Venue Updated Successfully']);
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function deleteSection(Request $request){
        
        VenueSection::where('id',$request->id)->delete();
        $response=['success'=>true,'msg'=>"Section deleted successfully."];
        return response()->json($response);
    }

    public function deleteCategory(Request $request){
        
        $category=Venuecategory::where('id',$request->id)->first();
        $category->section()->delete();
        $category->delete();
        
        $response=['success'=>true,'msg'=>"Category deleted successfully."];
        return response()->json($response);
    }
    
    
    public function addNewAddress(Request $request){
       $rules = [
            // 'category'     => 'required',
            // 'section'      => 'required',
            'address_line_1' => 'required|trim',
            'address_line_2' => 'required|trim',
            'postal_code'    =>'required|trim'
        
        ];
        $this->validate($request, $rules);
    }
    
    public function getVenueAddressDetails(Request $request){
        $venueAddress = VenueAdditionalInfo::where('venue_id',$request->id)->get();
        if($venueAddress->count() > 0){
            $html='<select name="venue_address" id="venue_address" class="form-control select2 venue_address" >';
            $html.='<option value="">Please select address</option>';
            $i=1;
            foreach($venueAddress as $address){
                $addressName="";
                $addressName .= isset($address->address_line_1) && !empty($address->address_line_1) ? $address->address_line_1.', ' : '';
                $addressName .= isset($address->address_line_2) && !empty($address->address_line_2) ? $address->address_line_2.', ' : '';
                $addressName .= isset($address->post_code) && !empty($address->post_code) ? $address->post_code : '';
                if($i==1){
                    $html.='<option value="'.$address->id.'" selected>'.$addressName.'</option>';
                }else{
                    $html.='<option value="'.$address->id.'">'.$addressName.'</option>';
                }
                $i++;
            }
            $html.='</select>';
            return response()->json(['html'=>$html]);
        }
    }
    
    public function storeTixstockCategory(Request $request){
        
     
        foreach($request->newly_add_category as $key=>$category){
           
                $catArray = explode('/',$category);
                $categoryArray=[
                    'name'=>isset($catArray[0]) && !empty($catArray[0]) ? $catArray[0]:'',
                    'venue_id'=>$request->venue_id,
                    'venue_category'=>$category,
                    'source_id'=>0,
                    'stubhub_name'=>isset($catArray[1]) && !empty($catArray[1]) ? $catArray[1]:''
                    ];
              
                $venueCategory = VenueCategory::create($categoryArray);
      
            if(!empty($request->newly_add_section[$key])){  
                    foreach($request->newly_add_section[$key] as $section){
                        $secArray=explode('/',$section);
                         $sectionArray=[
                            'name'=>isset($secArray[0]) && !empty($secArray[0]) ? $secArray[0]:'',
                            'category_id'=>$venueCategory->id,
                            'source_id'=>0,
                            'stubhub_name'=>isset($secArray[1]) && !empty($secArray[1]) ? $secArray[1]:'',
                        ];
                        VenueSection::create($sectionArray);
                    }
            }
        }
        return response()->json(['success'=>true,'msg'=>"Tixstock's version for category saved."]);
    }
    
    public function deleteVenueAddress(Request $request){
        
        VenueAdditionalInfo::where('id',$request->id)->delete();
        return response()->json(['success'=>true,'msg'=>"Address deleted successfully."]);
    }
    
    public function getMapListing(Request $request){
        
        $venueId = $request->id;
        $venueMaps = VenueMaps::with('maps')->where('venue_id',$request->id)->get();
        $maps  = Maps::orderBy('name')->get();
        $html = view('admin.venue.ajax.map_listing',compact('venueMaps','maps','venueId'))->render();
        return response()->json(['success'=>true,'html'=>$html]);
    }
    
    public function assignMapToVenue(Request $request){
        
        //check venue is associated maps
        $check = VenueMaps::where('venue_id',$request->venue_id)->where('map_id',$request->map)->first();
        if(!$check){
            VenueMaps::create(['venue_id'=>$request->venue_id,'map_id'=>$request->map]);
            return response()->json(['success'=>true,'msg'=>"Map is assigned to the venue."]);
        }else{
            return response()->json(['success'=>false,'msg'=>"Map is already associated with map."]);
        }
    }
    
    public function createMap($id){
        $venueId = $id;
        $categories = VenueCategory::where('venue_id',$id)->orderBy('name')->groupBy('name')->get();
        $categoriesTree = Categories::get();
        return view('admin.maps.create',compact('categories','categoriesTree','venueId'));
    }
    
    public function editMap($id){
        
        $venueMaps = VenueMaps::with('maps')->where('id',$id)->first();
        $venueId = $venueMaps->venue_id;
        $map = Maps::where('id',$venueMaps->map_id)->first();
        $content = file_get_contents($map->url);
        $categories = VenueCategory::with('section')->where('venue_id',$venueMaps->venue_id)->orderBy('name')->groupBy('name')->get();
        $catSection = [];
        foreach($categories as $category){
            $catSection[$category->name] = [];
            foreach($category->section as $section){
                $catSection[$category->name][] = $section->name;
            }
        }
        
        $categoriesTree = Categories::get();
        return view('admin.maps.tivo_maps',compact('categories','categoriesTree','venueId','content','map','catSection'));
    }
    public function deleteMap(Request $request){
        $input = $request->all();
        VenueMaps::where('id',$input['id'])->delete();
        return response()->json(['success'=>true,'msg'=>'Map is deleted successfully.']);
    }
    
    public function deleteBulkCategorySection(Request $request){
        $input = $request->all();
        
        VenueCategory::whereIn('id',explode(',',$input['categories']))->delete();
        VenueSection::whereIn('id',explode(',',$input['sections']))->delete();
        return response()->json(['success'=>true,'msg'=>'Categories deleted successfully.']);
    }
}
