<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\TixstockLog;
use App\Http\Controllers\API\StubhubController as StubhubApi;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use App\OldCategories;
use App\SubCategories;
use App\VenueMaps;
use App\Maps;
use Carbon\Carbon;
class ProcessEventsJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;
    public $events = [];
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($events)
    {
        //
        $this->events = $events;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        //
        if(!empty($this->events)){
            $eventIds = $this->events;
            $date = Carbon::today()->format('dmY');

            // Count the number of logs for today with the same batch ID
            $lastLog = TixstockLog::whereDate('created_at', today())
                                    ->where('batch_id', 'like', $date . '%')
                                    ->orderBy('id','desc')
                                    ->first();
            
            if($lastLog){
                $lastBatchId = explode('-',$lastLog->batch_id);
                $count = $lastBatchId[1]+1;
            }else{
                $count= 1;
            }

            // Set the batch ID
            $batch_id = $date . '-' . $count;
            
            if(!empty($eventIds)){
                foreach($eventIds as $eventId){
                
                
                $sources=Sources::where('name','Tixstock')->first();
                $api = Http::withToken($sources->token);
                
                $event = "";
                $event = Events::with(['venue','performers','venueAddress', 'eventcategory' ,'performer1', 'performer2','source','latestVersion'])->where('id', $eventId)->first();
                echo "Processing Event:".$event->external_id."\r\n";
                
                $eventCategory = isset($event->eventcategory->event_categories) && !empty($event->eventcategory->event_categories) ? $event->eventcategory->event_categories : [];
                $eventSection = isset($event->eventcategory->event_section) && !empty($event->eventcategory->event_section) ? $event->eventcategory->event_section : [];
                
                $category = [];
                if(!empty($eventCategory)){
                    $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                      return $query->whereIn('id',explode(',',$eventSection));
                    }])->whereIn('id',explode(',',$eventCategory))->where('venue_id',$event->venue_id)->get(['id','external_id','name','stubhub_name'])->toArray();
                }
                
                $removeId = function ($item) use (&$removeId) {
                    if($item['external_id'] != ""){
                        $item['id']=(string)$item['external_id'];
                    }else{
                        $item['id'] = (string)$item['id'];
                    }
                    
                    if (isset($item['sections'])) {
                        $item['sections'] = array_map($removeId, $item['sections']);
                    }
                    unset($item['external_id']);
                    return $item;
                };
                
                $category = array_map($removeId, $category);
                
              
                if ($event) {
                        
                        $address_line_1="";
                        $address_line_2="";
                        $post_code="";
                        
                        if(!empty($event->venue_address_id)){
                            $address_line_1=$event->venueAddress->address_line_1;
                            $address_line_2=$event->venueAddress->address_line_2;
                            $post_code= $event->venueAddress->post_code;
                        }else{
                            if($event->venue->venueAdditionalInfo->count() > 0){
                                $address_line_1 = $event->venue->venueAdditionalInfo[0]->address_line_1;
                                $address_line_2 = $event->venue->venueAdditionalInfo[0]->address_line_2;
                                $post_code = $event->venue->venueAdditionalInfo[0]->post_code;
                            }
                        }
                        //maps logic
                        $map_url="";
                        if(!empty($event->map_id)){
                            $map = Maps::where('id',$event->map_id)->first();
                            $map_url = $map->url;
                        }
                        if(empty($map_url)){
                            $maps = VenueMaps::with('maps')->where('venue_id',$event->venue_id)->first();
                            if($maps){
                                $map_url = $maps->maps->url;
                            }
                        }
                    
                        $form_data = [];
                        $form_data['id'] = (string)$event->external_id;
                        $form_data['name'] = $event->name;
                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->event_date));
                        $form_data['currency'] = $event->currency_code!=NULL ? $event->currency_code : "EUR";
                        $form_data['use_tixstock_venue_details'] = 0;
                       
                        $form_data['venue']['id'] = isset($event->venue->tixstock_id) && !empty($event->venue->tixstock_id) ? (string)$event->venue->tixstock_id : (string)$event->venue->id;
                        $form_data['venue']['name'] = $event->venue->name;
                        $form_data['venue']['address_line_1'] = $address_line_1;//isset($event->venueAddress->address_line_1) && !empty($event->venueAddress->address_line_1) ? $event->venueAddress->address_line_1: "" ;
                        $form_data['venue']['address_line_2'] = $address_line_2;//$event->venue->address_line_2;
                        $form_data['venue']['city'] = $event->venue->city;
                        $form_data['venue']['country_code'] = $event->venue->country;
                        $form_data['venue']['postcode'] = $post_code;//isset($event->venueAddress->post_code) && !empty($event->venueAddress->post_code) ? $event->venueAddress->post_code: ''; //$event->venue->post_code;
                        $form_data['map_url'] = $map_url;
                        $venue_details=$category;//VenueCategory::with('sections')->whereIn('id',explode(',',$event->eventcategory->event_categories))->where('venue_id',$event->venue->id)->get(['id','name'])->toArray();
                        
                        if(!empty($venue_details) && $venue_details != NULL){
                            $form_data['venue_details']=$venue_details;
                        }
                        
                        if($event->performers->count() > 0){
                            $k=0;
                            foreach($event->performers as $performer){
                                $form_data['performers'][$k]['id'] = isset($performer->external_id) && !empty($performer->external_id) ? (string)$performer->external_id : (string)$performer->id;
                                $form_data['performers'][$k]['name'] = isset($performer->name) && !empty($performer->name)? $performer->name : '';
                                $k++;
                            } 
                        }
                       
                        $form_data['category'] = $event->eventcategory->primary_category;
                        
                        // $form_data['category']=!empty($event->eventcategory->secondary_category) && $event->eventcategory->secondary_category != NULL ? $event->eventcategory->secondary_category : $event->eventcategory->primary_category ;
                        $form_data['status'] = "Active";
            
                  
                    if($event->mapped_status == 0 && $event->update_requires == 0){
                        $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                        
                        $response_arr = $response->json();
                        $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->external_id;
                        
                        if(!empty($response_arr['data'])){
                            //store success log into database
                            $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $addedEvents [] = $event->external_id;
                            $event->mapped_status=1;
                                $event->save();
                        }else{
                           // store error log in to database
                            $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $errorEvents[] = $event->external_id;
                        }
                        
                            //send using stubhub,ticket evolution token
                            
                            if(!empty($eventCategory)){
                                $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                                  return $query->whereIn('id',explode(',',$eventSection));
                                }])->whereIn('id',explode(',',$eventCategory))->where('stubhub_name',NULL)->where('parent_category_id',NULL)->where('venue_id',$event->venue_id)->get(['id','name','stubhub_name'])->toArray();
                            }
                            if(!empty($category) && $category != NULL){
                                $form_data['venue_details']=$category;
                            }
                            $form_data['use_tixstock_venue_details'] = 1;
                            unset($form_data['venue_details']);
                            if($event->source_id==0){
                                    continue;
                                }
                                
                            $sources=Sources::where('id',$event->source->id)->first();
                            $api = Http::withToken($sources->token);
                            unset($form_data['url']);
                            $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                            $response_arr = $response->json();
                            
                            if(!empty($response_arr['data'])){
                                //store success log into database
                                $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->external_id;
                                $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                                TixstockLog::create($tixstockLog);
                            }else{
                                //store error log into database
                                $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                                TixstockLog::create($tixstockLog);
                            } 
                    }
                    
                    if($event->update_requires == 1){
                         $response = $api->post(env('TIXSTOCK_URL').'/events/update/'.$event->external_id, $form_data);
                         
                         $response_arr = $response->json();
                       
                         $form_data['url'] = env('TIXSTOCK_URL').'/events/update/'.$event->external_id;
                         if(!empty($response_arr['data'])){
                             //store log in to database
                             $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                             TixstockLog::create($tixstockLog);
                             $updatedEvents[] = $event->external_id;
                             $event->update_requires = 0;
                             $event->mapped_status =1;
                             $event->save(); 
                         }else{
                             //store error log into database
                             $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                             TixstockLog::create($tixstockLog);
                             $errorEvents[] =$event->external_id; 
                         }
                         
                         
                        if( $event->source_id == 0 || (!empty($event->source) && $event->source->name=="Tixstock")){ 
                            
                                if(!empty($event->latestVersion) && $event->latestVersion->mapped_status==0 && $event->latestVersion->source->name == "Stubhub"){
                                    //move event to tixstock with stubhub token
                                    $eventCategory = isset($event->latestVersion->eventcategory->event_categories) && !empty($event->latestVersion->eventcategory->event_categories) ? $event->latestVersion->eventcategory->event_categories : [];
                                    $eventSection = isset($event->latestVersion->eventcategory->event_section) && !empty($event->latestVersion->eventcategory->event_section) ? $event->latestVersion->eventcategory->event_section : [];
                                    $category = [];
                                    if(!empty($eventCategory)){
                                        $category = VenueCategory::with(['sections'=>function($query) use ($eventSection){
                                          return $query->whereIn('id',explode(',',$eventSection));
                                        }])->whereIn('id',explode(',',$eventCategory))->where('venue_id',$event->latestVersion->venue_id)->get(['id','external_id','name','stubhub_name'])->toArray();
                                    }
                                    
                                    $removeId = function ($item) use (&$removeId) {
                                        if($item['external_id'] != ""){
                                            $item['id']=(string)$item['external_id'];
                                        }else{
                                            $item['id'] = (string)$item['id'];
                                        }
                                        
                                        if (isset($item['sections'])) {
                                            $item['sections'] = array_map($removeId, $item['sections']);
                                        }
                                        unset($item['external_id']);
                                        return $item;
                                    };
                                    
                                    $category = array_map($removeId, $category);
                                    
                                    $address_line_1="";
                                    $address_line_2="";
                                    $post_code="";
                        
                                    if(!empty($event->latestVersion->venue_address_id)){
                                        $address_line_1=$event->latestVersion->venueAddress->address_line_1;
                                        $address_line_2=$event->latestVersion->venueAddress->address_line_2;
                                        $post_code= $event->latestVersion->venueAddress->post_code;
                                    }else{
                                        if($event->latestVersion->venue->venueAdditionalInfo->count() > 0){
                                            $address_line_1 = $event->latestVersion->venue->venueAdditionalInfo[0]->address_line_1;
                                            $address_line_2 = $event->latestVersion->venue->venueAdditionalInfo[0]->address_line_2;
                                            $post_code = $event->latestVersion->venue->venueAdditionalInfo[0]->post_code;
                                        }
                                    }
                                    $form_data = [];
                                    $form_data['id'] = (string)$event->latestVersion->external_id;
                                    if($event->name != $event->latestVersion->name){
                                        $event->latestVersion->name = $event->name;
                                        $form_data['name'] = $event->name;
                                    }else{
                                        $form_data['name'] = $event->latestVersion->name;
                                    }
                                    //$form_data['name'] = $event->latestVersion->name;
                                    //$form_data['datetime'] = date(DATE_ISO8601, strtotime($event->latestVersion->event_date));
                                    
                                    if($event->event_date != $event->latestVersion->event_date){
                                        $event->latestVersion->event_date = $event->event_date;
                                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->event_date));
                                    }else{
                                        $form_data['datetime'] = date(DATE_ISO8601, strtotime($event->latestVersion->event_date));
                                    }
                                    
                                    $form_data['currency'] = $event->latestVersion->currency_code!=NULL ? $event->latestVersion->currency_code : "EUR";
                                    $form_data['use_tixstock_venue_details'] = 1;
                                   
                                    $form_data['venue']['id'] = isset($event->latestVersion->venue->tixstock_id) && !empty($event->latestVersion->venue->tixstock_id) ? (string)$event->latestVersion->venue->tixstock_id : (string)$event->latestVersion->venue->id;
                                    $form_data['venue']['name'] = $event->latestVersion->venue->name;
                                    $form_data['venue']['address_line_1'] = $address_line_1;//isset($event->venueAddress->address_line_1) && !empty($event->venueAddress->address_line_1) ? $event->venueAddress->address_line_1: "" ;
                                    $form_data['venue']['address_line_2'] = $address_line_2;//$event->venue->address_line_2;
                                    $form_data['venue']['city'] = $event->latestVersion->venue->city;
                                    $form_data['venue']['country_code'] = $event->latestVersion->venue->country;
                                    $form_data['venue']['postcode'] = $post_code;//isset($event->venueAddress->post_code) && !empty($event->venueAddress->post_code) ? $event->venueAddress->post_code: ''; //$event->venue->post_code;
                                   
                                    $venue_details=$category;//VenueCategory::with('sections')->whereIn('id',explode(',',$event->eventcategory->event_categories))->where('venue_id',$event->venue->id)->get(['id','name'])->toArray();
                                
                                    if(!empty($venue_details) && $venue_details != NULL){
                                        //$form_data['venue_details']=$venue_details;
                                    }
                        
                                    if($event->latestVersion->performers->count() > 0){
                                        $k=0;
                                        foreach($event->latestVersion->performers as $performer){
                                            $form_data['performers'][$k]['id'] = isset($performer->external_id) && !empty($performer->external_id) ? (string)$performer->external_id : (string)$performer->id;
                                            $form_data['performers'][$k]['name'] = isset($performer->name) && !empty($performer->name)? $performer->name : '';
                                            $k++;
                                        } 
                                    }
                        
                                    //$form_data['category'] = $event->latestVersion->eventcategory->primary_category;
                                    if($event->eventcategory->primary_category != $event->latestVersion->eventcategory->primary_category){
                                        $event->latestVersion->eventcategory->primary_category = $event->eventcategory->primary_category;
                                        $event->latestVersion->search_type = $event->eventcategory->primary_category;
                                        $form_data['category'] = $event->eventcategory->primary_category;
                                    }else{
                                        $form_data['category'] = $event->latestVersion->eventcategory->primary_category;
                                    }
                                    
                                    $form_data['status'] = "Active";
                                    $sources=Sources::where('id',$event->latestVersion->source->id)->first();
                                    $api = Http::withToken($sources->token);
                                    
                                    $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                                    $response_arr = $response->json();
                                    
                                    if(!empty($response_arr['data'])){
                                        //store success log into database
                                        $event->latestVersion->mapped_status =1;
                                        $event->latestVersion->save();
                                        $form_data['url'] = env('TIXSTOCK_URL').'/events/add/'.$event->external_id;
                                        $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                                        TixstockLog::create($tixstockLog);
                                    }else{
                                        //store error log into database
                                        $event->latestVersion->save();
                                        $tixstockLog=['batch_id'=>$batch_id,'event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                                        TixstockLog::create($tixstockLog);
                                    }
                                }
                            }
                    }
                }
            }
            }else{
                echo "Events not found.";
            }
        }
        
    }
}
