<?php 
namespace App\Traits;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\Logs;
use Illuminate\Support\Facades\Log;
use App\Traits\ApiTrait;
use App\Http\Controllers\API\StubhubController as StubhubApi;
use App\Http\Controllers\API\TicketmasterController as TicketmasterApi;
use App\Http\Controllers\API\TicketEvolutionController as TivoApi;
use App\Http\Controllers\CategoryController;
use App\VenueAdditionalInfo;
use App\SourceCategoryMapping;
use App\Categories;
use App\VenueMaps;
use App\EventPerformers;
trait StubhubTrait{ 
    use ApiTrait;
    
   
    public function fetchStubhubData($resource,$slug){
       
        if(in_array($slug,['Sports','Music','UpdateData'])){
            $this->fetchStubhubEvents($resource,$slug);
        }
        
        if(in_array($slug,['Zones'])){
            //fetch zones
            $this->fetchStuhubhubEventsZones($resource,$slug);
        }
    }

    public function fetchStubhubEvents($resource,$slug){
        
        $this->processData($resource,$slug,null);
      
    }
    
    public function processData($resource,$slug,$start=null){
        
        date_default_timezone_set('Europe/London');
        $date="";
        $countResults = 0;
        $totalResults = 0;
        $createdEvents=[];
        $updatedEvents=[];
        $day = "";
        $categoriesArray= [];
        
        
        $sourceCategoryMapping = new CategoryController();
        $categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
        if($slug=="UpdateData"){
            $days = 5;
            $date= date('Y-m-d', strtotime(' - '.$days.' days'));
        }
        
        if($date==""){
            $date=date('Y-m-d');
            $start = Events::where('search_type','Music')->where('mapped_status',0)->get()->count();
            Log::info($start." fetched on ".$date);
            $url=env('STUBHUB_URL')."?categoryName=".$slug."&rows=500&start=".$start;
        }else{
            if($start == null){
                $start=0;
            }
            
            $stubhubapi = new StubhubApi();
            $url=env('STUBHUB_API_URL')."".$stubhubapi::URL_GET_EVENT."?dateLastModified=".$date."&rows=500&start=".$start;
            
        }
        
        //create stuhub  request to fetch events
        $stubhubToken = $stubhubapi::createToken();
        $response=$this->fetchData($url,$stubhubToken);
        $totalResults=isset($response['numFound']) && !empty($response['numFound']) ? $response['numFound'] : 0;
        
        $log = Log::channel('cron');
        $log->info( "start: ".$start." of ".$totalResults." event");
        
        if(isset($response['events']) && count($response['events']) >= 1){
            foreach ($response['events'] as $key => $value) {
              $log->info( "Processing event: " .$value['id']);
                $event_category=array();
                $cat_arr=array();
                $event_arr = array();

                $performer_1_id = NULL;
                $performer_2_id = NULL;
                $venue_id = 0;
                $event_id = 0;
                
                
                //check country code if its not in europe skip it
                if(isset(Events::COUNTRY_CODES[$value['venue']['country']]) === false) {
                    echo 'Skipped event invalid country ' . $value['venue']['country'] . "\r\n";
                    $log->info($value['id']." event skipped.");
                    $log->info("=============================================");
                    continue;
                }
                
                
                //get currency code
                $currencyCode="";
                $eventWithCurrency = $stubhubapi->getEvent($value['id']);
                if(!empty($eventWithCurrency) && !empty($eventWithCurrency['events'][0])){
                    $currencyCode = $eventWithCurrency['events'][0]['currencyCode'];
                }
               
                //performers data
                $performers = [];
                if(isset($value['performers']) && !empty($value['performers'])){
                    foreach($value['performers'] as $pk =>$performer){
                        $performer_arr=array();

                        //performer name
                        $performer_name = isset($performer['name']) ? $performer['name'] : '';
                        $performer_arr['name'] = str_replace("'", "",trim($performer_name));
                        $performer_arr['name_variation'] = str_replace("'", "",$performer_arr['name']);
                        $performer_arr['source_id']=$resource->id;
                        $performer_arr['external_id']=isset($performer['id']) && !empty($performer['id']) ? $performer['id'] : '';
                        
                        $checkPerformer = Performers::where('name',$performer_arr['name'])->first();
                        if(!$checkPerformer){   
                            $performer = Performers::create($performer_arr);
                            $performerId = $performer->id;
                            $performers[] = $performerId;
                            $log->info("Performer Id:".$performerId." created for event ".$value['id']);
                            
                        }
                        else{
                            if(!empty($checkPerformer->external_id) && !empty($perfomrer_arr['external_id']) && $checkPerformer->external_id != $performer_arr['external_id']){
                                $checkPerformer->external_id = $perfomrer_arr['external_id'] ;
                                $checkPerformer->save();
                                $log->info("Performer External id :".$perfomrer_arr['external_id'] ." saved for event ".$value['id']);
                            }
                            $performerId = $checkPerformer->id;
                             $performers[] = $performerId;
                        }
 
                    }
                }

                //venues data
                if(isset($value['venue']) && !empty($value['venue'])){
                   
                    $venue_arr = array();
                    $venue_name = isset($value['venue']['name']) ? $value['venue']['name'] : NULL;
                    $city = isset($value['venue']['city']) ? $value['venue']['city'] : NULL;
                    $state = isset($value['venue']['state']) ? $value['venue']['state'] : NULL;
                    $postalCode = isset($value['venue']['postalCode']) ? $value['venue']['postalCode'] : NULL;
                    $country = isset($value['venue']['country']) ? $value['venue']['country'] : NULL;

                    $venue_arr['name'] = str_replace("'", "",$venue_name);
                    $venue_arr['city'] = str_replace("'", "",$city);
                    $venue_arr['state'] = str_replace("'", "",$state);
                    $venue_arr['country'] = str_replace("'", "",$country);
                    $venue_arr['source_id']=$resource->id;
                    
                    //check Venue
                    //1->check With Venue Id
                    $checkWithVenueId = Venues::where('tixstock_id',$value['venue']['id'])->first();
                    if(!$checkWithVenueId){
                        //2->check with available data
                        
                            $checkVenue = Venues::where('name','like','%'.$venue_arr['name'].'%');
                            if(!empty($venue_arr['city']) && $venue_arr['city'] != NULL){
                                $checkVenue = $checkVenue->where('city','like','%'.$venue_arr['city'].'%');
                            }
                            // if(!empty($venue_arr['state']) && $venue_arr['state'] != NULL){
                            //     $checkVenue = $checkVenue->where('state','like','%'.$venue_arr['state'].'%');
                            // }
                            if(!empty($venue_arr['country']) && $venue_arr['country'] != NULL){
                                $checkVenue = $checkVenue->where('country','like','%'.$venue_arr['country'].'%');
                            }
                            
                            $checkVenue = $checkVenue->first();
                            if(!$checkVenue){
                                //create venue  
                                $venue_arr['tixstock_id']=$value['venue']['id'];
                                $venue = Venues::create($venue_arr);
                                $venue_id = $venue->id;
                            }else{
                                Venues::where('id',$checkVenue->id)->update($venue_arr);
                                $venue_id = $checkVenue->id;
                            }
                    }else{
                        $venue_id = $checkWithVenueId->id;
                    }
                    
                    //3->check for venue address
                    $venue_address_id = null;
                    $venue_arr['post_code'] = str_replace("'", "",$postalCode);
                    $venue_arr['post_code'] = str_replace(" ","",$postalCode);
                    $venueDetails = $stubhubapi->getVenue($value['venue']['id']);
                     
                    if(isset($venueDetails['venues'][0]) && !empty($venueDetails['venues'][0])){
                       $venue_arr['address_line_1'] = $venueDetails['venues'][0]['address']['address1'];
                     }
                     
                    $checkVenueAddress= new VenueAdditionalInfo();
                    if(isset($venue_arr['address_line_1']) && !empty($venue_arr['address_line_1'])){
                        $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $venue_arr['address_line_1']);
                        $checkVenueAddress = $checkVenueAddress->first();
                        if(!$checkVenueAddress){
                             //create Venue address
                             $venueAddress = [
                                                'venue_id'=>$venue_id,
                                                'address_line_1'=>$venue_arr['address_line_1'],
                                                'post_code'=>$venue_arr['post_code'],
                                                'source_id'=>$venue_arr['source_id']
                                            ];
                             $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                             if(!$checkwithParam){
                                $venueAddress = VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueAddress->id;
                             }else{
                                 $venue_address_id = $checkwithParam->id;
                             }
                             
                        }else{
                            //$venue_address_id = $checkVenueAddress->id;
                                if($checkVenueAddress->venue_id == $venue_id){
                                    $checkVenueAddress->post_code = !empty($venue_arr['post_code']) && $venue_arr['post_code'] != null ? $venue_arr['post_code'] : $checkVenueAddress->post_code; 
                                    $checkVenueAddress->address_line_1 = $venue_arr['address_line_1'];
                                    $checkVenueAddress->save();
                                    $venue_address_id = $checkVenueAddress->id;
                                }else{
                                    // if(!empty($venue_arr['post_code']) && $venue_arr['post_code'] != null){
                                    //     if(str_replace(' ','',$checkVenueAddress->post_code) != $venue_arr['post_code'])
                                    //     {
                                            $venueAddress=[
                                                        'venue_id'=>$venue_id,
                                                        'address_line_1'=>$venue_arr['address_line_1'],
                                                        'post_code'=>$venue_arr['post_code'],
                                                        'source_id'=>$venue_arr['source_id']
                                                        ];
                                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                            if(!$checkwithParam){
                                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                                $venue_address_id = $venueadditionalinfo->id;
                                            }else{
                                                 $venue_address_id = $checkwithParam->id;
                                            }
                                    //     }else{
                                    //         $venue_address_id = $checkVenueAddress->id;
                                    //     }
                                    // }
                            }
                        }
                    }
                }
                
                //check in category collection
                
                if(isset($eventWithCurrency['events'][0]['categoriesCollection']) && !empty($eventWithCurrency)){ 
                    $value['categoriesCollection'] = $eventWithCurrency['events'][0]['categoriesCollection'];
                }
                
                $primary_category = '';
                
                if(isset($value['categoriesCollection']) && !empty($value['categoriesCollection'])){
                    foreach($value['categoriesCollection']['categories'] as $category){
                        //check in tree
                        if(empty($cat_arr['primary_category'])){
                            if (false !== $key = array_search($category['name'], $categoriesArray)) {
                                //add category source mapping 
                                $primary_category = $category['name'];
                                
                            }else{
                                //check in category table
                                $check = SourceCategoryMapping::with('category')->where('source_category_name',$category['name'])->where('source_id',1)->first();
                                if($check){
                                    $primary_category = $check->category->name;
                                    
                                }
                            }
                        }
                            
                    }
                }
                
                // if(empty($primary_category)){
                    //check in groupings
                    if(isset($value['ancestors']['groupings']) && !empty($value['ancestors']['groupings'])){
                        foreach($value['ancestors']['groupings'] as $gk=>$gvalue){
                            $group_name = isset($gvalue['name']) ? $gvalue['name'] : '';
                            if(empty($cat_arr['primary_category'])){
                                if (false !== $key = array_search($group_name, $categoriesArray)) {
                                    //add category source mapping 
                                    $primary_category = $group_name;
                                
                                }else{
                                    //check in category table
                                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$group_name)->where('source_id',1)->first();
                                    if($check){
                                        $primary_category = $check->category->name;
                                      
                                    }
                                }
                            }
                        }
                    }
                // }
                
                if(empty($primary_category)){
                    if(isset($value['categoriesCollection']) && !empty($value['categoriesCollection'])){
                       $categoryCollection = end($value['categoriesCollection']['categories']);
                       $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                    if(isset($value['ancestors']['groupings']) && !empty($value['ancestors']['groupings'])){
                        $categoryCollection = end($value['ancestors']['groupings']);
                        $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                     $primary_category = 'No Category';
                 }
                 
                 
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryObj = new Categories();
                $categoryTree = $categoryObj->getParentCatgoryTree($primary_category);
                
                //get maps of venue
                
                $maps = VenueMaps::with('maps')->where('venue_id',$venue_id)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
                
                //event data
                $newEvent = 0;
                $new_event_id = '';
                $event_name = isset($value['name']) ? $value['name'] : '';
                
                $timezone = isset($event['timezone']) ? $event['timezone'] : date_default_timezone_get();
                echo $timezone;
                exit;
                date_default_timezone_set($timezone);
                $event_date = isset($value['eventDateLocal']) ? date('Y-m-d H:i:s',strtotime($value['eventDateLocal'])) : '';
                $event_date_utc = isset($value['eventDateUTC']) ? $value['eventDateUTC'] : '';  

                if(isset($value['ancestors']['categories'][0]['name']) && $value['ancestors']['categories'][0]['name'] == "Sports"){
                    $value['ancestors']['categories'][0]['name']= "Sport";
                }
                
                $event_arr['search_type'] = $primary_category;
                $event_arr['name'] = str_replace("'", "",$event_name);
                // $event_arr['performer_1_id'] = $performer_1_id;
                // $event_arr['performer_2_id'] = $performer_2_id;
                $event_arr['event_date'] = $event_date;
                $event_arr['event_date_utc'] = $event_date_utc;
                $event_arr['venue_id'] = $venue_id;
                $event_arr['source_id']=$resource->id;
                $event_arr['status'] = $value['status'];
                $event_arr['venue_address_id']=$venue_address_id;
                $eventArray['map_id'] = $map_id;
                //check event external id exist or not 
                $checkExternalId= Events::where('external_id',$value['id'])->orderBy('id','desc')->first();
                if(!$checkExternalId){
                    
                    // if event with external id doesn't exist create event
                    $event_arr['external_id'] = isset($value['id']) ? $value['id'] : '';
                    // $checkEvent=Events::where($event_arr)->first();
                  
                   //check event exist or not with any of the resource
                    $event_date_array = explode(' ',$event_arr['event_date']);
                    $event_date = $event_date_array[0];
                    $checkEvent = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", str_replace("-"," ",$event_arr['name']))
                                        ->where('venue_id',$venue_id)
                                        ->whereBetween('event_date',[$event_date." 00:00:00",$event_date." 23:59:59"])
                                        ->first();
                    
                    if(!$checkEvent){
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $event_arr['status']=$value['status'];
                        $event_arr['currency_code']= isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        
                        $event=Events::create($event_arr);
                        
                        $event_id=$event->id;
                        $new_event_id =$event->id;
                        $newEvent = 1;
                        
                        $log->info("created  event with external id : ".$value['id']);
                        $createdEvents[] = $event->external_id;
                    }else{
                         //check source of event(if matched then update event or create new event with resource)
                         $eventUpdateFlag = 0;
                        if($checkEvent->source_id = $resource->id){
                            
                            //if event is from same resource update the event
                            if($checkEvent->mapped_status == 1){
                                $excludeField = ['external_id','search_type','map_id','status','source_id'];
                                $prevArray = $checkEvent->toArray();
                                $prev_arr=[];
                                foreach($event_arr as $key=>$val){
                                    if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                        $eventUpdateFlag=1;
                                        $prev_arr['prev_'.$key] = $val;
                                        $prev_arr['update_requires'] =1;
                                        //$log->info("Updated event's ".$key." to: ".$value['id']);
                                    }
                                }
                                if($eventUpdateFlag==1){
                                    //update flag of previous event
                                    //$event_arr['update_requires']=1;
                                     Events::where('id',$checkEvent->id)->update($prev_arr);
                                    //create new event 
                                    $event_arr['update_requires'] = 0;
                                    $event_arr['current_version'] = 0;
                                    $event_arr['mapped_status'] = 0;
                                    $event_arr['parent_event_id'] = $checkEvent->id;
                                    $event = Events::create($event_arr);
                                    $event_id = $event->id;
                                    $new_event_id =$event->id;
                                    $newEvent = 1;
                                    
                                    
                                }
                            }
                            else{
                                Events::where('id',$checkEvent->id)->update($event_arr);
                                $event_id=$checkEvent->id;
                                //update performers
                                EventPerformers::where('event_id',$event_id)->delete();
                                if(!empty($performers)){
                                    foreach($performers as $performer){
                                        EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                    }
                                }
                            }
                            // $event_arr['update_requires']=1;
                            // $event= Events::where('id',$checkEvent->id)->update($event_arr);
                            
                        }else{
                            
                            // $checkEvent->current_version=0;
                            // $checkEvent->save();
                            
                            //create versioning
                            $event_arr['update_requires'] = 0;
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['parent_event_id'] = $checkEvent->id;
                            $event= Events::create($event_arr);
                            $event_id = $event->id;
                            $new_event_id =$event->id;
                            $newEvent = 1;
                        }
                    }  
                }else{
                    //check if same event exist or not 
                    $checkEventUpdate=Events::where($event_arr)->first();
                    if(!$checkEventUpdate){
                        //event is updated
                        $event_id = $checkExternalId->id;
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $eventUpdateFlag=0;
                        
                        if($checkExternalId->mapped_status == 1){
                            $excludeField = ['external_id','search_type','map_id','status','source_id'];
                            $prevArray = $checkExternalId->toArray();
                            $prev_arr = [];
                            foreach($event_arr as $key=>$val){
                                if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                    $eventUpdateFlag=1;
                                    $prev_arr['prev_'.$key] = $val;
                                    
                                    $log->info("Updated event's ".$key." to: ".$value['id']);
                                }
                            }
                            if($eventUpdateFlag==1){
                                $prev_arr['update_requires'] =1;
                            }
                        }
                        
                        if($eventUpdateFlag==1){
                            //$event_arr['update_requires']=1;
                            Events::where('id',$checkExternalId->id)->update($prev_arr);
                            
                           // Events::where('id',$checkExternalId->id)->update(['current_version'=>0]);
                            //create new event
                            $event_arr['external_id'] = $value['id'] ;
                            $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                            $event_arr['status']=$value['status'];
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                            $event_arr['update_requires']=0;
                            $event_arr['venue_address_id']=$venue_address_id;
                            $event_arr['parent_event_id']=$checkExternalId->id;
                            $event = Events::create($event_arr);
                            $event_id = $event->id;
                            $new_event_id =$event->id;
                            $newEvent = 1;
                            $log->info('New version of Event created for :'.$value['id']);
                            $updatedEvents[] = $checkExternalId->external_id;
                            $log->info("Event ".$value['id']." updated ");
                        }else{
                            $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                            Events::where('id',$checkExternalId->id)->update($event_arr);
                            $event_id = $checkExternalId->id;
                            EventPerformers::where('event_id',$event_id)->delete();
                            if(!empty($performers)){
                                foreach($performers as $performer){
                                    EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                }
                            }
                        }   
                    }else{
                        $checkEventUpdate->currency_code = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        $checkEventUpdate->save();
                        $event_id = $checkEventUpdate->id;
                        EventPerformers::where('event_id',$event_id)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                            }
                        }
                    }
                }
                
                
                //event category data

                if(!empty($new_event_id)){
                    $event_id = $new_event_id;
                    $cat_arr['event_id'] = $new_event_id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                        }
                    }
                }
                
                $checkEventCategory=EventCategory::where('event_id',$event_id)->first();
                if(!$checkEventCategory){
                    $cat_arr['primary_category'] = $primary_category;
                    $cat_arr['event_id'] = $event_id;
                    $cat_arr['venue_id']=$venue_id;
                    $cat_arr['venue_address_id']=$venue_address_id;
                    $cat_arr['source_id']=$resource->id;
                    EventCategory::create($cat_arr);
                    
                    $log->info("Event category created for event :".$value['id']);
                }
                
                
                if($newEvent == 1){
                    $this->fetchStuhubhubEventsZones($resource,$event->external_id);
                    
                }
                 $log->info("End event processing of event :".$value['id']);
                  $log->info("=============================================");
            }
        }else{
            $logData=['type'=>"cron",'domain'=>"Stubhub:last ".$days." days data","message"=>"No events found."];
            Logs::create($logData);
            Log::info("No events found.");
            exit;
        }

        $countResults+=count($response['events']);
        $countResults=$start+$countResults;

        if($countResults != $totalResults){
            $this->processData($resource,$slug,$countResults);
        }
        
        $logMessage='';
        if(!empty($createdEvents)){
            $logMessage.='('.implode(',',$createdEvents).") events Created.";
        }
        if(!empty($updatedEvents)){
            $logMessage.='('.implode(',',$updatedEvents).") events Updated";
        }
        Log::info("last 10 day's data updated");
        if(!empty($logMessage)){
            $logData=['type'=>'cron','domain'=>"Stubhub:last ".$days." days data","message"=>$logMessage];
            Logs::create($logData);
        }

    }
    
    public function fetchStuhubhubEventsZones($resource,$external_id){

        $event=Events::where('is_run',0)->where('external_id',$external_id)->orderBy('id','desc')->first();
       
        $log=Log::channel('cron');
        $log->info("Fetching ".$external_id." Event's Zone thorugh stubhub Api");
        $eventsUpdated = [];
        if($event){
            // foreach($events as $event){
                $log->info('Processing event '.$event->external_id);
                $event_id=$event->external_id;
                $id=$event->id;
                $venue_id=$event->venue_id;
                
                $url = "https://api.stubcloudprod.com/partners/catalog/events/v3/".$event_id."/sectionZones";
                $stubhubapi = new StubhubApi();
                $stubhubToken = $stubhubapi::createToken();
                $response=$this->fetchData($url,$stubhubToken);
                
                if(!empty($response)){
                    if(count($response['zones']) >= 1){
                        $venue_category = 0;
                        $send_category_arr = array();
                        $send_section_arr = array();
                        $getCategory = EventCategory::where('event_id',$id)->where('venue_id',$venue_id)->where('source_id',$resource->id)->first();
    
                        $flag=0;
                        if(!empty($getCategory->secondary_category)){
                            $flag=1;
                            $venue_category = $getCategory->secondary_category;
                        }
                        if($flag==0){
                            if(!empty($getCategory->sub_category)){
                                $flag=1;
                                $venue_category = $getCategory->sub_category;
                            }
                        }
                        if($flag==0){
                            if(!empty($getCategory->primary_category)){
                                $flag=1;
                                $venue_category = $getCategory->primary_category;
                            }
                        }
                        $categoryArray= [];
                        $sectionArray=[];
                        
                        foreach ($response['zones'] as $key => $value) {
                            $cat_arr = array();
                            $category_id = 0;
                            $category_name = $value['name'];
    
                            $cat_arr['name'] = str_replace("'", "",trim($category_name));
                            $cat_arr['venue_id'] = $venue_id;
                           // $cat_arr['venue_category'] = $venue_category;
                            
    
                            $checkCategory=VenueCategory::where($cat_arr)->first();
                            
                            if(!$checkCategory){
                                $cat_arr['source_id'] = $resource->id;
                                $cat_arr['external_id']=$value['id'];
                                $cat_arr['venue_category'] = $venue_category;
                                $category = VenueCategory::create($cat_arr);
                                $category_id = $category->id;
                                $log->info('Created Category'.$category_id.' for event '.$event->external_id);
                                
                            }else{
                                $checkCategory->external_id=$value['id'];
                                $checkCategory->save();
                                $category_id = $checkCategory->id;
                            }
                            $categoryArray[]=$category_id;
                            //add sections of venue 
                            foreach ($value['sections'] as $sk => $section_value) {
                                $sec_arr = array();
                                $section_name = trim($section_value['name']);
                                $sec_arr['name'] = str_replace("'", "",$section_name);
                                $sec_arr['category_id'] = $category_id;
                                
                                
                                //check same section is available or not 
                                $checkVenueSection=VenueSection::where($sec_arr)->first();
                                if(!$checkVenueSection){
                                    $sec_arr['source_id'] = $resource->id;
                                    $sec_arr['external_id']=$section_value['id'];
                                    $venueSection = VenueSection::create($sec_arr);
                                    $section_id = $venueSection->id;
                                     $log->info('Created section'.$section_id.' for category '.$category_id);
                                }else{
                                    $checkVenueSection->external_id = $section_value['id'];
                                    $checkVenueSection->save();
                                    $section_id = $checkVenueSection->id;
                                }
                                
                                $sectionArray[] = $section_id;
                            }
                        }
                        
                        
                        $categoryArray = implode(',',$categoryArray);
                        $sectionArray = implode(',',$sectionArray);
                        
                        if(!empty($categoryArray) && !empty($sectionArray)){
                            $getCategory->event_categories = $categoryArray;
                            $getCategory->event_section = $sectionArray;
                            $getCategory->save();
                        }
                        
                    }
                    $update_data = array();	
                    $update_data['is_run'] = strtotime('now');
                    Events::where('id',$id)->update($update_data);
                    $eventsUpdated[] = $event->external_id;
                    
                    $log->info($event->id." Event's zone updated successfully.");
                    $log->info("==================================");
                }
            
            if(!empty($eventsUpdated)){
                $eventsUpdated = implode(',',$eventsUpdated);
                $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"(".$eventsUpdated.") Events zone updated successfully."];
                //Logs::create($logData);
            }
        }else{
            $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"No Events available to update zones"];
            //Logs::create($logData);
            Log::info("No Events available to update zones");
        }
    }
    
    public function processSingleEvent($resource,$event){
      
        if($resource->name == "Stubhub"){
            $sourceCategoryMapping = new CategoryController();
            $categoriesArray = $sourceCategoryMapping->getLeafCategories();
        
                $event_category=array();
                $cat_arr=array();
                $event_arr = array();
        
                $performer_1_id = NULL;
                $performer_2_id = NULL;
                $venue_id = 0;
                $event_id = 0;
                        
                $stubhubapi = new StubhubApi();       
                //check country code if its not in europe skip it
                $countryCodes = Events::COUNTRY_CODES;
               
              
                if(isset($countryCodes[$event['venue']['country']]) === false) {
                    return response()->json(['success'=>false,'msg'=>'Country Code is not valid.']);
                }
                        
                        
                //get currency code
                $currencyCode="";
                $eventWithCurrency = $stubhubapi->getEvent($event['id']);
                if(!empty($eventWithCurrency) && !empty($eventWithCurrency['events'][0])){
                    $currencyCode = $eventWithCurrency['events'][0]['currencyCode'];
                }
               
                //performers data
                $performers = [];
                if(isset($event['performers']) && !empty($event['performers'])){
                    foreach($event['performers'] as $pk =>$performer){
                        $performer_arr=array();
            
                        //performer name
                        $performer_name = isset($performer['name']) ? $performer['name'] : '';
                        $performer_arr['name'] = str_replace("'", "",$performer_name);
                        $performer_arr['name_variation'] = str_replace("'", "",$performer_arr['name']);
                        $performer_arr['source_id']=$resource->id;
                        $performer_arr['external_id']=isset($performer['id']) && !empty($performer['id']) ? $performer['id'] : '';
                        
                        $checkPerformer = Performers::where('name',$performer_arr['name'])->where('source_id',$resource->id)->first();
                        if(!$checkPerformer){   
                            $performer = Performers::create($performer_arr);
                            $performerId = $performer->id;  
                            $performers[] = $performerId;
                        }
                        else{
                            if(!empty($checkPerformer->external_id) && !empty($perfomrer_arr['external_id']) && $checkPerformer->external_id != $performer_arr['external_id']){
                                $checkPerformer->external_id = $perfomrer_arr['external_id'] ;
                                $checkPerformer->save();
                            }
                            $performerId = $checkPerformer->id;
                            $performers[] = $performerId;
                        }  
                    }
                }
            
                //venues data
                if(isset($event['venue']) && !empty($event['venue'])){
                   
                    $venue_arr = array();
                    $venue_name = isset($event['venue']['name']) ? $event['venue']['name'] : NULL;
                    $city = isset($event['venue']['city']) ? $event['venue']['city'] : NULL;
                    $state = isset($event['venue']['state']) ? $event['venue']['state'] : NULL;
                    $postalCode = isset($event['venue']['postalCode']) ? $event['venue']['postalCode'] : NULL;
                    $country = isset($event['venue']['country']) ? $event['venue']['country'] : NULL;

                    $venue_arr['name'] = str_replace("'", "",$venue_name);
                    $venue_arr['city'] = str_replace("'", "",$city);
                    $venue_arr['state'] = str_replace("'", "",$state);
                    $venue_arr['country'] = str_replace("'", "",$country);
                    $venue_arr['source_id']=$resource->id;
                    
                    
                   
               
                
                    //check Venue
                    //1->check With Venue Id
                    $checkWithVenueId = Venues::where('tixstock_id',$event['venue']['id'])->first();
                    if(!$checkWithVenueId){
                        //2->check with available data
                        
                            $checkVenue = Venues::where('name','like','%'.$venue_arr['name'].'%');
                            if(!empty($venue_arr['city']) && $venue_arr['city'] != NULL){
                                $checkVenue = $checkVenue->where('city','like','%'.$venue_arr['city'].'%');
                            }
                            // if(!empty($venue_arr['state']) && $venue_arr['state'] != NULL){
                            //     $checkVenue = $checkVenue->where('state','like','%'.$venue_arr['state'].'%');
                            // }
                            if(!empty($venue_arr['country']) && $venue_arr['country'] != NULL){
                                $checkVenue = $checkVenue->where('country','like','%'.$venue_arr['country'].'%');
                            }
                            
                            $checkVenue = $checkVenue->first();
                            if(!$checkVenue){
                                //create venue  
                                $venue_arr['tixstock_id']=$event['venue']['id'];
                                $venue = Venues::create($venue_arr);
                                $venue_id = $venue->id;
                            }else{
                                Venues::where('id',$checkVenue->id)->update($venue_arr);
                                $venue_id = $checkVenue->id;
                            }
                    }else{
                        $venue_id = $checkWithVenueId->id;
                    }
                    
                    //3->check for venue address
                    $venue_address_id = null;
                    $venue_arr['post_code'] = str_replace("'", "",$postalCode);
                    $venue_arr['post_code'] = str_replace(" ","",$postalCode);
                    $venueDetails = $stubhubapi->getVenue($event['venue']['id']);
                     
                    if(isset($venueDetails['venues'][0]) && !empty($venueDetails['venues'][0])){
                       $venue_arr['address_line_1'] = $venueDetails['venues'][0]['address']['address1'];
                     }
                     
                    $checkVenueAddress= new VenueAdditionalInfo();
                    if(isset($venue_arr['address_line_1']) && !empty($venue_arr['address_line_1'])){
                        $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $venue_arr['address_line_1']);
                        $checkVenueAddress = $checkVenueAddress->first();
                        if(!$checkVenueAddress){
                             //create Venue address
                             $venueAddress = [
                                                'venue_id'=>$venue_id,
                                                'address_line_1'=>$venue_arr['address_line_1'],
                                                'post_code'=>$venue_arr['post_code'],
                                                'source_id'=>$venue_arr['source_id']
                                            ];
                             $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                             if(!$checkwithParam){
                                $venueAddress = VenueAdditionalInfo::create($venueAddress);
                                $venue_address_id = $venueAddress->id;
                             }else{
                                 $venue_address_id = $checkwithParam->id;
                             }
                             
                        }else{
                            //$venue_address_id = $checkVenueAddress->id;
                                if($checkVenueAddress->venue_id == $venue_id){
                                    $checkVenueAddress->post_code = !empty($venue_arr['post_code']) && $venue_arr['post_code'] != null ? $venue_arr['post_code'] : $checkVenueAddress->post_code; 
                                    $checkVenueAddress->address_line_1 = $venue_arr['address_line_1'];
                                    $checkVenueAddress->save();
                                    $venue_address_id = $checkVenueAddress->id;
                                }else{
                                    // if(!empty($venue_arr['post_code']) && $venue_arr['post_code'] != null){
                                    //     if(str_replace(' ','',$checkVenueAddress->post_code) != $venue_arr['post_code'])
                                    //     {
                                            $venueAddress=[
                                                        'venue_id'=>$venue_id,
                                                        'address_line_1'=>$venue_arr['address_line_1'],
                                                        'post_code'=>$venue_arr['post_code'],
                                                        'source_id'=>$venue_arr['source_id']
                                                        ];
                                            $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                            if(!$checkwithParam){
                                                $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                                $venue_address_id = $venueadditionalinfo->id;
                                            }else{
                                                 $venue_address_id = $checkwithParam->id;
                                            }
                                    //     }else{
                                    //         $venue_address_id = $checkVenueAddress->id;
                                    //     }
                                    // }
                            }
                    }
                    }
                }
                
                //check in category collection
                
                if(isset($eventWithCurrency['events'][0]['categoriesCollection']) && !empty($eventWithCurrency)){ 
                    $event['categoriesCollection'] = $eventWithCurrency['events'][0]['categoriesCollection'];
                }
                
                $primary_category = '';
                if(isset($event['categoriesCollection']) && !empty($event['categoriesCollection'])){
                    foreach($event['categoriesCollection']['categories'] as $category){
                        //check in tree
                        if(empty($cat_arr['primary_category'])){
                            if (false !== $key = array_search($category['name'], $categoriesArray)) {
                                //add category source mapping 
                                $primary_category = $category['name'];
                                break;
                            }else{
                                //check in category table
                                $check = SourceCategoryMapping::with('category')->where('source_category_name',$category['name'])->where('source_id',1)->first();
                                if($check){
                                    $primary_category = $check->category->name;
                                    break;
                                }
                            }
                        }
                            
                    }
                }
                
                if(empty($primary_category)){
                    //check in groupings
                    if(isset($event['ancestors']['groupings']) && !empty($event['ancestors']['groupings'])){
                        foreach($event['ancestors']['groupings'] as $gk=>$gvalue){
                            $group_name = isset($gvalue['name']) ? $gvalue['name'] : '';
                            if(empty($cat_arr['primary_category'])){
                                if (false !== $key = array_search($group_name, $categoriesArray)) {
                                    //add category source mapping 
                                    $primary_category = $group_name;
                                    break;
                                }else{
                                    //check in category table
                                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$group_name)->where('source_id',1)->first();
                                    if($check){
                                        $primary_category = $check->category->name;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
                
                if(empty($primary_category)){
                    if(isset($event['categoriesCollection']) && !empty($event['categoriesCollection'])){
                       $categoryCollection = end($event['categoriesCollection']['categories']);
                       $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                    if(isset($event['ancestors']['groupings']) && !empty($event['ancestors']['groupings'])){
                        $categoryCollection = end($event['ancestors']['groupings']);
                        $primary_category = $categoryCollection['name'];
                    }
                }
                
                if(empty($primary_category)){
                     $primary_category = 'No Category';
                 }
                 
                 
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryObj = new Categories();
                $categoryTree = $categoryObj->getParentCatgoryTree($primary_category);
                
                //get maps of venue
                
                $maps = VenueMaps::with('maps')->where('venue_id',$venue_id)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
               
                $newEvent = 0;
                $new_event_id = '';
                $event_name = isset($event['name']) ? $event['name'] : '';
                $timezoneArray  = ['AST'];
                $timezone = isset($event['timezone']) ? $event['timezone'] : date_default_timezone_get();
                // if(!in_array($timezone,$timezoneArray)){
                 
                // try{
                //   date_default_timezone_set($timezone);
                // }catch (\ErrorException $e) {
                //     date_default_timezone_set('Europe/London');
                // }
                
                $event_date = "";
                if(isset($event['eventDateLocal']) && !empty($event['eventDateLocal'])){
                    $carbonDate = new Carbon($event['eventDateLocal']);
                    $event_date = $carbonDate->format('Y-m-d H:i:s');
                }
                $event_date_utc = isset($event['eventDateUTC']) ? $event['eventDateUTC'] : '';
                
                // $event_date = isset($event['eventDateLocal']) ? date('Y-m-d H:i:s',strtotime($event['eventDateLocal'])) : '';
                // $event_date_utc = isset($event['eventDateUTC']) ? $event['eventDateUTC'] : '';
                
                if(isset($event['ancestors']['categories'][0]['name']) && $event['ancestors']['categories'][0]['name'] == "Sports"){
                    $value['ancestors']['categories'][0]['name']= "Sport";
                }
                
                $event_arr['search_type'] = $primary_category;
                $event_arr['name'] = str_replace("'", "",$event_name);
                // $event_arr['performer_1_id'] = $performer_1_id;
                // $event_arr['performer_2_id'] = $performer_2_id;
                $event_arr['event_date'] = $event_date;
                $event_arr['event_date_utc'] = $event_date_utc;
                $event_arr['venue_id'] = $venue_id;
                $event_arr['source_id']=$resource->id;
                $event_arr['status'] = $event['status'];
                $event_arr['venue_address_id']=$venue_address_id;
                $event_arr['map_id'] =$map_id;
                
                //check event external id exist or not 
                $checkExternalId= Events::where('external_id',$event['id'])->orderBy('id','desc')->first();
                if(!$checkExternalId){
                    
                    // if event with external id doesn't exist create event
                    $event_arr['external_id'] = isset($event['id']) ? $event['id'] : '';
                    // $checkEvent=Events::where($event_arr)->first();
                  
                   //check event exist or not with any of the resource
                    $event_date_array = explode(' ',$event_arr['event_date']);
                    $event_date = $event_date_array[0];
                    $checkEvent = Events::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $event_arr['name'])
                                        ->where('venue_id',$venue_id)
                                        ->whereBetween('event_date',[$event_date." 00:00:00",$event_date." 23:59:59"])
                                        ->first();
                   
                    if(!$checkEvent){
                        $event_arr['url'] = isset($event['webURI']) ? $event['webURI'] : '';
                        $event_arr['status']=$event['status'];
                        $event_arr['currency_code']= isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        
                        $events=Events::create($event_arr);
                        
                        $event_id=$events->id;
                        $new_event_id =$events->id;
                        $newEvent = 1;
                        
                        
                        $createdEvents[] = $events->external_id;
                    }else{
                        
                         //check source of event(if matched then update event or create new event with resource)
                         $eventUpdateFlag = 0;
                        if($checkEvent->source_id == $resource->id){
                            
                            //if event is from same resource update the event
                            if($checkEvent->mapped_status == 1){
                                $excludeField = ['external_id','search_type','map_id','status','source_id'];
                                $prevArray = $checkEvent->toArray();
                                $prev_arr=[];
                                foreach($event_arr as $key=>$val){
                                    if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                        $eventUpdateFlag=1;
                                        $prev_arr['prev_'.$key] = $val;
                                        $prev_arr['update_requires'] =1;
                                        //$log->info("Updated event's ".$key." to: ".$value['id']);
                                    }
                                }
                                if($eventUpdateFlag==1){
                                    //update flag of previous event
                                    //$event_arr['update_requires']=1;
                                     Events::where('id',$checkEvent->id)->update($prev_arr);
                                    //create new event 
                                    if($checkEvent->source_id == 0){
                                        //for manual events only.
                                        if($checkEvent->name != $event_arr['name']){
                                            $event_arr['name'] = $checkEvent->name;
                                        }
                                        
                                        if($checkEvent->search_type != $event_arr['search_type']){
                                            $event_arr['search_type'] = $checkEvent->search_type;
                                        }
                                    }
                                    
                                    $event_arr['update_requires'] = 0;
                                    $event_arr['current_version'] = 0;
                                    $event_arr['mapped_status'] = 0;
                                    $event_arr['parent_event_id'] = $checkEvent->id;
                                    $events = Events::create($event_arr);
                                    $event_id = $events->id;
                                    $new_event_id =$events->id;
                                    $newEvent = 1;
                                }
                            }
                            else{
                                Events::where('id',$checkEvent->id)->update($event_arr);
                                $event_id=$checkEvent->id;
                                //update performers
                                EventPerformers::where('event_id',$event_id)->delete();
                                if(!empty($performers)){
                                    foreach($performers as $performer){
                                        EventPerformers::create(['event_id'=>$event->id,'performer_id'=>$performer]);
                                    }
                                }
                            }
                            // $event_arr['update_requires']=1;
                            // $event= Events::where('id',$checkEvent->id)->update($event_arr);
                            
                        }else{
                            
                            // $checkEvent->current_version=0;
                            // $checkEvent->save();
                            
                            //create versioning
                            if($checkEvent->source_id == 0){
                                //for manual events only.
                                if($checkEvent->name != $event_arr['name']){
                                    $event_arr['name'] = $checkEvent->name;
                                }
                                
                                if($checkEvent->search_type != $event_arr['search_type']){
                                    $event_arr['search_type'] = $checkEvent->search_type;
                                }
                                $primary_category = $checkEvent->search_type;
                            }
                            
                            $event_arr['update_requires'] = 0;
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['parent_event_id'] = $checkEvent->id;
                            $events= Events::create($event_arr);
                            $event_id = $events->id;
                            $new_event_id =$events->id;
                            $newEvent = 1;
                            Events::where('id',$checkEvent->id)->update(['update_requires'=>1]);
                        }
                    }  
                }else{
                    //check if same event exist or not 
                    $checkEventUpdate=Events::where($event_arr)->first();
                    if(!$checkEventUpdate){
                        //event is updated
                        $event_id = $checkExternalId->id;
                        $event_arr['url'] = isset($event['webURI']) ? $event['webURI'] : '';
                        $eventUpdateFlag=0;
                        
                        if($checkExternalId->mapped_status == 1){
                            $excludeField = ['external_id','search_type','map_id','status','source_id'];
                            $prevArray = $checkExternalId->toArray();
                            $prev_arr = [];
                            foreach($event_arr as $key=>$val){
                                if(!in_array($key,$excludeField) && !empty($val) && $prevArray[$key] != $val){
                                    $eventUpdateFlag=1;
                                    $prev_arr['prev_'.$key] = $val;
                                    
                                    // $log->info("Updated event's ".$key." to: ".$value['id']);
                                }
                            }
                            if($eventUpdateFlag==1){
                                $prev_arr['update_requires'] =1;
                            }
                        }
                        
                        if($eventUpdateFlag==1){
                            //$event_arr['update_requires']=1;
                            Events::where('id',$checkExternalId->id)->update($prev_arr);
                            if($checkExternalId->source_id == 0){
                                //for manual events only.
                                if($checkExternalId->name != $event_arr['name']){
                                    $event_arr['name'] = $checkExternalId->name;
                                }
                                
                                if($checkExternalId->search_type != $event_arr['search_type']){
                                    $event_arr['search_type'] = $checkExternalId->search_type;
                                }
                            }
                           // Events::where('id',$checkExternalId->id)->update(['current_version'=>0]);
                            //create new event
                            $event_arr['external_id'] = $event['id'] ;
                            $event_arr['url'] = isset($event['webURI']) ? $event['webURI'] : '';
                            $event_arr['status']=$event['status'];
                            $event_arr['current_version'] = 0;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                            $event_arr['update_requires']=0;
                            $event_arr['venue_address_id']=$venue_address_id;
                            $event_arr['parent_event_id']=$checkExternalId->id;
                            $events = Events::create($event_arr);
                            $event_id = $events->id;
                            $new_event_id =$events->id;
                            $newEvent = 1;
                            
                            $updatedEvents[] = $checkExternalId->external_id;
                            
                        }else{
                             $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                             Events::where('id',$checkExternalId->id)->update($event_arr);
                             $event_id = $checkExternalId->id;
                             EventPerformers::where('event_id',$event_id)->delete();
                             if(!empty($performers)){
                                foreach($performers as $performer){
                                    EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                                }
                             }
                        }
                        
                    }else{
                        $checkEventUpdate->currency_code = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        $checkEventUpdate->save();
                        $event_id = $checkEventUpdate->id;
                        EventPerformers::where('event_id',$event_id)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                            }
                        }
                    }
                   
                }
                 
                //event category data
              
                
                if(!empty($new_event_id)){
                    $event_id = $new_event_id;
                    $cat_arr['event_id'] = $new_event_id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$performer]);
                        }
                    }else{
                        $tbcperformer = Performers::where('name','TBC Performer')->first();
                        EventPerformers::create(['event_id'=>$event_id,'performer_id'=>$tbcperformer->id]);
                    }
                }
                
                    
                
                $checkEventCategory=EventCategory::where('event_id',$event_id)->first();
                if(!$checkEventCategory){
                    $cat_arr['primary_category']=$primary_category;
                    $cat_arr['event_id'] = $event_id;
                    $cat_arr['venue_id']=$venue_id;
                    $cat_arr['venue_address_id']=$venue_address_id;
                    $cat_arr['source_id']=$resource->id;
                   
                    EventCategory::create($cat_arr);
                    // $log->info("Event category created for event :".$value['id']);
                }
                
                
                
                if($newEvent == 1){
                    $this->fetchStuhubhubEventsZones($resource,$events->external_id);
                }
                
                $response= [];
                $response['msg']="Event Processed Successfully.";
                if($newEvent == 1){
                    $response['msg']="Event Added Successfully.";
                }
                $response['success']=true;
                return response()->json($response);
        }
        else if($resource->name=="Tixstock"){
            
            $sources=Sources::where('name','Tixstock')->first();
            //check venue details insert if needed
                $venueArray = [];
                if(isset($event['venue']['name']) && !empty($event['venue']['name'])){
                        //$checkVenue = Venues::whereRaw("MATCH(name) AGAINST(? IN BOOLEAN MODE)", $venue['name']);
                        $checkVenue = Venues::where("name","like","%".$event['venue']['name']."%");
                }
                
                if(isset($event['venue']['city']) && !empty($event['venue']['city'])){
                       $checkVenue = $checkVenue->where("city","like","%".$event['venue']['city']."%"); 
                }
                
                if(isset($event['venue']['state']) && !empty($event['venue']['state'])){
                       $checkVenue = $checkVenue->where("state","like","%".$event['venue']['state']."%"); 
                }
                
                if(isset($event['venue']['country_code']) && !empty($event['venue']['country_code'])){
                       $checkVenue = $checkVenue->where("country", $event['venue']['country_code']); 
                }
                $venueArray = [
                        'name'=>isset($event['venue']['name']) && !empty($event['venue']['name']) ? $event['venue']['name'] : NULL,
                        'city'=>isset($event['venue']['city']) && !empty($event['venue']['city']) ? $event['venue']['city'] : "",
                        'state'=>isset($event['venue']['state']) && !empty($event['venue']['state']) ? $event['venue']['state'] : NULL,
                        'country'=>isset($event['venue']['country_code']) && !empty($event['venue']['country_code']) ? $event['venue']['country_code'] : NULL,
                    ];
                    
                $checkVenue = $checkVenue->first();
                $venueId = "";
                //check venue id exist or not
                if(!empty($event['venue']['id'])){
                    $checkVenueId = Venues::where('tixstock_id',$event['venue']['id'])->where('source_id',$sources->id)->first();
                    if($checkVenueId){
                        $venueId = $checkVenueId->id;
                    }
                }
                if(empty($venueId)){
                    if(!$checkVenue){
                        $venueArray['tixstock_id'] = $event['venue']['id'];
                        $venueArray['source_id'] = $sources->id;
                        $venue=Venues::create($venueArray);
                        $venueId = $venue->id;
                    }else{
                        $venueId = $checkVenue->id;
                    }
                }
                
                $venue_address_id = null;
                $venueAddress = [
                                    'venue_id'=>$venueId,
                                    'address_line_1'=>isset($event['venue']['address_line_1']) && !empty($event['venue']['address_line_1']) ? $event['venue']['address_line_1'] : NULL,
                                    'address_line_2'=>isset($event['venue']['address_line_2']) && !empty($event['venue']['address_line_2']) ? $event['venue']['address_line_2'] : NULL,
                                    'post_code'=>isset($event['venue']['postcode']) && !empty($event['venue']['state']) ? str_replace(' ','',$event['venue']['postcode']) : NULL,
                                    'source_id'=>$sources->id
                                ];
                                  
                $checkVenueAddress= new VenueAdditionalInfo();
                if(isset($event['venue']['address_line_1']) && !empty($event['venue']['address_line_1'])){
                     $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_1) AGAINST(? IN BOOLEAN MODE)", $event['venue']['address_line_1']);
                }
                
                if(isset($event['venue']['address_line_2']) && !empty($event['venue']['address_line_2']) && $event['venue']['address_line_2'] != NULL){
                     $checkVenueAddress = $checkVenueAddress->whereRaw("MATCH(address_line_2) AGAINST(? IN BOOLEAN MODE)", $event['venue']['address_line_2']);
                }
                
                $checkVenueAddress= $checkVenueAddress->first();
                
                if(!$checkVenueAddress){
                        $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                        if(!$checkwithParam){
                        $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                        $venue_address_id = $venueadditionalinfo->id;
                        }else{ 
                            $venue_address_id = $checkwithParam->id;
                        }
                    
                }else{
                    if($checkVenueAddress->venue_id == $venueId){
                        $checkVenueAddress->address_line_1 = isset($event['venue']['address_line_1']) && !empty($event['venue']['address_line_1']) ? $event['venue']['address_line_1'] : $checkVenueAddress->address_line_1;
                        $checkVenueAddress->address_line_2 = isset($event['venue']['address_line_2']) && !empty($event['venue']['address_line_2']) ? $event['venue']['address_line_2'] : $checkVenueAddress->address_line_2;
                        $checkVenueAddress->post_code = isset($event['venue']['postcode']) && !empty($event['venue']['postcode']) ? str_replace(' ','',$event['venue']['postcode']) : $checkVenueAddress->post_code;
                        $checkVenueAddress->save();
                        $venue_address_id = $checkVenueAddress->id;
                    }else{
                        // if(!empty($venueAddress['post_code']) && $venueAddress['post_code'] != NULL){
                        //     if($checkVenueAddress->post_code != $venueAddress['post_code'])
                        //     {
                                $checkwithParam = VenueAdditionalInfo::where($venueAddress)->first();
                                if(!$checkwithParam){
                                    $venueadditionalinfo= VenueAdditionalInfo::create($venueAddress);
                                    $venue_address_id = $venueadditionalinfo->id;
                                }
                            // }else{
                            //     $venue_address_id = $checkVenueAddress->id;
                            // }
                        // }
                    }
                }
                
                //check venue section and category
                $categoryArray=[];
                $sectionArray=[];
                if(!empty($event['venue_details'])){
                    foreach($event['venue_details'] as $venueDetail){
                        //check venue category
                        $checkCategory = VenueCategory::where('name',$venueDetail['name'])
                                                        ->where('venue_id',$venueId)
                                                        // ->where('source_id',$sources->id)
                                                        ->first();
                            if(!$checkCategory){
                                $categoryArray=[
                                                'name'=>$venueDetail['name'],
                                                'venue_id'=>$venueId,
                                                'venue_category'=>$venueDetail['name'],
                                                'external_id'=>$venueDetail['id'],
                                                'source_id'=>$sources->id
                                            ];
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                                $categoryArray[]=$category->id;
                            }else{
                                    $categoryId = $checkCategory->id;
                                    $categoryArray[]=$checkCategory->id;
                            }
                            //check section
                            foreach($venueDetail['sections'] as $section){
                                    $checkSection=VenueSection::where('name',$section['name'])
                                                                ->where('category_id',$categoryId)
                                                                // ->where('source_id',$sources->id)
                                                                ->first();
                                    if(!$checkSection){
                                        $sectionsArray = [
                                                            'name'=>$section['name'],
                                                            'category_id'=>$categoryId,
                                                            'source_id'=>$sources->id,
                                                            'external_id' => $section['id'] ?? '',
                                                        ];
                                        $section = VenueSection::create($sectionsArray);
                                        $sectionArray[] = $section->id;
                                    }else{
                                        $sectionArray[]=$checkSection->id;
                                        
                                    }
                            }
                    }
                }
                
                //check performers 
                $count=1;
                $performer_1_id=null;
                $performer_2_id=null;
                foreach($event['performers'] as $performer){
                    $checkPerformer=Performers::where('name',$performer['name']);
                    if(!empty($performer['id'])){
                        $checkPerformer=$checkPerformer->where('external_id',$performer['id']);
                    }
                                               
                    $checkPerformer=$checkPerformer->where('source_id',$sources->id)->first();
                    if(!$checkPerformer){
                        $performerArray=[
                                            'name'=>$performer['name'],
                                            'external_id'=>$performer['id'],
                                            'source_id'=>$sources->id,
                        ];
                        $performer = Performers::create($performerArray);
                        if($count==1){
                            $performer_1_id = $performer->id;
                            $performers[] = $performer->id;
                        }
                        else{
                            $performer_2_id= $performer->id;
                            $performers[] = $performer->id;
                        }
                    }else{
                        if(isset($performer['id']) && !empty($performer['id'])){
                                if($checkPerformer->external_id != $performer['id']){
                                    $checkPerformer->external_id = $performer['id'];
                                    $checkPerformer->save();
                                }
                            }
                        if($count==1){
                            $performer_1_id=$checkPerformer->id;
                            $performers[] = $checkPerformer->id;
                        }else{
                            $performer_2_id = $checkPerformer->id;
                            $performers[] = $checkPerformer->id;
                        }
                    }
                    $count++;
                }
                
                //check for mapping of category in tree
                
                $event['category']['name'] = isset($event['category']['name']) && !empty($event['category']['name']) ? $event['category']['name'] : "No Category";
                if (false !== $key = array_search($event['category']['name'], $this->categoriesArray)) {
                    //add category source mapping 
                    $mappingArray['our_category_id'] = $key;
                    $mappingArray['source_category_name']=$event['category']['name'];
                    $mappingArray['source_id']= 1;
                    $check = SourceCategoryMapping::where($mappingArray)->first();
                    if(!$check){
                        SourceCategoryMapping::create($mappingArray);
                    }
                    $primaryCategory = $event['category']['name'];
                }else{
                    //check in category table
                    $check = SourceCategoryMapping::with('category')->where('source_category_name',$event['category']['name'])->where('source_id',1)->first();
                    if($check){
                        $primaryCategory = $check->category->name;
                    }else{
                        $primaryCategory = $event['category']['name'];
                    }
                }
                
                //check for maps
                $map_id="";
                $categoryTree = [];
                $categoryObj = new Categories();
                $categoryTree = $categoryObj->getParentCatgoryTree($primaryCategory);
                
                //get maps of venue
                
                $maps = VenueMaps::with('maps')->where('venue_id',$venueId)->get();
                if($maps->count() > 0){
                    if(!empty($categoryTree)){
                        foreach($maps as $map){
                            $mapCategory = explode(',',$map->maps->categories);
                            $commonCategory = array_intersect($categoryTree,$mapCategory);
                            if(!empty($commonCategory)){
                                $map_id = $map->map_id;
                            }
                        }
                    }else{
                        $map_id = $maps[0]->map_id;
                    }
                }else{
                    $map_id = null;
                }
                
                
                
                //check events 
                $event_name = isset($event['name']) ? $event['name'] : '';
                $event_date = isset($event['datetime']) ? date ('Y-m-d H:i:s',strtotime($event['datetime'])) : '';
                $event_date_utc = isset($event['datetime']) ? $event['datetime'] : '';

    
                $eventArray['name'] = str_replace("'", "",$event_name);
                $eventArray['search_type'] = $primaryCategory;
                // $eventArray['performer_1_id'] = $performer_1_id;
                // $eventArray['performer_2_id'] = $performer_2_id;
                $eventArray['event_date'] = $event_date;
                $eventArray['event_date_utc'] = $event_date_utc;
                $eventArray['venue_id'] = $venueId;
                $eventArray['source_id']=$sources->id;
                $eventArray['mapped_status']=true;
                $eventArray['status'] = $event['status'];
                $eventArray['use_tixstock_venue_details'] = isset($event['use_tixstock_venue_details']) && !empty($event['use_tixstock_venue_details']) && $event['use_tixstock_venue_details'] == "true" ? 1 : 0 ;
                $eventArray['currency_code']= isset($event['currency']) ? $event['currency'] : '';
                $eventArray['venue_address_id'] = $venue_address_id;
                $eventArray['external_id'] = $event['id'];
                $eventArray['map_id'] = $map_id;
                $event_date = date('Y-m-d',strtotime($event['datetime']));
                //check external id exist or not
                $checkEvents=Events::where($eventArray)->first();
                $checkEventExternalId = Events::where('external_id',$event['id'])->first();
                if(!$checkEventExternalId){
                    $events = Events::create($eventArray);
                    $eventId = $events->id;
                    if(!empty($performers)){
                        foreach($performers as $performer){
                            EventPerformers::create(['event_id'=>$eventId,'performer_id'=>$performer]);
                        }
                    }
                }else{
                        $events = Events::where('id',$checkEventExternalId->id)->update($eventArray);
                        $eventId = $checkEventExternalId->id;
                        EventPerformers::where('event_id',$eventId)->delete();
                        if(!empty($performers)){
                            foreach($performers as $performer){
                                EventPerformers::create(['event_id'=>$eventId,'performer_id'=>$performer]);
                            }
                        }
                }
                
               /* if(!$checkEvents){
                    $eventArray['external_id'] = isset($event['id']) && !empty($event['id']) ? $event['id'] : '';
                    $eventArray['mapped_status']=true;
                    $eventArray['status'] = $event['status'];
                    $eventArray['use_tixstock_venue_details'] = isset($event['use_tixstock_venue_details']) && !empty($event['use_tixstock_venue_details']) && $event['use_tixstock_venue_details'] == "true" ? 1 : 0 ;
                    $eventArray['search_type'] = $primaryCategory; //isset($event['category']['name']) && !empty($event['category']['name']) ? $event['category']['name'] : '';

                    $newevent = Events::create($eventArray);
                    $eventId = $newevent->id;
                    
                }else{
                    
                    $eventId = $checkEvents->id;
                    $checkEvents->external_id = isset($event['id']) && !empty($event['id']) ? $event['id'] : '';
                    $checkEvents->mapped_status=true;
                    $checkEvents->status = $event['status'];
                    $checkEvents->use_tixstock_venue_details = isset($event['use_tixstock_venue_details']) && !empty($event['use_tixstock_venue_details']) && $event['use_tixstock_venue_details'] == "true" ? 1 : 0 ;
                    $checkEvents->search_type = isset($event['category']['name']) && !empty($event['category']['name']) ? $event['category']['name'] : '';
                    $checkEvents->save();
                }
                
                */
                
                //check event category
                $eventCategory="";
                $eventSection="";
                if(!empty($categoryArray)){
                    $eventCategory = implode(',',$categoryArray);
                }
                if(!empty($sectionArray)){
                    $eventSection = implode(',',$sectionArray);
                }
                if($event['category'] != null){
                    $checkEventCategory = EventCategory::where('event_id',$eventId)->where('source_id',$sources->id)->first();
                    $eventCategoryArray=[
                                             'primary_category'=>$primaryCategory,
                                             'event_id'=>$eventId,
                                             'event_categories'=>$eventCategory,
                                             'event_section'=>$eventSection,
                                             'venue_id'=>$venueId,
                                             'source_id'=>$sources->id
                                        ];
                    if(!$checkEventCategory){
                        EventCategory::create($eventCategoryArray);
                        $response= [];
                        $response['msg']="Event Created Successfully.";
                        $response['success']=true;
                        return response()->json($response);
                    }else{
                        EventCategory::where('id',$checkEventCategory->id)->update($eventCatArray);
                    }
                }
                
                $response= [];
                $response['msg']="Event Processed Successfully.";
                $response['success']=true;
                return response()->json($response);
        }
        else if($resource->name=="TicketMaster"){
           
            $ticketmasterApi = new TicketmasterApi();
            $ticketmasterApi->manual_event =1;
            $response = $ticketmasterApi->processEvent($event);
            if($response['success']==true){
                $response['msg']="Event Processed Successfully.";
                $response['success']=true;
                return response()->json($response);
            }
        }
        else if($resource->name == "Ticket Evolution"){
            $tivoApi = new TivoApi();
            $tivoApi->manual_event = 1;
            $response = $tivoApi->processEvent($event);
            if($response['success']==true){
                $response['msg']="Event Processed Successfully.";
                $response['success']=true;
                return response()->json($response);
            }
        }
    }
    
}
?>