@extends('admin.layout.app')

@section('main_title', 'Maps')
@section('title', 'Maps')
@section('sub_title', 'Control Panel')

@push('styles')
<link href="{{ asset('public/assets/libs/datatables.net-bs5/css/dataTables.bootstrap5.min.css') }}" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/datatables.net-bs5/css/editor.dataTables.min.css') }}" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/datatables.net-responsive-bs5/css/responsive.bootstrap5.min.css') }}" rel="stylesheet" type="text/css" />
<link href="https://cdn.datatables.net/datetime/1.1.0/css/dataTables.dateTime.min.css" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/x-editable/bootstrap-editable/css/bootstrap-editable.css') }}" rel="stylesheet" type="text/css" />
<style>
    table.dataTable tbody>tr.selected, table.dataTable tbody>tr>.selected {
    background-color: #FFFFFF !important;
}
</style>
@endpush

@section('content')
<div class="row">
    <div class="col-12">
         <div class="card filter_box">
            <form id="filter">
                <div class="card-body">
                   
                    <div class="row mb-2">
                        <div class="col-sm-4">
                            <div class="input-group input-group-md">
                                <input type="text" name="map_name" class="form-control" id="map_name" placeholder="Map Name">
                            </div>
                        </div>
                        <div class="col-sm-4">
                            <div class="text-sm-end mt-3 mt-md-0 form-group">
                                <select class="form-control" id="map_status">
                                    <option value="">Select Map Status</option>
                                    <option value="1">Grouped</option>
                                    <option value="0"> UnGrouped</option>
                                </select>
                            </div>
                        </div>
                        <div class="col-sm-4" style="text-align:center">
                             
                            <button type="button" class="btn btn-primary filter waves-effect waves-light mb-2 me-4 fw-bold" id="search">Search
                            <div class="search_loader spinner-border spinner-border-sm m-0" style="display:none" role="status"></div>
                            </button>
                             <button type="reset" class="btn btn-primary mb-2 me-4 fw-bold">Reset
                            </button>
                        </div> 
                    </div>
            
                    <div class="row mb-2">
                        <div class="progress mb-0" style="height: 2px;">
                            <div class="progress-bar" role="progressbar" style="width: 20%" aria-valuenow="20" aria-valuemin="0" aria-valuemax="100"></div>
                            <div class="progress-bar bg-danger" role="progressbar" style="width: 20%" aria-valuenow="20" aria-valuemin="0" aria-valuemax="100"></div>
                            <div class="progress-bar bg-warning" role="progressbar" style="width: 20%" aria-valuenow="20" aria-valuemin="0" aria-valuemax="100"></div>
                            <div class="progress-bar bg-success" role="progressbar" style="width: 20%" aria-valuenow="20" aria-valuemin="0" aria-valuemax="100"></div>
                            <div class="progress-bar bg-info" role="progressbar" style="width: 20%" aria-valuenow="20" aria-valuemin="0" aria-valuemax="100"></div>
                        </div>
                    </div>
                </div> <!-- end card-body-->
            </form>
        </div> <!-- end card-->
        <div class="card">
            <div class="card-body">
                
                <div class="row mb-2">
                    <div class="col-sm-4">
                        <a href="{{route('maps.create')}}" class="btn btn-primary mb-2"><i class="mdi mdi-plus-circle me-2"></i>Create New Map</a>
                    </div>
                </div>
                
                <div class="table-responsive">
                    <table id="maps_table" class="table m-0 table-centered table-striped dt-responsive w-100">
                        <thead>
                            <tr>
                                <th></th>
                                <th>Name</th>
                                <th>Categories</th>
                                <th>Preview Url</th>
                                <th>Source</th>
                                <th>Status</th>
                                <th>Action</th>
                                
                            </tr>
                        </thead>
                        <tbody></tbody>
                    </table>
                </div>

                
            </div> <!-- end card-body-->
        </div> <!-- end card-->
    </div> <!-- end col -->
@endsection

@push('scripts')

<script src="https://cdnjs.cloudflare.com/ajax/libs/jquery/3.7.1/jquery.min.js" integrity="sha512-v2CJ7UaYy4JwqLDIrZUI/4hqeoQieOmAZNXBeQyjo21dadnwR+8ZaIJVT8EE2iyI61OV8e6M8PP2/4hpQINQ/g==" crossorigin="anonymous" referrerpolicy="no-referrer"></script>
<script src="{{ asset('public/assets/libs/datatables.net/js/jquery.dataTables.min.js') }}"></script>
<script src="{{ asset('public/assets/libs/datatables.net/js/dataTables.editor.min.js') }}"></script>
<script src="{{ asset('public/assets/libs/datatables.net-bs5/js/dataTables.bootstrap5.min.js') }}"></script>
<script src="{{ asset('public/assets/libs/datatables.net-responsive/js/dataTables.responsive.min.js') }}"></script>
<script src="{{ asset('public/assets/libs/datatables.net-responsive-bs5/js/responsive.bootstrap5.min.js') }}"></script>
<script src="https://cdn.datatables.net/datetime/1.1.0/js/dataTables.dateTime.min.js"></script>
<script src="{{ asset('public/assets/libs/x-editable/bootstrap-editable/js/bootstrap-editable.min.js') }}"></script>

<script>
    $(document).ready(function(){
      
        $.ajaxSetup({
                headers: {
                    'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
                }
        });
    var datatable = $('#maps_table').DataTable({
      
      'processing': true,
      'serverSide': true,
      'serverMethod': 'post',
      'ajax': {
           'url':'{{route("map.data")}}',
           'data': function(d) {
           
               d.name = $("#map_name").val();
               d.status = $("#map_status").val();
           }
      },
    drawCallback: function() {
        $(".dataTables_paginate > .pagination").addClass("pagination-rounded");
        
    },
    "columnDefs": [
            {
                "targets": [ 0 ],
                "visible": false,
                "searchable": false
            }
        ],
      'columns': [
         { data: 'id' },
         { data: 'name' },
         { data: 'categories' },
         { data: 'preview_url' },
         { data: 'source'},
         {data: 'status'},
         { data: 'action' },
      ],
     select: {
            style:    'os',
            selector: 'td:first-child'
        },
   });
   
       $(document).on('click','#search',function(){
           datatable.draw();
       })
    });
    
    $(document).on('click','.delete',function(){
        var _token = $('input[name="_token"]').val();
        $id=$(this).data('id');
        $action=$(this).data('src');
        Swal.fire({
        title: "Are you sure?",
        text: "Map will be Deleted!",
        type: "warning",
        showCancelButton: true,
        confirmButtonClass: "btn-danger",
        confirmButtonText: "Yes, Delete Map!",
        closeOnConfirm: false
        }).then((result)=>{
            if(result['isConfirmed']){
                $.ajax({
                url: $action,
                type: "DELETE",
                data:{
                "_token": "{{ csrf_token() }}",
                },
                success: function (){
                Swal.fire("Deleted!", "Map has been deleted.", "success");
                $('#maps_table').DataTable().ajax.reload();
                }
            });
            }
        })
    });
    
</script>

@endpush