@extends('admin.layout.app')

@section('main_title', 'Users')
@section('title', 'User')
@section('sub_title', 'Control Panel')
@push('styles')
<link href="{{ asset('public/assets/libs/sweetalert2/sweetalert2.min.css') }}" rel="stylesheet" type="text/css" />
<!-- datatables-->
<link href="{{ asset('public/assets/libs/datatables.net-bs5/css/dataTables.bootstrap5.min.css') }}" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/datatables.net-bs5/css/editor.dataTables.min.css') }}" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/datatables.net-responsive-bs5/css/responsive.bootstrap5.min.css') }}" rel="stylesheet" type="text/css" />
<link href="https://cdn.datatables.net/datetime/1.1.0/css/dataTables.dateTime.min.css" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/x-editable/bootstrap-editable/css/bootstrap-editable.css') }}" rel="stylesheet" type="text/css" />
<!-- Loading button css -->
<link href="{{ asset('public/assets/libs/ladda/ladda.min.css') }}" rel="stylesheet" type="text/css" />
<link href="{{ asset('public/assets/libs/ladda/ladda-themeless.min.css') }}" rel="stylesheet" type="text/css" />

@endpush
<!-- start content -->
@section('content')
<div class="row">
    <div class="col-12">
        <div class="card">   
            <div class="card-body">
                <div class="row mb-2">
                    <div class="col-sm-4">
                        <a href="{{ route('user.create')}}" class="btn btn-danger mb-2" ><i class="fa fa-plus"></i> Create New</a>
                    </div>
                </div>
                <div class="table-responsive">
                    <table id="user_table" class="table m-0 table-centered table-striped dt-responsive w-100">
                        <thead>
                            <tr>
                                <th></th>
                                <th>Name</th>
                                <th>Email</th>
                                <th>Role</th>
                                <th>Status</th>
                                <th>Action</th>
                            </tr>
                        </thead>
                        <tbody></tbody>
                    </table>
                </div>
            </div> <!-- end card-body-->
        </div> <!-- end card-->
    </div> <!-- end col -->
</div>
@endsection

@push('scripts')
        
        <!-- datatables js -->
        <script src="{{ asset('public/assets/libs/datatables.net/js/jquery.dataTables.min.js') }}"></script>
        <script src="{{ asset('public/assets/libs/datatables.net/js/dataTables.editor.min.js') }}"></script>
        <script src="{{ asset('public/assets/libs/datatables.net-bs5/js/dataTables.bootstrap5.min.js') }}"></script>
        <script src="{{ asset('public/assets/libs/datatables.net-responsive/js/dataTables.responsive.min.js') }}"></script>
        <script src="{{ asset('public/assets/libs/datatables.net-responsive-bs5/js/responsive.bootstrap5.min.js') }}"></script>
        <script src="https://cdn.datatables.net/datetime/1.1.0/js/dataTables.dateTime.min.js"></script>
        <script src="{{ asset('public/assets/libs/x-editable/bootstrap-editable/js/bootstrap-editable.min.js') }}"></script>
        <!-- Sweet Alerts js -->
        <script src="{{ asset('public/assets/libs/sweetalert2/sweetalert2.all.min.js') }}"></script>
        <!-- tool tip js -->
        <script src="{{ asset('public/assets/libs/tippy.js/tippy.all.min.js') }}"></script>
        <!-- Loading buttons js -->
        <script src="{{ asset('public/assets/libs/ladda/spin.min.js') }}"></script>
        <script src="{{ asset('public/assets/libs/ladda/ladda.min.js') }}"></script>
        <script>
            
            
            $(document).ready(function(){
                
                var user_datatable = $('#user_table').DataTable({
                  'processing': true,
                  'serverSide': true,
                  'serverMethod': 'get',
                  'ajax': {
                       'url':"{{route('users.data')}}",
                  },
                drawCallback: function() {
                    $(".dataTables_paginate > .pagination").addClass("pagination-rounded");
                    tippy('[data-plugin="tippy"]');
                },
                "columnDefs": [
                        {
                            "targets": [ 0 ],
                            "visible": false,
                            "searchable": false
                        }
                    ],
                  'columns': [
                     { data: 'id' },
                     { data: 'name' },
                     { data: 'email' },
                     { data: 'role' },
                     { data:  'status'},
                     { data: 'actions',name: 'actions', orderable: false, searchable: false },
                    //  { data: 'edit' },
                  ],
                 select: {
                        style:    'os',
                        selector: 'td:first-child'
                    },
               });
            });
            $(document).on('submit','form',function(e){
                e.preventDefault();
                var me = $(this);
                var fd=new FormData($(this)[0]);
                
                $.ajax({
                    url: me.attr('action'),
                    type: me.attr('method'),
                    data: fd,
                    headers: {'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')},
                    dataType: 'json',
                    contentType: false,
                    processData: false,
                    success: function(response){
                        Swal.fire("Done",response.msg,"success");
                        $('.modal').modal('hide');
                        $("#roles_table").DataTable().ajax.reload();
                    },
                    error:function(response){
                        $.each(response.responseJSON.errors, function(key, value) {
                        var element = $('#' + key);
                        element.closest('div.form-group')
                        .removeClass('.has-error')
                        .addClass(value.length > 0 ? 'has-error':'has-success' )
                        .find('.text-danger')
                        .remove();
                        element.after('<p class="text text-danger">'+value+'</p>');
                        });
                    }
                });
            })
            $(document).on('click','.delete',function(){
                var _token = $('input[name="_token"]').val();
                $id=$(this).attr('id');
                $action=$(this).data('src');
                Swal.fire({
                title: "Are you sure?",
                text: "User will be Deleted!",
                type: "warning",
                showCancelButton: true,
                confirmButtonClass: "btn-danger",
                confirmButtonText: "Yes, Delete User!",
                closeOnConfirm: false
                }).then((result)=>{
                    if(result['isConfirmed']){
                        $.ajax({
                        url: $action,
                        type: "DELETE",
                        data:{
                        "_token": "{{ csrf_token() }}",
                        },
                        success: function (){
                        Swal.fire("Deleted!", "User has been deleted.", "success");
                        $('#user_table').DataTable().ajax.reload();
                        }
                    });
                    }
                })
            });
        </script>
@endpush