<?php

use Illuminate\Support\Facades\Route;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

Route::get('/', function () {
    return redirect(route('login'));
});

Auth::routes();

Route::group(['middleware' => ['auth','checkrole']], function(){
//homep page route
Route::get('/dashboard', 'HomeController@index')->name('home');

//roles
Route::resource('roles','RolesController');
Route::get('fetch/roles/data','RolesController@data')->name('role.data');
Route::post('role/update/{id}','RolesController@updatedata')->name('role.update.data');

//user routes
Route::resource('user','UserController');
Route::get('fetch/user/data','UserController@data')->name('users.data');

//events routes
// Route::resource('events','EventsController');
Route::resource('event','EventsController');
Route::get('events/{domain?}/{type?}','EventsController@index')->name('events.index');
Route::post('fetch/event/data','EventsController@data')->name('event.data');
Route::post('fetch/event/venue/data','EventsController@fetchEventVenueDetails')->name('fetch.event.venue.details');
Route::post('events/data/update','EventsController@eventsDataUpdate')->name('events.data.update');
Route::post('events/update/category/section','EventsController@eventsUpdateCategorySection')->name('events.update.category.section');
Route::post('events/venue/category/update','EventsController@eventsVenueCategoryUpdate')->name('events.venue.category.update');
Route::post('events/performers/update','EventsController@eventsPerformersUpdate')->name('events.performers.update');
Route::post('events/performers/delete','EventsController@eventsPerformersDelete')->name('events.performers.delete');
Route::post('events/tixstock/log','EventsController@getTixstockLog')->name('fetch.event.tixstock.log');
Route::post('events/get/versions','EventsController@getEventVersion')->name('get.event.versions');
Route::post('events/get/source/versions','EventsController@getSourceEventVersion')->name('get.event.source.versions');
Route::post('events/set/default','EventsController@setEventDefault')->name('events.make.default');
Route::post('events/set/primary','EventsController@setEventPrimary')->name('events.make.primary');
Route::post('events/update_category','EventsController@updateEventCategory')->name('update.event.category');
Route::post('events/assign/map','EventsController@assignMap')->name('assign.map.to.event');
Route::post('events/remove/map','EventsController@removeMap')->name('remove.map.from.event');
Route::post('events/assign/bulk/map','EventsController@bulkUpdateEventsMap')->name('update.events.map');
Route::post('events/update/bulk/performers','EventsController@bulkUpdateEventPerformer')->name('update.event.performers');

Route::get('events/fetch/single/event','EventsController@fetchSingleEvent')->name('fetch.SingleEvent.fromSource');
Route::get('events/tixstock/log','EventsController@tixstockLog')->name('events.tixstock.log');
Route::post('fetch/single/event','EventsController@fetchSingleEventSubmit')->name('fetch.single.event');
Route::post('fetch/single/event/submit','EventsController@fetchedSingleEventSubmit')->name('fetch.single.event.submit');
Route::post('events/bulk/update','EventsController@bulkEventsUpdate')->name('bulk.event.update');
Route::post('events/bulk/delete/','EventsController@deleteBulkEvents')->name('delete.bulk.events');
Route::get('fetch/events/manually','EventsController@importEvents')->name('import.events');
Route::post('events/export','EventsController@exportEvents')->name('export.events');
Route::post('fetch/events/manually/submit','EventsController@importEventsSubmit')->name('import.events.submit');
Route::post('events/bulk/update/category/section','EventsController@bulkUpdateEventsCatSection')->name('bulk.event.update.cat.section');
Route::post('event/submit/bulk/update/event/category/section','EventsController@submitBulkUpdateEventsCatSection')->name('submit.bulk.event.cat.section');
Route::post('events/bulk/version','EventsController@bulkEventsVersioning')->name('bulk.event.version');
//venue routes
Route::resource('venue','VenueController');
Route::get('venues/listing','VenueController@data')->name('venue.data');
Route::post('venues/category/section/data','VenueController@getVenueCategorySectionData')->name('get.venue.category.section');
Route::post('venue/category/delete','VenueController@deletecategory')->name('venue.category.delete');
Route::post('venue/category/delete/section','VenueController@deleteSection')->name('venue.category.delete.section');
Route::post('venue/address/detail','VenueController@getAddressDetails')->name('get.venue.address.details');
Route::post('venue/add/new/address','VenueController@addNewAddress')->name('venue.add.new.address');
Route::post('venue/get/address/details','VenueController@getVenueAddressDetails')->name('venue.get.address.details');
Route::post('venue/single/category/section','VenueController@getSingleCategorySection')->name('get.single.category.section');
Route::post('venue/store/single/category/section','VenueController@storeSingleCategorySection')->name('store.single.category.section');
Route::post('venue/edit/single/category/section','VenueController@editSingleCategorySection')->name('edit.single.category.section');
Route::post('venue/store/tixstock/category/section','VenueController@storeTixstockCategory')->name('store.tixstock.category.section');
Route::post('venue/delete/address','VenueController@deleteVenueAddress')->name('venue.delete.address');
Route::post('venue/maps/listing','VenueController@getMapListing')->name('get.venue.maps.listing');
Route::post('venue/assign/map','VenueController@assignMapToVenue')->name('venue.assign.maps');
Route::get('venue/create/map/{id}','VenueController@createMap')->name('venue.create.map');
Route::get('venue/edit/map/{id}','VenueController@editMap')->name('venue.edit.map');
Route::post('venue/delete/map','VenueController@deleteMap')->name('venue.delete.map');
Route::post('venue/bulk/category/delete','VenueController@deleteBulkCategorySection')->name('venue.bulk.category.delete');
//performers routes
Route::resource('performer','PerformerController');
Route::post('fetch/performers','PerformerController@getPerformers')->name('fetch.performer');
Route::post('fetch/perfomer/data','PerformerController@data')->name('performer.data');

//categories routes
// Route::resource('category','CategoryController');
/* Route::post('categories/data/update','CategoryController@categoriesDataUpdate')->name('categories.data.update'); */
//////////////////
Route::get('category/add','CategoryController@add')->name('category.add');
Route::get('category/view','CategoryController@index')->name('category.index');
Route::post('category/store','CategoryController@store')->name('category.store');
Route::get('category/get_sub_category/{id}/{count}','CategoryController@get_sub_category')->name('category.getsubcategory');
Route::get('category/get_sub_subcategory/{id}/{count}','CategoryController@get_sub_subcategory')->name('category.getsubsubcategory');
Route::post('category/update_order','CategoryController@updateorder')->name('category.updateorder');

Route::post('category/delete/{id}','CategoryController@delete')->name('category.delete');
Route::post('category/update','CategoryController@update')->name('category.update');
Route::get('category/get_selected_category/{id}','CategoryController@get_seleted_category')->name('get.categorydetail');
Route::get('category/source/mapping','CategoryController@categorySourceMapping')->name('category.source.mapping');
Route::post('category/add/source/mapping','CategoryController@addCategorySourceMapping')->name('add.category.source.mapping');
Route::get('category/source/mapping/data','CategoryController@categorySourceMappingData')->name('category.source.mapping.data');
Route::post('category/source/mapping/delete','CategoryController@deleteMapping')->name('category.source.mapping.delete');
Route::get('category/tivo/index','CategoryController@tivoCategory')->name('category.tivo.index');
Route::post('category/tivo/update_api_flag','CategoryController@tivoCategoryUpdateFlag')->name('category.tivo.update.flag');
//resource routes 
Route::resource('sources','SourcesController');
Route::resource('logs','LogsController');

Route::get('event/tixstock/log','LogsController@tixstockLog')->name('logs.tixstock.index');
Route::get('fetch/cron/log/data','LogsController@Crondata')->name('cronlog.data');
Route::get('event/failed/jobs','LogsController@FailedJobs')->name('failedjobs.index');
Route::post('event/failed/jobs/data','LogsController@FailedJobsData')->name('failedjobs.data');
Route::get('fetch/tixstock/log/data','LogsController@tixstockLogData')->name('tixstocklog.data');
Route::get('fetch/sources/data','SourcesController@data')->name('sources.data');

//map routes
Route::resource('maps','MapsController');
Route::get('map/tivo/edit/{id}','MapsController@tivo_edit')->name('map.tivo.edit');
Route::post('fetch/map/data','MapsController@data')->name('map.data');
Route::post('fetch/map/section','MapsController@getSection')->name('get.maps.section');

});